@@ -106,6 +106,9 @@ namespace OpenBabel
       void SetParent(OBMol *ptr)  {        _parent= ptr;        }
       //! Change the bond length to @p length, while keeping @p fixed stationary
       void SetLength(OBAtom *fixed,double length);
+      //! Change the bond length to @p length, moving both atoms halfway
+      //! \since version 2.2
+      void SetLength(double length);
       //! Set the main bond information (i.e., when creating a bond)
       void Set(int index, OBAtom* begin,OBAtom* end,int order,int flags);
       //! \deprecated Use SetBondOrder() instead
@@ -160,9 +163,11 @@ namespace OpenBabel
       //! \return The set of property flags defined for this bond.
       unsigned int     GetFlags()         const { return(_flags);      }
       //! \return The atom index for the end atom in this bond (from OBAtom::GetIdx()
-      unsigned int     GetBeginAtomIdx()  const { return(_bgn->GetIdx()); }
+      unsigned int     GetBeginAtomIdx()  const 
+        { return (_bgn ? _bgn->GetIdx() : 0); }
       //! \return The atom index for the end atom in this bond (from OBAtom::GetIdx()
-      unsigned int     GetEndAtomIdx()    const { return(_end->GetIdx()); }
+      unsigned int     GetEndAtomIdx()    const 
+        { return (_end ? _end->GetIdx() : 0); }
       //! \return The "beginning" atom for this bond
       OBAtom *GetBeginAtom()    { return(_bgn);    }
       const OBAtom *GetBeginAtom() const 
@@ -182,15 +187,18 @@ namespace OpenBabel
       OBMol  *GetParent()                 {return(_parent);}
       //! \return The expected "equilibrium" length based on the covalent radii and bond order
       /** Length is given in Angstroms **/
-      double  GetEquibLength();
+      double  GetEquibLength() const;
       //! \return The current length of this bond in Angstroms
-      double  GetLength();
+      double  GetLength() const;
       //! \return The index to the neighboring atom of @p ptr (i.e., the end if @p ptr is the start)
       /** \warning If @p ptr is not part of the bond, the beginning atom
           index will always be returned **/
       unsigned int     GetNbrAtomIdx(OBAtom *ptr)
         {
-          return((ptr!=_bgn)?_bgn->GetIdx():_end->GetIdx());
+          if (ptr!=_bgn)
+            return (_bgn ? _bgn->GetIdx() : 0); 
+          else
+            return (_end ? _end->GetIdx() : 0); 
         }
       //@}
 
