@@ -27,6 +27,7 @@ GNU General Public License for more details.
 #include <fstream>
 #include <vector>
 #include <string>
+#include <cstring>
 
 namespace OpenBabel
 {
@@ -81,9 +82,9 @@ namespace OpenBabel
   class OBAPI OBElement
     {
       int _num;
-      char _symbol[3];
+      char _symbol[4];
       std::string _name;
-      double _Rcov,_Rvdw,_mass,_elNeg,_ionize,_elAffinity;
+      double _Rcov,_Rvdw,_mass,_elNeg,_ARENeg,_ionize,_elAffinity;
       double _red, _green, _blue;
       int _maxbonds;
     public:
@@ -92,6 +93,7 @@ namespace OpenBabel
       /** Constructor
           @param num     Atomic number
           @param sym     Elemental symbol (maximum 3 characters)
+          @param ARENeg  Allred-Rochow electronegativity
           @param rcov    Covalent radius (in Angstrom)
           @param rvdw    van der Waals radius (in Angstrom)
           @param maxbo   Maximum bonding valence
@@ -104,22 +106,22 @@ namespace OpenBabel
           @param blue    RGB value for a suggest visualization color (0 .. 1)
           @param name Full IUPAC name
       **/
-      OBElement(int num, const char *sym, double rcov, double rvdw,
-                int maxbo, double mass, double elNeg, double ionize,
+      OBElement(int num, const char *sym, double ARENeg, double rcov, 
+      		double rvdw, int maxbo, double mass, double elNeg, double ionize,
                 double elAffin, double red, double green, double blue,
                 std::string name) :
         _num(num), _name(name), _Rcov(rcov), _Rvdw(rvdw), _mass(mass), 
-        _elNeg(elNeg), _ionize(ionize), _elAffinity(elAffin), 
+        _elNeg(elNeg), _ARENeg(ARENeg), _ionize(ionize), _elAffinity(elAffin), 
         _red(red), _green(green), _blue(blue),
         _maxbonds(maxbo)
         {
-          strncpy(_symbol, sym, 3);
+          strncpy(_symbol, sym, 4);
         }
 
       //! \return the atomic number of this element
       int GetAtomicNum()         {       return(_num);    }
       //! \return the atomic symbol for this element
-      const char *GetSymbol()    {       return(_symbol); }
+      char *GetSymbol()          {       return(_symbol); }
       //! \return the covalent radius of this element
       double GetCovalentRad()    {       return(_Rcov);   }
       //! \return the van der Waals radius of this element
@@ -130,6 +132,8 @@ namespace OpenBabel
       int GetMaxBonds()          {       return(_maxbonds);}
       //! \return the Pauling electronegativity for this element
       double GetElectroNeg()     {       return(_elNeg);  }
+      //! \return the Allred-Rochow electronegativity for this element
+      double GetAllredRochowElectroNeg() { return(_ARENeg); }
       //! \return the ionization potential (in eV) of this element
       double GetIonization()     {       return(_ionize);  }
       //! \return the electron affinity (in eV) of this element
@@ -186,6 +190,8 @@ namespace OpenBabel
       int	GetMaxBonds(int);
       //! \return the Pauling electronegativity for this element
       double GetElectroNeg(int);
+      //! \return the Allred-Rochow electronegativity for this element
+      double GetAllredRochowElectroNeg(int);
       //! \return the ionization potential (in eV) for this element
       double GetIonization(int);
       //! \return the electron affinity (in eV) for this element
@@ -297,19 +303,6 @@ namespace OpenBabel
       bool AssignBonds(OBMol &,OBBitVec &);
     };
 
-  //! Opens a datafile in a directory where OpenBabel expects to find it.
-  // full documentation in data.cpp
-  OBAPI std::string OpenDatafile(std::ifstream& fs, 
-                                 const std::string& filename,
-                                 const std::string& envvar = "BABEL_DATADIR");
-
-  // Used by other code for reading files
-#ifdef WIN32
-#define FILE_SEP_CHAR "\\"
-#else
-#define FILE_SEP_CHAR "/"
-#endif
-
 } // end namespace OpenBabel
 
 #endif //DATA_H
