@@ -105,11 +105,12 @@ namespace OpenBabel
     OBAtom              *_ptr;
     OBBitVec             _notVisited;
     std::queue<OBAtom *> _queue;
+    std::vector<int>     _depth;
   public:
 
     OBMolAtomBFSIter(): _parent(NULL), _ptr(NULL) { }
-    OBMolAtomBFSIter(OBMol *mol);
-    OBMolAtomBFSIter(OBMol &mol);
+    OBMolAtomBFSIter(OBMol *mol, int StartIndex = 1);
+    OBMolAtomBFSIter(OBMol &mol, int StartIndex = 1);
     OBMolAtomBFSIter(const OBMolAtomBFSIter &ai);
     ~OBMolAtomBFSIter() { }
 
@@ -124,6 +125,9 @@ namespace OpenBabel
     OBAtom* operator->() const   { return _ptr;      }
     //! \return a reference to the current atom
     OBAtom& operator*() const    { return *_ptr;     }
+    //! \return the current depth of the iterator
+    //! \since version 2.2
+    int CurrentDepth() const;
   };
 
   //! \brief Iterate over all bonds in an OBMol
@@ -306,9 +310,11 @@ namespace OpenBabel
   
   //! \brief Iterate over all pairs of atoms (>1-4) in an OBMol
   class OBAPI OBMolPairIter {
+    std::vector<OBAtom*>::iterator _i;
+    std::vector<OBAtom*>::iterator _j;
     OBMol *_parent;
-    std::vector<std::vector<unsigned int> > _vpair;
-    std::vector<std::vector<unsigned int> >::iterator _i;
+    //std::vector<std::vector<unsigned int> > _vpair;
+    //std::vector<std::vector<unsigned int> >::iterator _i;
     std::vector<unsigned int> _pair;
  
   public:
@@ -321,7 +327,7 @@ namespace OpenBabel
 
     OBMolPairIter& operator=(const OBMolPairIter &ai);
     //! \return Whether the iterator can still advance (i.e., visit more 1-4 atom pairs)
-    operator bool() const        { return (_i != _vpair.end()); }
+    operator bool() const        { return _pair.size()>0; }
     //! Preincrement -- advance to the next 1-4 atom pair and return
     OBMolPairIter& operator++();
     //! \return A vector of two atom indexes specifying the 1-4 atom pair
