@@ -2,17 +2,12 @@ public class OBTest
 {
   public OBTest()
   {
-    System.loadLibrary("openbabeljni");
+    System.loadLibrary("openbabel");
   }
 
   public void run()
   {
-    OBConversion conv = new OBConversion();
-    conv.SetInFormat("smi");
-
-    OBMol mol = new OBMol();
-    conv.ReadString(mol, "CCCCC");
-    System.out.println(mol.NumAtoms());
+    OBConversion c = new OBConversion();
   }
 
   public static void main(String[] args)
