@@ -2,7 +2,7 @@
 atom.cpp - Handle OBAtom class.
  
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
-Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
+Some portions Copyright (C) 2001-2008 by Geoffrey R. Hutchison
 Some portions Copyright (C) 2003 by Michael Banck
  
 This file is part of the Open Babel project.
@@ -69,7 +69,7 @@ namespace OpenBabel
       }
       \endcode
       A number of the property member functions indicate that atoms
-      have some knowlege of their covalently attached neighbor atoms.
+      have some knowledge of their covalently attached neighbor atoms.
       Bonding information is partly redundant within a molecule in
       that an OBMol has a complete list of bonds in a molecule, and
       an OBAtom has a list bonds of which it is a member. The following
@@ -148,22 +148,38 @@ namespace OpenBabel
   OBAtom &OBAtom::operator=(OBAtom &src)
   //copy atom information
   //bond info is not copied here as ptrs may be invalid
-  //vdata is also not copied yet (again it's unclear what can work)
   {
     _idx = src.GetIdx();
-    _hyb = src.GetHyb();
-    _ele = src.GetAtomicNum();
-    _isotope = src.GetIsotope();
-    _fcharge = src.GetFormalCharge();
-    _spinmultiplicity = src.GetSpinMultiplicity();
-    strncpy(_type,src.GetType(), sizeof(_type) - 1);
-    _type[sizeof(_type) - 1] = '\0';
-    _pcharge = src.GetPartialCharge();
-    _v = src.GetVector();
-    _flags = src.GetFlag();
-    _residue = (OBResidue*)NULL;
+    Duplicate(&src);
     return(*this);
   }
+  
+  void OBAtom::Duplicate(OBAtom *src)
+  {
+    if (!src)
+      return;
+      
+    _hyb = src->GetHyb();
+    _ele = src->GetAtomicNum();
+    _isotope = src->GetIsotope();
+    _fcharge = src->GetFormalCharge();
+    _spinmultiplicity = src->GetSpinMultiplicity();
+    strncpy(_type,src->GetType(), sizeof(_type) - 1);
+    _type[sizeof(_type) - 1] = '\0';
+    _pcharge = src->GetPartialCharge();
+    _v = src->GetVector();
+    _flags = src->GetFlag();
+    _residue = (OBResidue*)NULL;
+    
+    _vdata.clear();
+    //Copy all the OBGenericData, providing the new atom
+    vector<OBGenericData*>::iterator itr;
+    for(itr=src->BeginData();itr!=src->EndData();++itr)
+      {
+        OBGenericData* pCopiedData = (*itr)->Clone(this);
+        SetData(pCopiedData);
+      }
+  }
 
   bool OBAtom::IsConnected(OBAtom *a1)
   {
@@ -385,7 +401,7 @@ namespace OpenBabel
       _isotope = 2;
   }
 
-  void OBAtom::SetType(string &type)
+  void OBAtom::SetType(const string &type)
   {
     strncpy(_type,type.c_str(), sizeof(_type) - 1);
     _type[sizeof(_type) - 1] = '\0';
@@ -410,10 +426,15 @@ namespace OpenBabel
   }
 
   OBResidue *OBAtom::GetResidue()
+  {
+    return GetResidue(true); // default is to always perceive chains
+  }
+
+  OBResidue *OBAtom::GetResidue(bool perception)
   {
     if (_residue != NULL)
       return _residue;
-    else if (!((OBMol*)GetParent())->HasChainsPerceived())
+    else if (perception && !((OBMol*)GetParent())->HasChainsPerceived())
       {
         ((OBMol*)GetParent())->SetChainsPerceived();
         if ( chainsparser.PerceiveChains(*((OBMol*)GetParent())) )
@@ -448,8 +469,9 @@ namespace OpenBabel
   char *OBAtom::GetType()
   {
     OBMol *mol = (OBMol*)GetParent();
-    if (mol && !mol->HasAtomTypesPerceived())
-      atomtyper.AssignTypes(*((OBMol*)GetParent()));
+    if (mol)
+      if (!mol->HasAtomTypesPerceived())
+        atomtyper.AssignTypes(*((OBMol*)GetParent()));
 
     if (strlen(_type) == 0) // Somehow we still don't have a type!
       {
@@ -1064,6 +1086,10 @@ namespace OpenBabel
 
     v1 = this->GetVector() - b->GetVector();
     v2 = c->GetVector() - b->GetVector();
+    if (IsNearZero(v1.length(), 1.0e-3)
+      || IsNearZero(v2.length(), 1.0e-3)) {
+        return(0.0);
+    }
 
     return(vectorAngle(v1, v2));
   }
@@ -1076,6 +1102,11 @@ namespace OpenBabel
     v1 = this->GetVector() - mol->GetAtom(b)->GetVector();
     v2 = mol->GetAtom(c)->GetVector() - mol->GetAtom(b)->GetVector();
 
+    if (IsNearZero(v1.length(), 1.0e-3)
+      || IsNearZero(v2.length(), 1.0e-3)) {
+        return(0.0);
+    }
+
     return(vectorAngle(v1, v2));
   }
 
@@ -1092,6 +1123,9 @@ namespace OpenBabel
         v = VX;
         v *= length;
         v += GetVector();
+        // Check that the vector is still finite before returning
+        if (!isfinite(v.x()) || !isfinite(v.y()) || !isfinite(v.z()))
+          v.Set(0.0, 0.0, 0.0);
         return(true);
       }
 
@@ -1146,6 +1180,9 @@ namespace OpenBabel
 
         v *= length;
         v += GetVector();
+        // Check that the vector is still finite before returning
+        if (!isfinite(v.x()) || !isfinite(v.y()) || !isfinite(v.z()))
+          v.Set(0.0, 0.0, 0.0);
         return(true);
       }
 
@@ -1188,6 +1225,9 @@ namespace OpenBabel
         v *= length;
 
         v += GetVector();
+        // Check that the vector is still finite before returning
+        if (!isfinite(v.x()) || !isfinite(v.y()) || !isfinite(v.z()))
+          v.Set(0.0, 0.0, 0.0);
         return(true);
       }
 
@@ -1199,6 +1239,12 @@ namespace OpenBabel
         for (atom = BeginNbrAtom(i);atom;atom = NextNbrAtom(i))
           {
             vtmp = GetVector() - atom->GetVector();
+            if (vtmp.length_2() < 0.1) { // atoms are nearly on top of each other
+              // Move the other atom a small random distance apart
+              vtmp.randomUnitVector();
+              vtmp *= length;
+              atom->SetVector(vtmp + GetVector());
+            } // Now this should be a real distance, not a zero vector
             vtmp.normalize();
             vtmp /= 3.0;
             vsum += vtmp;
@@ -1207,6 +1253,9 @@ namespace OpenBabel
         v = vsum;
         v *= length;
         v += GetVector();
+        // Check that the vector is still finite before returning
+        if (!isfinite(v.x()) || !isfinite(v.y()) || !isfinite(v.z()))
+          v.Set(0.0, 0.0, 0.0);
         return(true);
       }
 
@@ -1358,29 +1407,34 @@ namespace OpenBabel
 
         //find ring atoms first
         for (nbr = BeginNbrAtom(i);nbr;nbr = NextNbrAtom(i))
-          if ((*i)->IsInRing())
-           {
-             if (!r1)
-                r1 = nbr;
-              else if (!r2)
-                r2 = nbr;
-              else if (!r3)
-                r3 = nbr;
-           }
+          if ((*i)->IsInRing()) {
+            if (!r1) {
+              r1 = nbr;
+            }
+            else if (!r2) {
+              r2 = nbr;
+            }
+            else if (!r3) {
+              r3 = nbr;
+            }
+          }
 
         //find non-ring atoms
         for (nbr = BeginNbrAtom(i);nbr;nbr = NextNbrAtom(i))
-          if (!(*i)->IsInRing())
-           {
-             if (!a1)
-                a1 = nbr;
-              else if (!a2)
-                a2 = nbr;
-              else if (!a3)
-                a3 = nbr;
-              else if (!a4)
-               a4 = nbr;
+          if (!(*i)->IsInRing()) {
+            if (!a1) {
+              a1 = nbr;
             }
+            else if (!a2) {
+              a2 = nbr;
+            }
+            else if (!a3) {
+              a3 = nbr;
+            }
+            else if (!a4) {
+              a4 = nbr;
+            }
+          }
 
         //adjust geometries of heavy atoms according to hybridization
         if (hyb == 1)
@@ -1755,27 +1809,46 @@ namespace OpenBabel
 
   bool OBAtom::IsHbondAcceptor()
   {
-    return _ele == 7 || _ele == 8 || _ele == 9 ;
+    // Changes from Liu Zhiguo
+    if (_ele == 8 || _ele == 9)
+      return true;
+    if (_ele == 7) {
+      // N+ ions and sp2 hybrid N with 3 valences should not be Hbond acceptors
+      if (!((GetValence() == 4 && GetHyb() == 3) 
+            || (GetValence() == 3 && GetHyb() == 2)))
+            return true;
+    }
+    return false;
   }
 
   bool OBAtom::IsHbondDonor()
   {
-    return MatchesSMARTS("[$([#8,#7H,#9;!H0])]");
+    // return MatchesSMARTS("[$([#8,#7H,#9;!H0])]");
+    if (!(_ele == 7 || _ele == 8 || _ele == 9))
+      return false;
+
+    OBAtom *nbr;
+    OBBondIterator i;
+    for (nbr = BeginNbrAtom(i);nbr;nbr = NextNbrAtom(i))
+      if (nbr->IsHydrogen())
+        return true;
+
+    return false;
   }
 
   bool OBAtom::IsHbondDonorH()
   {
     if (!IsHydrogen()) return(false);
 
+    // Recursive definition -- this atom is an H atom
+    // are any neighbors HbondDonors?
+
     OBAtom *atom;
     OBBond *bond;
     OBBondIterator i;
-    for (bond = BeginBond(i);bond;bond = NextBond(i))
-      {
+    for (bond = BeginBond(i);bond;bond = NextBond(i)) {
         atom = bond->GetNbrAtom(this);
-        if (atom->GetAtomicNum() == 7) return(true);
-        if (atom->GetAtomicNum() == 8) return(true);
-        if (atom->GetAtomicNum() == 9) return(true);
+      if (atom->IsHbondDonor()) return(true);
       }
 
     return(false);
