@@ -145,6 +145,18 @@ namespace OpenBabel
       }
   }
 
+  void OBBond::SetLength(double length)
+  {
+    OBAtom *atom1 = GetBeginAtom();
+    OBAtom *atom2 = GetEndAtom();
+    
+    //split the length difference in half, and modify the bond twice
+    double firstLength = length + ((GetLength() - length) / 2);
+    
+    SetLength(atom1, firstLength);
+    SetLength(atom2, length);
+  }
+
   bool OBBond::IsRotor()
   {
     return(_bgn->GetHvyValence() > 1 && _end->GetHvyValence() > 1 &&
@@ -519,10 +531,10 @@ namespace OpenBabel
     return(HasFlag(OB_CLOSURE_BOND));
   }
 
-  double OBBond::GetEquibLength()
+  double OBBond::GetEquibLength() const
   {
     double length;
-    OBAtom *begin, *end;
+    const OBAtom *begin, *end;
     // CorrectedBondRad will always return a # now
     //  if (!CorrectedBondRad(GetBeginAtom(),rad1)) return(0.0);
     //  if (!CorrectedBondRad(GetEndAtom(),rad2))   return(0.0);
@@ -541,10 +553,10 @@ namespace OpenBabel
     return(length);
   }
 
-  double OBBond::GetLength()
+  double OBBond::GetLength() const
   {
     double	d2;
-    OBAtom *begin, *end;
+    const OBAtom *begin, *end;
     begin = GetBeginAtom();
     end = GetEndAtom();
 
