@@ -2,7 +2,7 @@
 data.cpp - Global data and resource file parsers.
  
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
-Some portions Copyright (C) 2001-2007 by Geoffrey R. Hutchison
+Some portions Copyright (C) 2001-2008 by Geoffrey R. Hutchison
  
 This file is part of the Open Babel project.
 For more information, see <http://openbabel.sourceforge.net/>
@@ -24,6 +24,7 @@ GNU General Public License for more details.
 #include <openbabel/babelconfig.h>
 #include <openbabel/data.h>
 #include <openbabel/mol.h>
+#include <openbabel/locale.h>
 
 // data headers with default parameters
 #include "element.h"
@@ -69,7 +70,7 @@ namespace OpenBabel
       - van der Waal radii
       - expected maximum bonding valence
       - molar mass (by IUPAC recommended atomic masses)
-      - electronegativity
+      - electronegativity (Pauling and Allred-Rochow)
       - ionization potential
       - electron affinity
       - RGB colors for visualization programs
@@ -96,16 +97,17 @@ namespace OpenBabel
   void OBElementTable::ParseLine(const char *buffer)
   {
     int num,maxbonds;
-    char symbol[5];
+    char symbol[4];
     char name[256];
-    double Rcov,Rvdw,mass, elNeg, ionize, elAffin;
+    double Rcov,Rvdw,mass, elNeg, ARENeg, ionize, elAffin;
     double red, green, blue;
 
     if (buffer[0] != '#') // skip comment line (at the top)
       {
-        sscanf(buffer,"%d %5s %lf %*f %lf %d %lf %lf %lf %lf %lf %lf %lf %255s",
+        sscanf(buffer,"%d %4s %lf %lf %*f %lf %d %lf %lf %lf %lf %lf %lf %lf %255s",
                &num,
                symbol,
+               &ARENeg,
                &Rcov,
                &Rvdw,
                &maxbonds,
@@ -118,7 +120,7 @@ namespace OpenBabel
                &blue,
                name);
 
-        OBElement *ele = new OBElement(num,symbol,Rcov,Rvdw,maxbonds,mass,elNeg,
+        OBElement *ele = new OBElement(num,symbol,ARENeg,Rcov,Rvdw,maxbonds,mass,elNeg,
                                        ionize, elAffin, red, green, blue, name);
         _element.push_back(ele);
       }
@@ -137,7 +139,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return("\0");
 
     return(_element[atomicnum]->GetSymbol());
@@ -148,7 +150,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return(0);
 
     return(_element[atomicnum]->GetMaxBonds());
@@ -159,18 +161,30 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return(0.0);
 
     return(_element[atomicnum]->GetElectroNeg());
   }
 
+  double OBElementTable::GetAllredRochowElectroNeg(int atomicnum)
+  {
+    if (!_init)
+      Init();
+
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
+      return(0.0);
+
+    return(_element[atomicnum]->GetAllredRochowElectroNeg());
+  }
+
+
   double OBElementTable::GetIonization(int atomicnum)
   {
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return(0.0);
 
     return(_element[atomicnum]->GetIonization());
@@ -182,7 +196,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return(0.0);
 
     return(_element[atomicnum]->GetElectronAffinity());
@@ -196,7 +210,7 @@ namespace OpenBabel
     vector <double> colors;
     colors.reserve(3);
     
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       {
         colors.push_back(0.0);
         colors.push_back(0.0);
@@ -216,7 +230,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return("Unknown");
 
     return(_element[atomicnum]->GetName());
@@ -227,7 +241,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return(0.0);
 
     return(_element[atomicnum]->GetVdwRad());
@@ -239,7 +253,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return(1.0);
 
     rad = _element[atomicnum]->GetCovalentRad();
@@ -258,7 +272,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return(1.95);
 
     rad = _element[atomicnum]->GetVdwRad();
@@ -276,7 +290,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return(0.0);
 
     return(_element[atomicnum]->GetCovalentRad());
@@ -287,7 +301,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (atomicnum < 0 || atomicnum > static_cast<int>(_element.size()))
+    if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       return(0.0);
 
     return(_element[atomicnum]->GetMass());
@@ -562,7 +576,7 @@ namespace OpenBabel
     if (!_init)
       Init();
 
-    if (!from.size())
+    if (from.empty())
       return("");
 
     if (_from >= 0 && _to >= 0 &&
@@ -639,7 +653,7 @@ namespace OpenBabel
     vector3 v;
 
     int bo;
-    unsigned int skipres=0;
+    string skipres = ""; // Residue Number to skip
     string rname = "";
     //assign residue bonds
     for (a1 = mol.BeginAtom(i);a1;a1 = mol.NextAtom(i))
@@ -648,12 +662,12 @@ namespace OpenBabel
         if (r1 == NULL) // atoms may not have residues
           continue;
 
-        if (skipres && r1->GetNum() == skipres)
+        if (skipres.length() && r1->GetNumString() == skipres)
           continue;
 
         if (r1->GetName() != rname)
           {
-            skipres = SetResName(r1->GetName()) ? 0 : r1->GetNum();
+            skipres = SetResName(r1->GetName()) ? "" : r1->GetNumString();
             rname = r1->GetName();
           }
         //assign bonds for each atom
@@ -663,15 +677,18 @@ namespace OpenBabel
             if (r2 == NULL) // atoms may not have residues
               continue;
 
-            if (r1->GetNum() != r2->GetNum())
+            if (r1->GetNumString() != r2->GetNumString())
               break;
             if (r1->GetName() != r2->GetName())
               break;
 
             if ((bo = LookupBO(r1->GetAtomID(a1),r2->GetAtomID(a2))))
               {
-                v = a1->GetVector() - a2->GetVector();
-                if (v.length_2() < 3.5) //check by distance
+                // Suggested by Liu Zhiguo 2007-08-13
+                // for predefined residues, don't perceive connection
+                // by distance
+                //                v = a1->GetVector() - a2->GetVector();
+                //                if (v.length_2() < 3.5) //check by distance
                   mol.AddBond(a1->GetIdx(),a2->GetIdx(),bo);
               }
           }
@@ -682,7 +699,7 @@ namespace OpenBabel
 
     //types and hybridization
     rname = ""; // name of current residue
-    skipres = 0; // don't skip any residues right now
+    skipres = ""; // don't skip any residues right now
     for (a1 = mol.BeginAtom(i);a1;a1 = mol.NextAtom(i))
       {
         if (a1->IsOxygen() && !a1->GetValence())
@@ -717,13 +734,13 @@ namespace OpenBabel
 	
         r1 = a1->GetResidue();
         if (r1 == NULL) continue; // atoms may not have residues
-        if (skipres && r1->GetNum() == skipres)
+        if (skipres.length() && r1->GetNumString() == skipres)
           continue;
 
         if (r1->GetName() != rname)
           {
             // if SetResName fails, skip this residue
-            skipres = SetResName(r1->GetName()) ? 0 : r1->GetNum();
+            skipres = SetResName(r1->GetName()) ? "" : r1->GetNumString();
             rname = r1->GetName();
           }
 
@@ -856,9 +873,14 @@ namespace OpenBabel
     ifstream ifs;
     char charBuffer[BUFF_SIZE];
 
-    OpenDatafile(ifs, _filename, _envvar);
+    // Set the locale for number parsing to avoid locale issues: PR#1785463
+    obLocale.SetLocale();
+
+    // Check return value from OpenDatafile
+    // Suggestion from Zhiguo Liu
+    string fn_open = OpenDatafile(ifs, _filename, _envvar);
      
-    if ((ifs))
+    if (fn_open != "" && (ifs))
       {
         while(ifs.getline(charBuffer,BUFF_SIZE))
           ParseLine(charBuffer);
@@ -886,6 +908,9 @@ namespace OpenBabel
           obErrorLog.ThrowError(__FUNCTION__, s, obWarning);
         }
 
+    // return the locale to the original one
+    obLocale.RestoreLocale();
+    
     if (ifs)
       ifs.close();
 
@@ -899,72 +924,6 @@ namespace OpenBabel
       
   }
 
-  /** Opens the filestream with the first file called @p filename
-     found by looking 
-     successively in the following directories:
-     - the current directory
-     - in a subdirectory (of the directory below) with the version of 
-     OpenBabel as its name
-     - the parent directory specified by the environment variable 
-     named @p envvar 
-     or "BABEL_DATADIR" if @p envvar is not specified, or the compiled-in
-     macro BABEL_DATADIR if the environment variable is not set
-
-     \param ifs        Stream to load
-     \param filename   Name of the data file to load
-     \param envvar     Name of the environment variable 
-
-     \return the full name of the file that was opened (i.e with path) except 
-     if it is in current directory
-
-  **/
-  std::string OpenDatafile(std::ifstream& ifs, const std::string& filename, 
-                           const std::string& envvar)
-  {
-    ios_base::openmode imode = ios_base::in;
-#ifdef ALL_READS_BINARY //Makes unix files compatible with VC++6
-		imode = ios_base::in|ios_base::binary;
-#endif
-
-    // check the current directory
-    ifs.close();
-    ifs.clear();
-    ifs.open(filename.c_str(),imode);
-    if(ifs)
-      return filename;
-
-    string file;
-    const char* datadir = getenv(envvar.c_str());
-    if(!datadir)
-      datadir = BABEL_DATADIR;
-
-    // check the subdirectory for this version number
-    file = datadir;
-    file += FILE_SEP_CHAR;
-    file += BABEL_VERSION;
-    file += FILE_SEP_CHAR + filename;
-
-    ifs.clear();
-    ifs.open(file.c_str(),imode);
-    if(ifs)
-      return file;
-
-    // couldn't find it with the version built in, so try the parent
-    file = datadir;
-    file += FILE_SEP_CHAR;
-    file += filename;
-
-    ifs.clear();
-    ifs.open(file.c_str(),imode);
-
-    if (ifs)
-      return file;
-
-    ifs.clear();
-    ifs.close();
-    return(""); // error
-  }
-
 } // end namespace OpenBabel
 
 //! \file data.cpp
