@@ -24,6 +24,7 @@ GNU General Public License for more details.
 #include <dirent.h>
 #include <sys/stat.h>
 #include <dlfcn.h>
+#include <cstring>
 #include <cstdlib>
 
 using namespace std;
@@ -40,7 +41,13 @@ namespace OpenBabel {
 
 int matchFiles (SCANDIR_CONST struct dirent *entry_p)
 {
-  return (strstr(entry_p->d_name, DLHandler::getFormatFilePattern()) != 0);
+	string filename(entry_p->d_name);
+	string::size_type extPos = filename.rfind(DLHandler::getFormatFilePattern());
+
+	if(extPos!=string::npos && filename.substr(extPos) == DLHandler::getFormatFilePattern())
+		return true;
+		
+	return false;
 }
 
 bool DLHandler::getConvDirectory(string& convPath)
@@ -75,14 +82,14 @@ int DLHandler::findFiles (std::vector <std::string>& file_list,const std::string
       
       OpenBabel::tokenize(vs, buffer, "\r\n\t :");
       
-      if (vs.size() > 0)
+      if (!vs.empty())
         {
           for (unsigned int i = 0; i < vs.size(); ++i)
             paths.push_back(vs[i]);
         }
     }
   
-  if (paths.size() == 0)
+  if (paths.empty())
     paths.push_back("./"); // defaults to current directory
   
   /* Old method using scandir. Replaced with readdir (below) as for example
@@ -128,7 +135,7 @@ int DLHandler::findFiles (std::vector <std::string>& file_list,const std::string
         }
     }
   
-  if (file_list.size() == 0)
+  if (file_list.empty())
     return(-1); // error, didn't find any files at all
   return file_list.size();
 }
