@@ -33,14 +33,27 @@ namespace OpenBabel
 
   const unsigned int OBFingerprint::bitsperint = 8 * sizeof(unsigned int);
 
-  void OBFingerprint::SetBit(vector<unsigned int>& vec, unsigned int n)
+  void OBFingerprint::SetBit(vector<unsigned int>& vec, const unsigned int n)
   {
     vec[n/Getbitsperint()] |= (1 << (n % Getbitsperint()));
   }
 
+  bool OBFingerprint::GetBit(const vector<unsigned int>& vec, const unsigned int n)
+  {
+    unsigned int word =vec[n/Getbitsperint()];
+    return (word &= (1 << (n % Getbitsperint())))!=0;
+  }
+
   ////////////////////////////////////////	
   void OBFingerprint::Fold(vector<unsigned int>& vec, unsigned int nbits)
   {
+    if(nbits<Getbitsperint())
+    {
+      stringstream ss;
+      ss << "Can't fold to less than " << Getbitsperint() << "bits";
+      obErrorLog.ThrowError(__FUNCTION__, ss.str(), obError);
+      return;
+    }
     while(vec.size()*Getbitsperint()/2 >= nbits) 
       vec.erase(transform(vec.begin(),vec.begin()+vec.size()/2,
                           vec.begin()+vec.size()/2, vec.begin(), bit_or()), vec.end());
@@ -87,10 +100,15 @@ namespace OpenBabel
         int andfp = vec1[i] & vec2[i];
         int orfp = vec1[i] | vec2[i];
         //Count bits
+#ifdef __GNUC__
+        andbits += __builtin_popcount(andfp);
+        orbits += __builtin_popcount(orfp);
+#else
         for(;andfp;andfp=andfp<<1)
           if(andfp<0) ++andbits;
         for(;orfp;orfp=orfp<<1)
           if(orfp<0) ++orbits;
+#endif
       }
     if(orbits==0)
       return 0.0;
@@ -315,8 +333,7 @@ namespace OpenBabel
   OBFingerprint* FptIndex::CheckFP()
   {
     //check that fingerprint type is available
-    string tempFP(header.fpid);
-    OBFingerprint* pFP = OBFingerprint::FindFingerprint(tempFP);
+    OBFingerprint* pFP = OBFingerprint::FindFingerprint(header.fpid);
     if(!pFP)
       {
         stringstream errorMsg;
@@ -337,6 +354,7 @@ namespace OpenBabel
     _pindex= new FptIndex;
     _pindex->header.headerlength = sizeof(FptIndexHeader);
     strncpy(_pindex->header.fpid,fpid.c_str(),15);
+    _pindex->header.fpid[15]='\0'; //ensure fpid is terminated at 15 characters.
     strncpy(_pindex->header.datafilename, datafilename.c_str(), 255);
 
     //check that fingerprint type is available
@@ -420,7 +438,7 @@ namespace OpenBabel
 
     For more on these specific implementations of fingerprints in Open
     Babel, please take a look at the developer's wiki:
-    http://openbabel.sourceforge.net/wiki/Fingerprints
+    http://openbabel.org/wiki/Fingerprints
  
     Fingerprints derived from this abstract base class OBFingerprint can be for any 
     object derived from OBBase (not just for OBMol). 
@@ -523,7 +541,7 @@ namespace OpenBabel
     of candidates by another method (like OBSmartsPattern).
 
     Note that the index files are not portable. They need to be prepared on the
-    computer that will acess them.
+    computer that will access them.
 
     <h4>Using FastSearch and FastSearchIndexer in a program</h4>
 
