@@ -1,6 +1,6 @@
 /**********************************************************************
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
-Some portions Copyright (C) 2001-2005 by Geoffrey R. Hutchison
+Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
 Some portions Copyright (C) 2004 by Chris Morley
  
 This program is free software; you can redistribute it and/or modify
@@ -12,10 +12,9 @@ but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
+#include <openbabel/babelconfig.h>
 
-#include "mol.h"
-#include "obconversion.h"
-#include "obmolecformat.h"
+#include <openbabel/obmolecformat.h>
 
 using namespace std;
 namespace OpenBabel
@@ -60,7 +59,7 @@ CCCFormat theCCCFormat;
 bool CCCFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 {
 
-    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+    OBMol* pmol = pOb->CastAndClear<OBMol>();
     if(pmol==NULL)
         return false;
 
