@@ -53,8 +53,8 @@ namespace OpenBabel
     virtual bool ReadMolecule(OBBase* pOb, OBConversion* pConv);
     virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
 
-    static bool ReadChem3d(istream &ifs,OBMol &mol,bool mmads,char *type_key);
-    static bool WriteChem3d(ostream &ofs,OBMol &mol, char *mol_typ);
+    static bool ReadChem3d(istream &ifs,OBMol &mol,bool mmads,const char *type_key);
+    static bool WriteChem3d(ostream &ofs,OBMol &mol, const char *mol_typ);
 
   };
   //***
@@ -75,7 +75,7 @@ namespace OpenBabel
     OBMol &mol = *pmol;
     mol.SetTitle( pConv->GetTitle()); //default title is the filename
 
-    return(ReadChem3d(ifs,mol,false,const_cast<char*>("MM2")));
+    return(ReadChem3d(ifs,mol,false,"MM2"));
   }
 
   ////////////////////////////////////////////////////////////////
@@ -90,7 +90,7 @@ namespace OpenBabel
     ostream &ofs = *pConv->GetOutStream();
     OBMol &mol = *pmol;
 
-    return(WriteChem3d(ofs,mol,const_cast<char*>("MM2")));
+    return(WriteChem3d(ofs,mol,"MM2"));
   }
 
   //***********************************************************
@@ -144,7 +144,7 @@ namespace OpenBabel
     OBMol &mol = *pmol;
     mol.SetTitle( pConv->GetTitle()); //default title is the filename
 
-    return(CHEM3D1Format::ReadChem3d(ifs,mol,false,const_cast<char*>("C3D")));
+    return(CHEM3D1Format::ReadChem3d(ifs,mol,false,"C3D"));
   }
 
   ////////////////////////////////////////////////////////////////
@@ -159,12 +159,12 @@ namespace OpenBabel
     ostream &ofs = *pConv->GetOutStream();
     OBMol &mol = *pmol;
 
-    return(CHEM3D1Format::WriteChem3d(ofs,mol,const_cast<char*>("C3D")));
+    return(CHEM3D1Format::WriteChem3d(ofs,mol,"C3D"));
   }
 
   //****************************************************************
 
-  bool CHEM3D1Format::ReadChem3d(istream &ifs,OBMol &mol,bool mmads,char *type_key)
+  bool CHEM3D1Format::ReadChem3d(istream &ifs,OBMol &mol,bool mmads,const char *type_key)
   {
     char buffer[BUFF_SIZE];
     int natoms,i;
@@ -263,7 +263,7 @@ namespace OpenBabel
     return(true);
   }
 
-  bool CHEM3D1Format::WriteChem3d(ostream &ofs,OBMol &mol, char *mol_typ)
+  bool CHEM3D1Format::WriteChem3d(ostream &ofs,OBMol &mol, const char *mol_typ)
   {
     int atnum;
     int type_num;
