@@ -18,6 +18,7 @@ GNU General Public License for more details.
 
 #include <openbabel/babelconfig.h>
 #include <openbabel/obmolecformat.h>
+#include <openbabel/math/spacegroup.h>
 
 #include <sstream>
 #include <vector>
@@ -34,7 +35,7 @@ namespace OpenBabel
     //Register this format type ID
     CIFFormat()
     {
-      OBConversion::RegisterFormat("cif", this, "chemical/x-cif");
+      RegisterFormat("cif", "chemical/x-cif");
     }
 
     virtual const char* Description() //required
@@ -46,9 +47,6 @@ namespace OpenBabel
     virtual const char* SpecificationURL()
     {return "http://www.iucr.org/iucr-top/cif/spec/";}; //optional
 
-    virtual const char* GetMIMEType() 
-    { return "chemical/x-cif"; };
-
     //*** This section identical for most OBMol conversions ***
     ////////////////////////////////////////////////////
     /// The "API" interface functions
@@ -212,6 +210,7 @@ namespace OpenBabel
     float mOrthMatrix[3][3];
     /// Cartesian2Fractionnal matrix
     float mOrthMatrixInvert[3][3];
+    const SpaceGroup *mSpaceGroup;
   };
   /** Main CIF class - parses the stream and separates data blocks, comments, items, loops.
    * All values are stored as string, and Each CIF block is stored in a separate CIFData object.
@@ -292,10 +291,12 @@ namespace OpenBabel
   void CIFData::ExtractSpacegroup(const bool verbose)
   {
     map<ci_string,string>::const_iterator positem;
+    bool found = false;
     positem=mvItem.find("_space_group_IT_number");
     if(positem!=mvItem.end())
       {
         mSpacegroupNumberIT=CIFNumeric2Int(positem->second);
+        found = true;
         if(verbose) cout<<"Found spacegroup IT number:"<<mSpacegroupNumberIT<<endl;
       }
     else
@@ -304,14 +305,18 @@ namespace OpenBabel
         if(positem!=mvItem.end())
           {
             mSpacegroupNumberIT=CIFNumeric2Int(positem->second);
+            found = true;
             if(verbose) cout<<"Found spacegroup IT number (with OBSOLETE CIF #1.0 TAG):"<<mSpacegroupNumberIT<<endl;
           }
+        else
+          mSpacegroupNumberIT=0;
       }
       
     positem=mvItem.find("_space_group_name_Hall");
     if(positem!=mvItem.end())
       {
         mSpacegroupSymbolHall=positem->second;
+        found = true;
         if(verbose) cout<<"Found spacegroup Hall symbol:"<<mSpacegroupSymbolHall<<endl;
       }
     else
@@ -320,6 +325,7 @@ namespace OpenBabel
         if(positem!=mvItem.end())
           {
             mSpacegroupSymbolHall=positem->second;
+            found = true;
             if(verbose) cout<<"Found spacegroup Hall symbol (with OBSOLETE CIF #1.0 TAG):"<<mSpacegroupSymbolHall<<endl;
           }
       }
@@ -328,6 +334,7 @@ namespace OpenBabel
     if(positem!=mvItem.end())
       {
         mSpacegroupHermannMauguin=positem->second;
+        found = true;
         if(verbose) cout<<"Found spacegroup Hermann-Mauguin symbol:"<<mSpacegroupHermannMauguin<<endl;
       }
     else
@@ -336,9 +343,52 @@ namespace OpenBabel
         if(positem!=mvItem.end())
           {
             mSpacegroupHermannMauguin=positem->second;
-            if(verbose) cout<<"Found spacegroup Hall Hermann-Mauguin (with OBSOLETE CIF #1.0 TAG):"<<mSpacegroupHermannMauguin<<endl;
+            found = true;
+            if(verbose) cout<<"Found spacegroup Hermann-Mauguin symbol (with OBSOLETE CIF #1.0 TAG):"<<mSpacegroupHermannMauguin<<endl;
           }
       }
+    if (mSpacegroupSymbolHall.length() > 0)
+      mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupSymbolHall);
+    else
+      mSpaceGroup=NULL;
+    if (!mSpaceGroup && mSpacegroupHermannMauguin.length() > 0)
+      mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupHermannMauguin);
+    if (!mSpaceGroup)
+      mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupNumberIT);
+    SpaceGroup *sg = new SpaceGroup();
+    if (mSpacegroupSymbolHall.length() > 0)
+      sg->SetHallName(mSpacegroupSymbolHall);
+    if (mSpacegroupHermannMauguin.length() > 0)
+      sg->SetHMName(mSpacegroupHermannMauguin);
+    if (mSpacegroupNumberIT > 0)
+      sg->SetId(mSpacegroupNumberIT);
+    positem=mvItem.find("_symmetry_equiv_pos_as_xyz");
+    if(positem!=mvItem.end())
+      {
+        sg->AddTransform (positem->second);
+        found = true;
+      }
+    else for(map<set<ci_string>,map<ci_string,vector<string> > >::const_iterator loop=mvLoop.begin();
+      loop!=mvLoop.end();++loop)
+      {
+        map<ci_string,vector<string> >::const_iterator pos;
+        unsigned i, nb;
+        pos=loop->second.find("_symmetry_equiv_pos_as_xyz");
+        if (pos!=loop->second.end())
+          {
+            nb=pos->second.size();
+            found = true;
+            for (i = 0; i < nb; i++)
+              sg->AddTransform(pos->second[i]);
+            break; // found the transforms, so we have done with them
+          }
+    }
+    if (found)
+      mSpaceGroup = SpaceGroup::Find(sg);
+    delete sg;
+    if (mSpaceGroup != NULL)
+      // set the space group name to Hall symbol
+      mSpacegroupSymbolHall = mSpaceGroup->GetHallName();
   }
    
   void CIFData::ExtractName(const bool verbose)
@@ -729,6 +779,9 @@ namespace OpenBabel
           {//Tag
             string tag,value;
             in>>tag;
+            // Convert all dots to underscores to cover much of DDL2 with this DDL1 parser.
+            for (string::size_type pos = tag.find('.'); pos != string::npos; pos = tag.find('.', ++ pos))
+              tag.replace(pos, 1, 1, '_');
             value=CIFReadValue(in,lastc);
             mvData[block].mvItem[ci_string(tag.c_str())]=value;
             if(vv)cout<<"New Tag:"<<tag<<" ("<<value.size()<<"):"<<value<<endl;
@@ -765,6 +818,9 @@ namespace OpenBabel
                     break;
                   }
                 in>>tmp;
+                // Convert all dots to underscores to cover much of DDL2 with this DDL1 parser.
+                for (string::size_type pos = tmp.find('.'); pos != string::npos; pos = tmp.find('.', ++ pos))
+                  tmp.replace(pos, 1, 1, '_');
                 tit.push_back(ci_string(tmp.c_str()));
                 if(vv) cout<<" , "<<tmp;
               }
@@ -875,6 +931,7 @@ namespace OpenBabel
                              pos->second.mvLatticePar[4]/DEG_TO_RAD,
                              pos->second.mvLatticePar[5]/DEG_TO_RAD);
               pCell->SetSpaceGroup(spg);
+              pCell->SetSpaceGroup(pos->second.mSpaceGroup);
               pmol->SetData(pCell);
             }
           if(pos->second.mName!="") pmol->SetTitle(pos->second.mName);
@@ -888,13 +945,45 @@ namespace OpenBabel
           pmol->ReserveAtoms(nbatoms);
           for(vector<CIFData::CIFAtom>::const_iterator posat=pos->second.mvAtom.begin();posat!=pos->second.mvAtom.end();++posat)
             {
-              OBAtom *atom  = pmol->NewAtom();
-              int atomicNum = etab.GetAtomicNum(posat->mSymbol.c_str());
-              // :TODO: posat->mSymbol is not garanteed to actually be a symbol
+              // Problem: posat->mSymbol is not garanteed to actually be a symbol
               // see http://www.iucr.org/iucr-top/cif/cifdic_html/1/cif_core.dic/Iatom_type_symbol.html
+              // Try to strip the string to have a better chance to have a valid symbol
+              // This is not guaranteed to work still, as the CIF standard allows about any string...
+              string tmpSymbol=posat->mSymbol;
+              int nbc=0;
+              if((tmpSymbol.size()==1) && isalpha(tmpSymbol[0])) nbc=1;
+              else if(tmpSymbol.size()>=2)
+                {
+                  if(isalpha(tmpSymbol[0]) && isalpha(tmpSymbol[1])) nbc=2;
+                  else if(isalpha(tmpSymbol[0])) nbc=1;
+                }
+              
+              OBAtom *atom  = pmol->NewAtom();
+              
+              if(tmpSymbol.size()>nbc)
+                {// Try to find a formal charge in the symbol
+                  int charge=0;
+                  int sign=0;
+                  for(unsigned int i=nbc;i<tmpSymbol.size();++i)
+                    {// Use first number found as formal charge
+                      if(isdigit(tmpSymbol[i]) && (charge==0)) charge=atoi(tmpSymbol.substr(i,1).c_str());
+                      if('-'==tmpSymbol[i]) sign-=1;
+                      if('+'==tmpSymbol[i]) sign+=1;
+                    }
+                  if(0!=sign) // no sign, no charge
+                    {
+                      if(charge==0) charge=1;
+                      cout<<tmpSymbol<<" / symbol="<<tmpSymbol.substr(0,nbc)<<" charge= "<<sign*charge<<endl;
+                      atom->SetFormalCharge(sign*charge);
+                    }
+                }
+              
+              if(nbc>0) tmpSymbol=tmpSymbol.substr(0,nbc);
+              else tmpSymbol="C";//Something went wrong, no symbol ! Default to C ??
+              
+              int atomicNum = etab.GetAtomicNum(tmpSymbol.c_str());
               atom->SetAtomicNum(atomicNum); //set atomic number, or '0' if the atom type is not recognized
-              string symb=posat->mSymbol;// Need a no-const string to pass...
-              atom->SetType(symb); //set atomic number, or '0' if the atom type is not recognized
+              atom->SetType(tmpSymbol); //set atomic number, or '0' if the atom type is not recognized
               atom->SetVector(posat->mCoordCart[0],posat->mCoordCart[1],posat->mCoordCart[2]);
             }
           if (!pConv->IsOption("b",OBConversion::INOPTIONS))
@@ -936,6 +1025,21 @@ namespace OpenBabel
             << "_cell_angle_alpha " << pUC->GetAlpha() << endl
             << "_cell_angle_beta "  << pUC->GetBeta() << endl
             << "_cell_angle_gamma " << pUC->GetGamma() << endl;
+        // Save the space group if known
+        const SpaceGroup* pSG = pUC->GetSpaceGroup();
+        if (pSG != NULL)
+          {
+            ofs << "_space_group_name_Hall '" << pSG->GetHallName() << "'" << endl;
+            ofs << "loop_" <<endl
+                << "    _symmetry_equiv_pos_as_xyz" << endl;
+            transform3dIterator ti;
+            const transform3d *t = pSG->BeginTransform(ti);
+            while(t)
+              {
+                ofs << "    '" << t->DescribeAsString() << "'" << endl;
+                t = pSG->NextTransform(ti);
+             }
+          }
       }
 
 
