@@ -32,15 +32,16 @@ namespace OpenBabel
 
     virtual const char* Description() //required
     { return
-        "Fingerprint format\n \
-Constructs and displays fingerprints and (for multiple input objects)\n \
-the Tanimoto coefficient and whether a superstructure of the first object\n \
-Options e.g. -xfFP3 -xn128\n \
- f<id> fingerprint type\n \
- N# fold to specified number of bits, 32, 64, 128, etc.\n \
- h  hex output when multiple molecules\n \
- F  displays the available fingerprint types\n \
-";
+      "Fingerprint format\n"
+      "Constructs and displays fingerprints and (for multiple input objects)\n"
+      "the Tanimoto coefficient and whether a superstructure of the first object\n"
+      "Output options e.g. -xfFP3 -xN128\n"
+      " f<id> fingerprint type\n"
+      " N# fold to specified number of bits, 32, 64, 128, etc.\n"
+      " h  hex output when multiple molecules\n"
+      " s  describe each set bit\n"
+      " u  describe each unset bit\n"
+;
     };
 
     virtual unsigned int Flags(){return NOTREADABLE;};
@@ -60,13 +61,6 @@ Options e.g. -xfFP3 -xn128\n \
   bool FingerprintFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   {
     ostream &ofs = *pConv->GetOutStream();
-    if(pConv->IsOption("F"))
-      {
-        FOR_EACH(OBFingerprint, iter)
-          ofs << iter.ID() << " -- " << iter->Description() << endl;
-        return true;
-      }
-
 
     bool hexoutput=false;
     if(pConv->IsOption("h") || (pConv->GetOutputIndex()==1 && pConv->IsLast()))
@@ -81,7 +75,7 @@ Options e.g. -xfFP3 -xn128\n \
         fpid = fpid.substr(0,fpid.find('"'));
       }
 
-    OBFingerprint* pFP = OBFingerprint::FindFingerprint(fpid);
+    OBFingerprint* pFP = OBFingerprint::FindFingerprint(fpid.c_str());
     if(!pFP)
       {
         stringstream errorMsg;
@@ -92,7 +86,10 @@ Options e.g. -xfFP3 -xn128\n \
 
     p=pConv->IsOption("N");
     if(p)
-      nbits = atoi(p);		
+      nbits = atoi(p);
+    if(nbits<0)
+      obErrorLog.ThrowError(__FUNCTION__,
+      "The number of bits to fold to, in the-xN option, should be >=0", obWarning);
 
     vector<unsigned int> fptvec;
     if(!pFP->GetFingerprint(pOb, fptvec, nbits))
@@ -101,8 +98,26 @@ Options e.g. -xfFP3 -xn128\n \
     OBMol* pmol = dynamic_cast<OBMol*>(pOb);
     if(pmol)
       ofs << ">" << pmol->GetTitle();
+    
+    // checkmol-type output
+    if(pConv->IsOption("s") || pConv->IsOption("u"))
+    {
+      if(nbits!=0)
+      {
+        obErrorLog.ThrowError(__FUNCTION__,
+        "The fingerprint must be unfolded when describing bits.", obError);
+        return false;
+      }
+      string descr = pFP->DescribeBits(fptvec, pConv->IsOption("s")!=NULL);
+      if(descr=="")
+        obErrorLog.ThrowError(__FUNCTION__,
+        "Bit descriptions are not available for this fingerprint type", obError, onceOnly);
 
-    if(hexoutput)
+      ofs << descr;
+      return true;
+    }
+
+    if(hexoutput && pConv->GetOutputIndex()==1)
       {
         unsigned int i, bitsset=0;
         for (i=0;i<fptvec.size();++i)
@@ -111,7 +126,7 @@ Options e.g. -xfFP3 -xn128\n \
             for(;wd;wd=wd<<1)//count bits set by shifting into sign bit until word==0
               if(wd<0) ++bitsset;
           }
-        ofs  << "   " << bitsset << " bits set. "; 
+        ofs  << "   " << bitsset << " bits set "; 
       }
 
     if(pConv->GetOutputIndex()==1)
@@ -122,7 +137,9 @@ Options e.g. -xfFP3 -xn128\n \
           firstname=pmol->GetTitle();
         if(firstname.empty())
           firstname = "first mol";
-      }
+
+
+    }
     else
       {
         ofs << "   Tanimoto from " << firstname << " = " << OBFingerprint::Tanimoto(firstfp, fptvec);
