@@ -191,11 +191,11 @@ namespace OpenBabel
     FOR_ATOMS_OF_MOL(atom, mol)
       {
         if (atom->GetIsotope() == 0)
-          snprintf(buffer, BUFF_SIZE, "%-3s      %10.5f      %10.5f      %10.5f ",
+          snprintf(buffer, BUFF_SIZE, "%-3s      %10.5f      %10.5f      %10.5f",
                    etab.GetSymbol(atom->GetAtomicNum()),
                    atom->GetX(), atom->GetY(), atom->GetZ());
         else
-          snprintf(buffer, BUFF_SIZE, "%-3s(Iso=%d) %10.5f      %10.5f      %10.5f ",
+          snprintf(buffer, BUFF_SIZE, "%-3s(Iso=%d) %10.5f      %10.5f      %10.5f",
                    etab.GetSymbol(atom->GetAtomicNum()),
                    atom->GetIsotope(),
                    atom->GetX(), atom->GetY(), atom->GetZ());
@@ -264,14 +264,21 @@ namespace OpenBabel
     int charge = 0;
     unsigned int spin = 1;
     bool hasPartialCharges = false;
-    
+
+    //Vibrational data
+    std::vector< std::vector< vector3 > > Lx;
+    std::vector<double> Frequencies, Intensities;;
+    //Rotational data
+    std::vector<double> RotConsts(3);
+    int RotSymNum;
+    OBRotationData::RType RotorType;
+
     mol.BeginModify();
     
     while (ifs.getline(buffer,BUFF_SIZE))
       {
-        if (strstr(buffer,"Symbolic Z-matrix:") != NULL)
+        if (strstr(buffer,"Multiplicity") != NULL)
           {
-            ifs.getline(buffer,BUFF_SIZE); // Charge = # Multiplicty = #
             tokenize(vs, buffer, " \t\n");
             if (vs.size() == 6)
               {
@@ -307,6 +314,24 @@ namespace OpenBabel
                 tokenize(vs,buffer);
               }
           }
+        else if(strstr(buffer,"Dipole moment") != NULL)
+            {
+              ifs.getline(buffer,BUFF_SIZE); // actual components   X ###  Y #### Z ###
+              tokenize(vs,buffer);
+              if (vs.size() >= 6) 
+                {
+                  OBVectorData *dipoleMoment = new OBVectorData;
+                  dipoleMoment->SetAttribute("Dipole Moment");
+                  double x, y, z;
+                  x = atof(vs[1].c_str());
+                  y = atof(vs[3].c_str());
+                  z = atof(vs[5].c_str());
+                  dipoleMoment->SetData(x, y, z);
+                  dipoleMoment->SetOrigin(fileformatInput);
+                  mol.SetData(dipoleMoment);
+                }
+              if (!ifs.getline(buffer,BUFF_SIZE)) break;
+            }
         else if(strstr(buffer,"Total atomic charges") != NULL ||
                 strstr(buffer,"Mulliken atomic charges") != NULL)
           {
@@ -318,12 +343,51 @@ namespace OpenBabel
                    strstr(buffer,"Sum of ") == NULL)
               {
                 atom = mol.GetAtom(atoi(vs[0].c_str()));
+                if (!atom)
+                  break;
                 atom->SetPartialCharge(atof(vs[2].c_str()));
 		
                 if (!ifs.getline(buffer,BUFF_SIZE)) break;
                 tokenize(vs,buffer);
               }
           }
+
+        else if(strstr(buffer, " Frequencies --")) //vibrational frequencies
+        {
+          //The info should appear only once as several blocks starting with this line
+          tokenize(vs, buffer);
+          for(int i=2; i<vs.size(); ++i)
+            Frequencies.push_back(atof(vs[i].c_str()));
+          ifs.getline(buffer,BUFF_SIZE); //Red. masses
+          ifs.getline(buffer,BUFF_SIZE); //Frc consts
+          ifs.getline(buffer,BUFF_SIZE); //IR Inten
+          tokenize(vs, buffer);
+          for(int i=3; i<vs.size(); ++i)
+            Intensities.push_back(atof(vs[i].c_str()));
+          //** Lx yet to be done **
+        }
+
+        else if(strstr(buffer, " This molecule is "))//rotational data
+        {
+          if(strstr(buffer, "asymmetric"))
+            RotorType = OBRotationData::ASYMMETRIC;
+          else if(strstr(buffer, "symmetric"))
+            RotorType = OBRotationData::SYMMETRIC;
+          else if(strstr(buffer, "linear"))
+            RotorType = OBRotationData::LINEAR;
+          else
+             RotorType = OBRotationData::UNKNOWN;
+          ifs.getline(buffer,BUFF_SIZE); //symmetry number
+          tokenize(vs, buffer);
+          RotSymNum = atoi(vs[3].c_str());
+          ifs.getline(buffer,BUFF_SIZE); //Rot temperatures
+          ifs.getline(buffer,BUFF_SIZE); //Rot consts
+          tokenize(vs, buffer);
+          for(int i=3; i<vs.size(); ++i)
+            RotConsts[i-3] = atof(vs[i].c_str());
+         
+        }
+
         else if(strstr(buffer,"SCF Done:") != NULL)
           {
 #define HARTREE_TO_KCAL 627.509
@@ -336,6 +400,21 @@ namespace OpenBabel
       mol.EndModify();
       return false;
     }
+    
+    //Attach vibrational data, if there is any, to molecule
+    if(Frequencies.size()>0)
+    {
+      OBVibrationData* vd = new OBVibrationData;
+      vd->SetData(Lx, Frequencies, Intensities);
+      mol.SetData(vd);
+    }
+    //Attach rotational data, if there is any, to molecule
+    if(RotConsts[0]!=0.0)
+    {
+      OBRotationData* rd = new OBRotationData;
+      rd->SetData(RotorType, RotConsts, RotSymNum);
+      mol.SetData(rd);
+    }
 
     if (!pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.ConnectTheDots();
