@@ -75,16 +75,18 @@ namespace OpenBabel
     vector<string> vs;
 
     ifs.getline(buffer, BUFF_SIZE);
-    while (strstr(buffer,"mol") == NULL)
+    while (ifs.good() && strstr(buffer,"mol") == NULL)
       {
         ifs.getline(buffer, BUFF_SIZE);
         if (ifs.peek() == EOF || !ifs.good())
           return false;
       }
     ifs.getline(buffer, BUFF_SIZE);
+    if (!ifs.good())
+      return false; // ended early
 
     mol.BeginModify();
-    while (strstr(buffer,"endmol") == NULL)
+    while (ifs.good() && strstr(buffer,"endmol") == NULL)
       {
         tokenize(vs,buffer); // Don't really know how long it'll be
         if (vs.size() < 11)
@@ -95,6 +97,7 @@ namespace OpenBabel
 
         atom = mol.NewAtom();
         atom->SetAtomicNum(etab.GetAtomicNum(vs[3].c_str()));
+        atom->SetPartialCharge(atof(vs[6].c_str()));
         x = atof((char*)vs[7].c_str());
         y = atof((char*)vs[8].c_str());
         z = atof((char*)vs[9].c_str());
@@ -134,6 +137,8 @@ namespace OpenBabel
     mol.EndModify();
 
     mol.SetTitle(title);
+    mol.SetPartialChargesPerceived();
+    
     return(true);
   }
 
