@@ -1,12 +1,19 @@
 /*
- * International Union of Pure and Applied Chemistry (IUPAC)
  * International Chemical Identifier (InChI)
  * Version 1
- * Software version 1.01
- * July 21, 2006
+ * Software version 1.02-beta
+ * August 23, 2007
  * Developed at NIST
+ *
+ * The InChI library and programs are free software developed under the
+ * auspices of the International Union of Pure and Applied Chemistry (IUPAC);
+ * you can redistribute this software and/or modify it under the terms of 
+ * the GNU Lesser General Public License as published by the Free Software 
+ * Foundation:
+ * http://www.opensource.org/licenses/lgpl-license.php
  */
 
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -14,7 +21,7 @@
 
 #include "mode.h"
 
-#include "comdef.h"
+#include "incomdef.h"
 #include "inpdef.h"
 #include "util.h"
 #include "extr_ct.h"
