@@ -21,7 +21,11 @@ GNU General Public License for more details.
 #include <openbabel/obmolecformat.h>
 
 #include "inchi_api.h"
+#ifdef HAVE_SSTREAM
 #include <sstream>
+#else
+#include <strstream>
+#endif
 #include <set>
 #include <vector>
 #include "openbabel/chiral.h"
@@ -39,7 +43,11 @@ public:
     OBConversion::RegisterFormat("inchi",this);
     OBConversion::RegisterOptionParam("n", this, 0, OBConversion::INOPTIONS);
     OBConversion::RegisterOptionParam("t", this);
+    OBConversion::RegisterOptionParam("l", this);
     OBConversion::RegisterOptionParam("X", this, 1, OBConversion::OUTOPTIONS);
+    OBConversion::RegisterOptionParam("K", this, 0, OBConversion::OUTOPTIONS);
+    OBConversion::RegisterOptionParam("F", this, 0, OBConversion::OUTOPTIONS);
+    OBConversion::RegisterOptionParam("M", this, 0, OBConversion::OUTOPTIONS);
     OBConversion::RegisterOptionParam("X", this, 1, OBConversion::INOPTIONS);
   }
 
@@ -49,16 +57,21 @@ public:
     "InChI format\n"
     "IUPAC/NIST molecular identifier\n"
     "Write options, e.g. -xat\n"
-    " X <Option string> List of InChI options:\n"
+    " n do not use 'recommended' InChI options\n"
+    " X <Option string> List of additional InChI options\n"
+    " F include fixed hydrogen layer\n"
+    " M include bonds to metal\n"
     " t add molecule name\n"
     " a output auxilliary information\n"
+    " K output InChIKey\n"
+    " w don't warn on undef stereo or charge rearrangement\n"
+    " l display InChI log\n"
     " u output only unique molecules\n"
     " U output only unique molecules and sort them\n"
-    " e compare first molecule to others\n"
-    " w don't warn on undef stereo or charge rearrangement\n\n"
+    " e compare first molecule to others\n\n"
 
     "Input options, e.g. -at\n"
-    " X <Option string> List of InChI options:\n"
+    " X <Option string> List of InChI options\n"
     " n molecule name follows InChI on same line\n"
     " a add InChI string to molecule name\n\n"
     " The InChI options should be space delimited in a single quoted string.\n"
@@ -583,6 +596,12 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   }
   
   string ostring = inout.szInChI;
+  if(pConv->IsOption("K")) //Generate InChIKey and add after InChI on same line
+  {
+    char szINCHIKey[26];
+    GetINCHIKeyFromINCHI(inout.szInChI, szINCHIKey);
+    ostring = szINCHIKey;
+  }
   if(pConv->IsOption("t"))
   {
     ostring += ' ';
@@ -618,7 +637,11 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 
   if (pConv->IsOption("a"))
     ofs << inout.szAuxInfo << endl;
-  
+
+  if(pConv->IsOption("l"))
+    //Display InChI log message. With multiple molecules, it appears only once
+    obErrorLog.ThrowError("InChI log", inout.szLog, obError, onceOnly);
+
   if(pConv->IsOption("e"))
   {
     if(pConv->GetOutputIndex()==1)
@@ -734,7 +757,8 @@ public:
   virtual const char* Description() //required
   {
       return 
-"Compares first molecule with others using InChI\n"
+"Compare molecules using InChI\n"
+"The first molecule is compared with the rest\n"
 "e.g. babel first.smi second.mol third.cml -ok\n"
 "Same as  -oinchi -xet  and can take the same options as InChI format.\n"
 ;
@@ -769,7 +793,7 @@ public:
 "Test format\n"
 "Does a round trip conversion and compares before and after using InChI.\n"
 "Uses the input format unless -xO set\n"
-"Option e.g. -xOsmi\n"
+"Read/Write Option e.g. -xOsmi\n"
 " O<ext> Test the format that has the specified ID\n"
 " m      Output message for each successful molecule\n"
 " X <Option string> List of InChI options:\n\n"
@@ -900,25 +924,49 @@ bool TestFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 //If there are no options returns an empty string
 char* InChIFormat::GetInChIOptions(OBConversion* pConv, bool Reading)
 {
+  vector<string> optsvec; //the InChi options
+  if(!Reading && !pConv->IsOption("n"))
+    //without -xn option, the default is to write using these 'recommended' options
+    tokenize(optsvec, "FixedH RecMet SPXYZ SAsXYZ Newps Fb Fnud");
+
   char* opts;
   OBConversion::Option_type opttyp = Reading ? OBConversion::INOPTIONS : OBConversion::OUTOPTIONS;
   const char* copts = pConv->IsOption("X", opttyp);
   if(copts)
   {
-    vector<string> optsvec;
     string tmp(copts); // GCC doesn't like passing string temporaries to functions
-    tokenize(optsvec, tmp);
+    vector<string> useropts;
+    tokenize(useropts, tmp);
+    copy(useropts.begin(), useropts.end(), back_inserter(optsvec));
+  }
+
+  //Add a couple InChI options built in to OB
+  if(opttyp==OBConversion::OUTOPTIONS)
+  {
+    if(pConv->IsOption("F", opttyp))
+    {
+      string tmp2("FixedH");
+      optsvec.push_back(tmp2);
+    }
+    if(pConv->IsOption("M", opttyp))
+    {
+      string tmp2("RecMet");
+      optsvec.push_back(tmp2);
+    }
+  }
+  
+
 #ifdef WIN32
     string ch(" /");
 #else
     string ch(" -");
 #endif
-    string sopts;
-    for(unsigned int i=0;i<optsvec.size();++i)
-      sopts += ch + optsvec[i];
-    opts = new char[strlen(sopts.c_str())+1]; //has to be char, not const char
-    return strcpy(opts, sopts.c_str());
-  }
+
+  string sopts;
+  for(int i=0;i<optsvec.size();++i)
+    sopts += ch + optsvec[i];
+  opts = new char[strlen(sopts.c_str())+1]; //has to be char, not const char
+  return strcpy(opts, sopts.c_str());
   opts = new char[1];
   *opts = '\0';
   return opts;
