@@ -25,8 +25,10 @@ GNU General Public License for more details.
 #include <vector>
 #include <iomanip>
 #include <map>
+#include <algorithm>
 #include <openbabel/obmolecformat.h>
 #include <openbabel/chiral.h>
+#include <openbabel/alias.h>
 
 using namespace std;
 namespace OpenBabel
@@ -86,11 +88,11 @@ Write Options, e.g. -x3\n \
     bool ReadV3000Line(istream& ifs, vector<string>& vs);
     bool ReadAtomBlock(istream& ifs,OBMol& mol, OBConversion* pConv);
     bool ReadBondBlock(istream& ifs,OBMol& mol, OBConversion* pConv);
+    bool ReadCollectionBlock(istream& ifs,OBMol& mol, OBConversion* pConv);
     bool WriteV3000(ostream& ofs,OBMol& mol, OBConversion* pConv);
   private:
     bool  HasProperties;
     char* GetTimeDate(char* td);
-    bool  ParseAliasText(OBMol& mol, char* txt, int atomnumber);
 
     map<int,int> indexmap; //relates index in file to index in OBMol
     vector<string> vs;
@@ -154,6 +156,9 @@ public:
     string comment;
     string r1,r2;
 
+ 	  // Attempting to read past the end of the file -- don't bother
+		if( !ifs.good() || ifs.peek() == EOF ) return( false );
+
     if (!ifs.getline(buffer,BUFF_SIZE)) {
       errorMsg << "WARNING: Problems reading a MDL file\n";
       errorMsg << "Cannot read title line\n";
@@ -335,20 +340,26 @@ public:
               {
                 int i = atoi(buffer+6);
                 for(;i>0;--i)
-                  ifs.getline(buffer,BUFF_SIZE);
+                  if (ifs.good()) // check for EOL, suggested by Dalke
+                    ifs.getline(buffer,BUFF_SIZE);
                 continue;
               }
             if(buffer[0]=='A') //alias
             {
-              //Parses entries like 
-              //A  6
-              //CD2
               int atomnum = atoi(buffer+2);
               ifs.getline(buffer,BUFF_SIZE);
-              if(!ParseAliasText(mol, buffer, atomnum))
+              if(*buffer!='?' && *buffer!='*')
               {
-                obErrorLog.ThrowError(__FUNCTION__, "Error in alias block", obError);
-                return false;
+                AliasData* ad = new AliasData();
+                ad->SetAlias(buffer);
+                ad->SetOrigin(fileformatInput);
+                OBAtom* at = mol.GetAtom(atomnum);
+                if (at) {
+                  at->SetData(ad);
+                  at->SetAtomicNum(0);
+                  //The alias has now been added as a dummy atom with a AliasData object.
+                  ad->Expand(mol, atomnum); //Make chemically meaningful, if possible.
+                }
               }
               continue;
             }
@@ -356,13 +367,22 @@ public:
               continue;
             r1 = buffer;
             int n = atoi((r1.substr(6,3)).c_str()); //entries on this line
-            if(n==0) break;
+            if(n<=0 || n>8 || 6+n*8>r1.size()) //catch ill-formed line
+            {
+              obErrorLog.ThrowError(__FUNCTION__, "Error in line:\n" + r1, obError);
+                return false;
+            }
             int pos = 10;
             for(;n>0;n--,pos+=8)
               {
                 int atomnumber = atoi((r1.substr(pos,3)).c_str());
-                if (atomnumber==0) break;
                 OBAtom* at;
+                if (atomnumber==0 || (at=mol.GetAtom(atomnumber+offset))==NULL)
+                {
+                  obErrorLog.ThrowError(__FUNCTION__, "Error in line:\n" + r1, obError);
+                    return false;
+                }
+
                 at=mol.GetAtom(atomnumber+offset); //atom numbers start at 1
                 int value = atoi((r1.substr(pos+4,3)).c_str());
                 if(r1.substr(3,3)=="RAD")
@@ -522,9 +542,9 @@ public:
 
         OBAtom *atom;
         vector<OBAtom*>::iterator i;
-        int charge;
+        int charge=0; //Charge is now output in a M CHG statement
         for (atom = mol.BeginAtom(i);atom;atom = mol.NextAtom(i)) {
-          switch (atom->GetFormalCharge()) {
+/*          switch (atom->GetFormalCharge()) {
           case 1: charge = 3; break;
           case 2: charge = 2; break;
           case 3: charge = 1; break;
@@ -534,12 +554,12 @@ public:
           default:
             charge=0; break;
           }
-
+*/
           snprintf(buff, BUFF_SIZE, "%10.4f%10.4f%10.4f %-3s%2d%3d%3d%3d%3d",
                    atom->GetX(),
                    atom->GetY(),
                    atom->GetZ(),
-                   (etab.GetSymbol(atom->GetAtomicNum())),
+                   atom->GetAtomicNum() ? etab.GetSymbol(atom->GetAtomicNum()) : "* ",
                    0,charge,0,0,0);    
           ofs << buff << endl;
         }
@@ -568,7 +588,7 @@ public:
               ofs << buff << endl;
             }
 
-        vector<OBAtom*> rads, isos;
+        vector<OBAtom*> rads, isos, chgs;
         vector<OBAtom*>::iterator itr;
         for (atom = mol.BeginAtom(i);atom;atom = mol.NextAtom(i))
           {
@@ -576,6 +596,16 @@ public:
               rads.push_back(atom);
             if(atom->GetIsotope())
               isos.push_back(atom);
+            if(atom->GetFormalCharge())
+              chgs.push_back(atom);
+
+            if(atom->HasData(AliasDataType))
+            {
+              AliasData* ad = static_cast<AliasData*>(atom->GetData(AliasDataType));
+              if(!ad->IsExpanded()) //do nothing with an expanded alias
+                ofs << "A  " << atom->GetIdx() << '\n' << ad->GetAlias() << endl;
+            }
+
           }
         if(rads.size())
           {
@@ -591,6 +621,18 @@ public:
               ofs << setw(4) << (*itr)->GetIdx() << setw(4) << (*itr)->GetIsotope();
             ofs << endl;
           }                     
+        if(chgs.size())
+          {
+            int counter = 0;
+            for( itr=chgs.begin(); itr != chgs.end(); ++itr, counter++ ) {
+              if( counter % 8 == 0 ) {
+                if( counter > 0 ) ofs << endl;
+                ofs << "M  CHG" << setw(3) << min(static_cast<unsigned long int>(chgs.size() - counter), static_cast<unsigned long int>(8));
+              }
+              ofs << setw(4) << (*itr)->GetIdx() << setw(4) << (*itr)->GetFormalCharge();
+            }
+            ofs << endl;
+          }
       }
 
     ofs << "M  END" << endl;
@@ -605,8 +647,13 @@ public:
             if ((*k)->GetDataType() == OBGenericDataType::PairData)
               {
                 HasProperties = true;
-                ofs << ">  <" << (*k)->GetAttribute() << ">" << endl;
-                ofs << ((OBPairData*)(*k))->GetValue() << endl << endl;
+                //Since partial charges are not output
+                //in this format, don't need the annotation
+                if((*k)->GetAttribute()!="PartialCharges")
+                {
+                  ofs << ">  <" << (*k)->GetAttribute() << ">" << endl;
+                  ofs << ((OBPairData*)(*k))->GetValue() << endl << endl;
+                }
               }
           }
       }
@@ -648,16 +695,17 @@ public:
           ReadAtomBlock(ifs,mol,pConv);
         else if(vs[3]=="BOND")
           ReadBondBlock(ifs,mol,pConv);
+        else if(vs[3]=="COLLECTION")
+          ReadCollectionBlock(ifs,mol,pConv);
+          
         /*
-          else if(vs[3]=="COLLECTION")
-          //not currently implemented
           else if(vs[3]=="3D")
           //not currently implemented
           else if(vs[3]=="SGROUP")
           //not currently implemented
           else if(vs[3]=="RGROUP")
           //not currently implemented
-          */
+        */
       }while(DoMany && ifs.good());
     //  if(is3D){mol.SetDimension(3);cout<<"SetDim to 3"<<endl;}
     //  else if(is2D){mol.SetDimension(2);cout<<"SetDim to 2"<<endl;}
@@ -670,6 +718,7 @@ public:
     char buffer[BUFF_SIZE];
     if(!ifs.getline(buffer,BUFF_SIZE)) return false;
     tokenize(vs,buffer," \t\n\r");
+    if (vs.size() < 2) return false; // timvdm 18/06/2008
     if(vs[0]!="M" || (vs[1]!="V30" && vs[1]!="END")) return false;
         
     if(buffer[strlen(buffer)-1] == '-') //continuation char
@@ -807,6 +856,22 @@ public:
     return true;
   }
 
+////////////////////////////////////////////////////////////
+  bool MDLFormat::ReadCollectionBlock(istream& ifs,OBMol& mol, OBConversion* pConv)
+  {
+    //Not currently implemented
+    obErrorLog.ThrowError(__FUNCTION__, 
+      "COLLECTION blocks are not currently implemented and their contents ae ignored.", obWarning);
+    for(;;)
+    {
+      if(!ReadV3000Line(ifs,vs))
+        return false;
+      if(vs[2]=="END")
+        break;
+    }
+    return true;
+  }
+
   //////////////////////////////////////////////////////////
   bool MDLFormat::WriteV3000(ostream& ofs,OBMol& mol, OBConversion* pConv)
   {
@@ -944,68 +1009,5 @@ public:
     return td;
   }
 
-  bool MDLFormat::ParseAliasText(OBMol& mol, char* txt, int atomnumber)
-  {
-    //Crude implementation
-    //Only single character element symbols are handled
-    //Atom which replaces atomnumber is the first non-H 
-    //Will parse ND2 DS CH-
-    if(!isalpha(*txt)) //first char is the element that replaces atomnumber
-      return false;
-    //Swaps any leading H isotope with the first non-H atom
-    if(*txt=='H' || *txt=='D' || *txt=='T')
-    {
-      char* p =txt+1;
-      while(*p && *p=='H' && *p=='D' && *p=='T')p++;
-      if(*p)
-        std::swap(*p, *txt);
-    }
-    char symb[2];
-    symb[0]=*(txt++);
-    symb[1]='\0';
-    OBAtom* pAtom = mol.GetAtom(atomnumber);
-    if(!pAtom)
-      return false;
-    int iso = 0;
-    pAtom->SetAtomicNum(etab.GetAtomicNum(symb,iso));
-    if(iso)
-      pAtom->SetIsotope(iso);
-
-    while(*txt)
-    {
-      if(isspace(*txt)) {
-        ++txt;
-        continue;
-      }
-      int chg=0;
-      if(*txt=='-')
-        chg = -1;
-      else if(*txt=='+')
-        chg = 1;
-      if(chg)
-      {
-        pAtom->SetFormalCharge(pAtom->GetFormalCharge()+chg);//put on central atom e.g. CH-
-        ++txt;
-        continue;
-      }
-      if(!isalpha(*txt))
-        return false;
-      symb[0]=*txt;
-      int rep = atoi(++txt);
-      if(rep)
-        ++txt;
-      do //for each rep
-      {
-        OBAtom* newAtom = mol.NewAtom();
-        iso = 0;
-        newAtom->SetAtomicNum(etab.GetAtomicNum(symb,iso));
-        if(iso)
-          newAtom->SetIsotope(iso);
-
-        if (!mol.AddBond(atomnumber,mol.NumAtoms(),1,0)) return false;
-      }while(--rep>0);
-    }
-    return true;
-  }
 
 }
