@@ -116,14 +116,20 @@ namespace OpenBabel
         else if (lcount == 4) //energy
           {
             tokenize(vstr,buffer);
-            if (!vstr.empty() && vstr[0] == "Energy" && vstr.size() == 3)
-              mol.SetEnergy(atof(vstr[2].c_str()));
+            if (!vstr.empty() && vstr.size() == 3)
+              if (vstr[0] == "Energy")
+                mol.SetEnergy(atof(vstr[2].c_str()));
           }
         else if (lcount == 5) //comment
           {
             if ( buffer[0] )
               {
                 len = (int) strlen(buffer)+1;
+                // TODO allow better multi-line comments
+                // which don't allow ill-formed data to consume memory
+                // Thanks to Andrew Dalke for the pointer
+                if (comment != NULL)
+                  delete [] comment;
                 comment = new char [len];
                 memcpy(comment,buffer,len);
               }
@@ -168,12 +174,18 @@ namespace OpenBabel
           str = "S.O2";
         } else if (strncmp(temp_type,"S.o", 3) == 0) {
           str = "S.O";
-        } else if (strncmp(temp_type,"S.1", 3) == 0) {
-          str = "S.2"; // no idea what the best type might be here
-        } else if (strncmp(temp_type,"P", 1) == 0) {
-          str = "P.3";
         } else if (strncmp(temp_type,"SI", 2) == 0) {
           str = "Si";
+        // The following cases are entries which are not in openbabel/data/types.txt
+        // and should probably be added there
+        } else if (strncmp(temp_type,"S.1", 3) == 0) {
+          str = "S.2"; // no idea what the best type might be here
+        } else if (strncmp(temp_type,"P.", 2) == 0) {
+          str = "P.3";
+        } else if (strncasecmp(temp_type,"Ti.", 3) == 0) { // e.g. Ti.th
+          str = "Ti";
+        } else if (strncasecmp(temp_type,"Ru.", 3) == 0) { // e.g. Ru.oh
+          str = "Ru";
         }
 
         ttab.SetToType("ATN");
@@ -288,6 +300,9 @@ namespace OpenBabel
           }
       }
 
+    // Suggestion by Liu Zhiguo 2008-01-26
+    // Mol2 files define atom types -- there is no need to re-perceive
+    mol.SetAtomTypesPerceived();
     mol.EndModify();
 
     //must add generic data after end modify - otherwise it will be blown away
@@ -329,7 +344,7 @@ namespace OpenBabel
     //Define some references so we can use the old parameter names
     ostream &ofs = *pConv->GetOutStream();
     OBMol &mol = *pmol;
-    bool ligandsOnly = pConv->IsOption("l", OBConversion::OUTOPTIONS);
+    bool ligandsOnly = pConv->IsOption("l", OBConversion::OUTOPTIONS)!=NULL;
  
     //The old code follows....
     string str,str1;
@@ -436,12 +451,14 @@ namespace OpenBabel
           strcpy(label,"am");
         else
           snprintf(label,BUFF_SIZE,"%d",bond->GetBO());
+
         snprintf(buffer, BUFF_SIZE,"%6d%6d%6d%3s%2s",
                  bond->GetIdx()+1,bond->GetBeginAtomIdx(),bond->GetEndAtomIdx(),
                  "",label);
         ofs << buffer << endl;
       }
-    ofs << endl;
+    // NO trailing blank line (PR#1868929).
+    //    ofs << endl;
 
     return(true);
   }
