@@ -45,21 +45,15 @@ namespace OpenBabel
     };
 
     virtual const char* SpecificationURL()
-    { return "http://www.rcsb.org/pdb/docs/format/pdbguide2.2/guide2.2_frame.html";};
+    { return "http://www.wwpdb.org/docs.html";};
 
     virtual const char* GetMIMEType() 
     { return "chemical/x-pdb"; };
 
-    //Flags() can return be any the following combined by | or be omitted if none apply
-    // NOTREADABLE  READONEONLY  NOTWRITABLE  WRITEONEONLY
-    virtual unsigned int Flags()
-    {
-      return READONEONLY;
-    };
-
     //*** This section identical for most OBMol conversions ***
     ////////////////////////////////////////////////////
     /// The "API" interface functions
+  	virtual int SkipObjects(int n, OBConversion* pConv);
     virtual bool ReadMolecule(OBBase* pOb, OBConversion* pConv);
     virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
 
@@ -71,12 +65,27 @@ namespace OpenBabel
 
   /////////////////////////////////////////////////////////////////
 
-
-  static bool ParseAtomRecord(char *, OBMol &,int);
-  static bool ParseConectRecord(char *,OBMol &);
+  // Now in atomrecord.cpp (shared with PQR format)
+  bool ParseAtomRecord(char *, OBMol &,int);
+  bool ParseConectRecord(char *,OBMol &);
 
   //extern OBResidueData    resdat; now in mol.h
 
+  /////////////////////////////////////////////////////////////////
+ 	int PDBFormat::SkipObjects(int n, OBConversion* pConv)
+  {
+    if (n == 0)
+      ++ n;
+    istream &ifs = *pConv->GetInStream();
+    char buffer[BUFF_SIZE];
+    while (n && ifs.getline(buffer,BUFF_SIZE))
+      {
+        if (EQn(buffer,"ENDMDL",6))
+          -- n;
+      }
+      
+    return ifs.good() ? 1 : -1;       
+  }
   /////////////////////////////////////////////////////////////////
   bool PDBFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
   {
@@ -100,8 +109,15 @@ namespace OpenBabel
     mol.SetChainsPerceived(); // It's a PDB file, we read all chain/res info.
 
     mol.BeginModify();
-    while (ifs.getline(buffer,BUFF_SIZE) && !EQn(buffer,"END",3))
+    while (ifs.good() && ifs.getline(buffer,BUFF_SIZE))
       {
+        if (EQn(buffer,"ENDMDL",6))
+          break;
+        if (EQn(buffer,"END",3)) {
+          // eat anything until the next ENDMDL
+          while (ifs.getline(buffer,BUFF_SIZE) && !EQn(buffer,"ENDMDL",6));
+          break;
+        }
         if (EQn(buffer,"TER",3)) {
           chainNum++;
           continue;
@@ -121,7 +137,15 @@ namespace OpenBabel
 
         // another record type, add it as an OBPairData entry
         line = buffer;
-        key = line.substr(0,6); // the first 6 characters are the record name
+        // if the file is valid, all lines should have more than 6 characters
+        if (line.length() < 6)
+		  {
+            stringstream errorMsg;
+            errorMsg << "ERROR: not a valid PDB file" << endl;
+            obErrorLog.ThrowError(__FUNCTION__, errorMsg.str() , obError);
+            return false;
+          }
+       key = line.substr(0,6); // the first 6 characters are the record name
         Trim(key);
         value = line.substr(6);
         
@@ -152,6 +176,10 @@ namespace OpenBabel
     /*assign hetatm bonds based on distance*/
 
     mol.EndModify();
+    // Clear all virtual bond data
+    vector<OBGenericData*> vbonds = mol.GetAllData(OBGenericDataType::VirtualBondData);
+    mol.DeleteData(vbonds);
+
 
     if (!pConv->IsOption("b",OBConversion::INOPTIONS))
       mol.ConnectTheDots();
@@ -167,200 +195,6 @@ namespace OpenBabel
     return(true);
   }
 
-  ////////////////////////////////////////////////////////////////
-  static bool ParseAtomRecord(char *buffer, OBMol &mol,int chainNum)
-  /* ATOMFORMAT "(i5,1x,a4,a1,a3,1x,a1,i4,a1,3x,3f8.3,2f6.2,1x,i3)" */
-  {
-    string sbuf = &buffer[6];
-    if (sbuf.size() < 48)
-      return(false);
-
-    bool hetatm = (EQn(buffer,"HETATM",6)) ? true : false;
-
-    /* serial number */
-    string serno = sbuf.substr(0,5);
-    //SerialNum(the_atom) = atoi(tmp_str);
-
-    /* atom name */
-    string atmid = sbuf.substr(6,4);
-
-    /* element */
-    string element;
-    if (sbuf.size() > 71)
-      element = sbuf.substr(70,2);
-    else
-      element = "  ";
-
-    //trim spaces on the right and left sides
-    while (!atmid.empty() && atmid[0] == ' ')
-      atmid = atmid.substr(1,atmid.size()-1);
-
-    while (!atmid.empty() && atmid[atmid.size()-1] == ' ')
-      atmid = atmid.substr(0,atmid.size()-1);
-
-    /* residue name */
-
-    string resname = sbuf.substr(11,3);
-    if (resname == "   ")
-      resname = "UNK";
-    else
-      {
-        while (!resname.empty() && resname[0] == ' ')
-          resname = resname.substr(1,resname.size()-1);
-
-        while (!resname.empty() && resname[resname.size()-1] == ' ')
-          resname = resname.substr(0,resname.size()-1);
-      }
-
-    string type;
-    if (EQn(buffer,"ATOM",4))
-      {
-        type = atmid.substr(0,2);
-        if (isdigit(type[0])) {
-          // sometimes non-standard files have, e.g 11HH
-          if (!isdigit(type[1])) type = atmid.substr(1,1);
-          else type = atmid.substr(2,1); 
-        } else if (sbuf[6] == ' ' &&
-                   strncasecmp(type.c_str(), "Zn", 2) != 0 &&
-                   strncasecmp(type.c_str(), "Fe", 2) != 0)
-          type = atmid.substr(0,1);     // one-character element
-        
-
-        if (resname.substr(0,2) == "AS" || resname[0] == 'N')
-          {
-            if (atmid == "AD1")
-              type = "O";
-            if (atmid == "AD2")
-              type = "N";
-          }
-        if (resname.substr(0,3) == "HIS" || resname[0] == 'H')
-          {
-            if (atmid == "AD1" || atmid == "AE2")
-              type = "N";
-            if (atmid == "AE1" || atmid == "AD2")
-              type = "C";
-          }
-        if (resname.substr(0,2) == "GL" || resname[0] == 'Q')
-          {
-            if (atmid == "AE1")
-              type = "O";
-            if (atmid == "AE2")
-              type = "N";
-          }
-      }
-    else //must be hetatm record
-      {
-        if (isalpha(element[1]) && (isalpha(element[0]) || (element[0] == ' ')))
-          {
-            if (isalpha(element[0]))
-              type = element.substr(0,2);
-            else
-              type = element.substr(1,1);
-            if (type.size() == 2)
-              type[1] = tolower(type[1]);
-          }
-        else
-          {
-            if (isalpha(atmid[0])) {
-              
-              if (atmid.size() > 2 && (atmid[2] == '\0' || atmid[2] == ' '))
-                type = atmid.substr(0,2);
-              else if (atmid[0] == 'A') // alpha prefix
-                type = atmid.substr(1, atmid.size() - 1);
-              else
-                type = atmid.substr(0,1);
-            }
-            else if (atmid[0] == ' ')
-              type = atmid.substr(1,1); // one char element
-            else
-              type = atmid.substr(1,2);
-
-            // Some cleanup steps
-            if (atmid == resname)
-              {
-                type = atmid;
-                if (type.size() == 2)
-                  type[1] = tolower(type[1]);
-              }
-            else
-              if (resname == "ADR" || resname == "COA" || resname == "FAD" ||
-                  resname == "GPG" || resname == "NAD" || resname == "NAL" ||
-                  resname == "NDP" || resname == "ABA")
-                {
-                  if (type.size() > 1)
-                    type = type.substr(0,1);
-                  //type.erase(1,type.size()-1);
-                }
-              else
-                if (isdigit(type[0]))
-                  {
-                    type = type.substr(1,1);
-                  }
-                else
-                  if (type.size() > 1 && isdigit(type[1]))
-                    type = type.substr(0,1);
-                  else
-                    if (type.size() > 1 && isalpha(type[1])) {
-                      if (type[0] == 'O' && type[1] == 'H')
-                        type = type.substr(0,1); // no "Oh" element (e.g. 1MBN)
-                      else if(isupper(type[1]))
-                        {
-                          type[1] = tolower(type[1]);
-                        }
-                    }
-          }
-        
-      }
-
-    OBAtom atom;
-    /* X, Y, Z */
-    string xstr = sbuf.substr(24,8);
-    string ystr = sbuf.substr(32,8);
-    string zstr = sbuf.substr(40,8);
-    vector3 v(atof(xstr.c_str()),atof(ystr.c_str()),atof(zstr.c_str()));
-    atom.SetVector(v);
-
-    // useful for debugging unknown atom types (e.g., PR#1577238)
-    //    cout << mol.NumAtoms() + 1 << " " << atmid << " type: " << type << endl;
-    atom.SetAtomicNum(etab.GetAtomicNum(type.c_str()));
-
-    /* residue sequence number */
-    string resnum = sbuf.substr(16,4);
-    int        rnum = atoi(resnum.c_str());
-    OBResidue *res  = (mol.NumResidues() > 0) ? mol.GetResidue(mol.NumResidues()-1) : NULL;
-    if (res == NULL || res->GetName() != resname || static_cast<int>(res->GetNum())
-        != rnum)
-      {
-        vector<OBResidue*>::iterator ri;
-        for (res = mol.BeginResidue(ri) ; res ; res = mol.NextResidue(ri))
-          if (res->GetName() == resname && static_cast<int>(res->GetNum())
-              == rnum)
-            break;
-
-        if (res == NULL)
-          {
-            res = mol.NewResidue();
-            res->SetChainNum(chainNum);
-            res->SetName(resname);
-            res->SetNum(rnum);
-          }
-      }
-
-    if (!mol.AddAtom(atom))
-      return(false);
-    else
-      {
-        OBAtom *atom = mol.GetAtom(mol.NumAtoms());
-
-        res->AddAtom(atom);
-        res->SetSerialNum(atom, atoi(serno.c_str()));
-        res->SetAtomID(atom, atmid);
-        res->SetHetAtom(atom, hetatm);
-
-        return(true);
-      }
-  }
-
   /////////////////////////////////////////////////////////////////////////
   //! Utility function to read a 5-digit integer starting from a specified column
   /*! This function reads a 5-digit integer, starting from column
@@ -424,7 +258,7 @@ namespace OpenBabel
     Hydrogen bonds and salt bridges are ignored. --Stefan Kebekus.
   */
 
-  static bool ParseConectRecord(char *buffer,OBMol &mol)
+  bool ParseConectRecord(char *buffer,OBMol &mol)
   {
     stringstream errorMsg;
     string clearError;
@@ -526,8 +360,8 @@ namespace OpenBabel
         if (vs.size() <= 4)
           limit = vs.size() - 1;
 
-        for (unsigned int i = 0; i < limit; i++)
-          boundedAtomsSerialNumbersValid[i] = true;
+        for (unsigned int s = 0; s < limit; ++s)
+          boundedAtomsSerialNumbersValid[s] = true;
       }
     else
       {
@@ -601,9 +435,17 @@ namespace OpenBabel
     char buffer[BUFF_SIZE];
     char type_name[10], padded_name[10];
     char the_res[10];
-    char element_name[4];
+    char the_chain = ' ';
+    const char *element_name;
     int res_num;
     bool het=true;
+    int model_num = 0;
+    if (!pConv->IsLast() || pConv->GetOutputIndex() > 1)
+      { // More than one molecule record
+      model_num = pConv->GetOutputIndex(); // MODEL 1-based index
+      snprintf(buffer, BUFF_SIZE, "MODEL %8d", model_num);
+      ofs << buffer << endl;
+      }
 
     if (strlen(mol.GetTitle()) > 0)
       snprintf(buffer, BUFF_SIZE, "COMPND    %s ",mol.GetTitle());
@@ -658,11 +500,12 @@ namespace OpenBabel
             snprintf(type_name, sizeof(type_name), " %-3s", tmp);
           }
 
-        if ( (res = atom->GetResidue()) )
+        if ( (res = atom->GetResidue()) != 0 )
           {
             het = res->IsHetAtom(atom);
             snprintf(the_res,4,"%s",(char*)res->GetName().c_str());
             snprintf(type_name,5,"%s",(char*)res->GetAtomID(atom).c_str());
+	    the_chain = res->GetChain();
 
             //two char. elements are on position 13 and 14 one char. start at 14
             if (strlen(etab.GetSymbol(atom->GetAtomicNum())) == 1)
@@ -677,11 +520,13 @@ namespace OpenBabel
                   }
                 else
                   {
-                    type_name[4] = type_name[3];
+                    /*
+		    type_name[4] = type_name[3];
                     type_name[3] = type_name[2];
                     type_name[2] = type_name[1];
                     type_name[1] = type_name[0];
                     type_name[0] = type_name[4];
+		    */
                     type_name[4] = '\0';
                   }
               }
@@ -696,14 +541,13 @@ namespace OpenBabel
             res_num = 1;
           }
 
-        strncpy (element_name,etab.GetSymbol(atom->GetAtomicNum()),sizeof(element_name));
-        if (strlen(element_name) == 2)
-          element_name[1] = toupper(element_name[1]);
-        snprintf(buffer, BUFF_SIZE, "%s%5d %-4s %-3s  %4d    %8.3f%8.3f%8.3f  1.00  0.00          %2s  \n",
+        element_name = etab.GetSymbol(atom->GetAtomicNum());
+        snprintf(buffer, BUFF_SIZE, "%s%5d %-4s %-3s %c%4d    %8.3f%8.3f%8.3f  1.00  0.00          %2s  \n",
                  het?"HETATM":"ATOM  ",
                  i,
                  type_name,
                  the_res,
+		 the_chain,
                  res_num,
                  atom->GetX(),
                  atom->GetY(),
@@ -713,33 +557,49 @@ namespace OpenBabel
       }
 
     OBAtom *nbr;
-    int count;
     vector<OBBond*>::iterator k;
     for (i = 1; i <= mol.NumAtoms(); i ++)
       {
         atom = mol.GetAtom(i);
-        if (atom->GetValence() <= 4)
+        if (atom->GetValence() == 0)
+          continue; // no need to write a CONECT record -- no bonds
+
+        snprintf(buffer, BUFF_SIZE, "CONECT%5d", i);
+        ofs << buffer;
+        // Write out up to 4 real bonds per line PR#1711154
+        int currentValence = 0;
+        for (nbr = atom->BeginNbrAtom(k);nbr;nbr = atom->NextNbrAtom(k))
           {
-            snprintf(buffer, BUFF_SIZE, "CONECT%5d", i);
+            snprintf(buffer, BUFF_SIZE, "%5d", nbr->GetIdx());
             ofs << buffer;
-            for (nbr = atom->BeginNbrAtom(k);nbr;nbr = atom->NextNbrAtom(k))
-              {
-                snprintf(buffer, BUFF_SIZE, "%5d", nbr->GetIdx());
-                ofs << buffer;
-              }
-            for (count = 0; count < (4 - (int)atom->GetValence()); count++)
-              {
-                snprintf(buffer, BUFF_SIZE, "     ");
-                ofs << buffer;
-              }
-            ofs << "                                       " << endl;
+            if (++currentValence % 4 == 0) {
+              // Add the trailing space to finish this record
+              ofs << "                                       \n";
+              // write the start of a new CONECT record
+              snprintf(buffer, BUFF_SIZE, "CONECT%5d", i);
+              ofs << buffer;              
+            }
           }
+
+        // Add trailing spaces
+        int remainingValence = atom->GetValence() % 4;
+        for (int count = 0; count < (4 - remainingValence); count++) {
+          snprintf(buffer, BUFF_SIZE, "     ");
+          ofs << buffer;
+        }
+        ofs << "                                       \n";
       }
+
     snprintf(buffer, BUFF_SIZE, "MASTER        0    0    0    0    0    0    0    0 ");
     ofs << buffer;
     snprintf(buffer, BUFF_SIZE, "%4d    0 %4d    0\n",mol.NumAtoms(),mol.NumAtoms());
     ofs << buffer;
+
     ofs << "END\n";
+    if (model_num) {
+      ofs << "ENDMDL" << endl;
+    }
+
     return(true);
   }
 
