@@ -18,11 +18,22 @@ GNU General Public License for more details.
 
 #include <openbabel/math/matrix3x3.h>
 #include <openbabel/kinetics.h>
+#include <openbabel/atomclass.h>
 #include <openbabel/xml.h>
+#include <float.h>
+#ifdef HAVE_SHARED_POINTER
+  #include <openbabel/reaction.h>
+#endif
+
 
 #ifdef WIN32
 #pragma warning (disable : 4800)
 #endif
+#ifdef HAVE_SSTREAM
+#include <sstream>
+#else
+#include <strstream>
+#endif
 
 using namespace std;
 namespace OpenBabel
@@ -85,6 +96,7 @@ namespace OpenBabel
       return READXML | ZEROATOMSOK;
     };
 
+    virtual bool WriteChemObject(OBConversion* pConv);
     virtual bool WriteMolecule(OBBase* pOb, OBConversion* pConv);
   protected:
     virtual bool DoElement(const string& name);
@@ -100,14 +112,18 @@ namespace OpenBabel
     bool ParseFormula(string& formula, OBMol* pmol);
     void ReadNasaThermo();
 
+    void MakeAtomIds(OBMol& mol, vector<string>& atomIDs);
     void WriteFormula(OBMol mol); //passes copy of mol
     void WriteMetadataList();
     string getTimestr();
-    void WriteBondStereo(OBBond* pbond);
+    void WriteBondStereo(OBBond* pbond, vector<string>& atomIDs);
     void WriteCrystal(OBMol& mol);
     void WriteProperties(OBMol& mol, bool& propertyListWritten);
     void WriteThermo(OBMol& mol, bool& propertyListWritten);
     string GetMolID();//for error mesaages
+    bool WriteInChI(OBMol& mol);
+    bool WriteVibrationData(OBMol& mol);
+    bool WriteRotationData(OBMol& mol);
 
   private:
     map<string,int> AtomMap; //key=atom id, value= ob atom index
@@ -116,12 +132,15 @@ namespace OpenBabel
     vector< pair<string,string> > cmlBondOrAtom; //for cml1 only
     vector< pair<string,string> > molWideData;
     bool inBondArray; //for cml1 only
+    bool inFormula;
     string RawFormula;
     xmlChar* prefix;
     string CurrentAtomID;
-    int CrystalScalarsNeeded, PropertyScalarsNeeded;
+    int CrystalScalarsNeeded, PropertyScalarsNeeded, TransformsNeeded;
     vector<double> CrystalVals;
     OBUnitCell* pUnitCell;
+    SpaceGroup _SpaceGroup;
+    string SpaceGroupName;
     string titleonproperty;
   };
 
@@ -169,6 +188,7 @@ namespace OpenBabel
         AtomArray.clear();
         BondArray.clear();
         inBondArray = false;
+        inFormula = false;
         RawFormula.erase();
         molWideData.clear();
         CrystalScalarsNeeded=0;
@@ -188,11 +208,21 @@ namespace OpenBabel
           ptitle  = xmlTextReaderGetAttribute(reader(), BAD_CAST "molID");//Marvin
         if(ptitle)
           _pmol->SetTitle((const char*)ptitle);
+
+        ptitle = xmlTextReaderGetAttribute(reader(), BAD_CAST "spinMultiplicity");
+        if(ptitle)
+          _pmol->SetTotalSpinMultiplicity(atoi((const char*)ptitle));
+
+        // free((void*)ptitle);//libxml2 doc says "The string must be deallocated by the caller."
+
       }
     else if(name=="atomArray")
       {
-        inBondArray=false;
-        TransferArray(AtomArray);
+        if(!inFormula) //do nothing when a child of <formula>
+        {
+          inBondArray=false;
+          TransferArray(AtomArray);
+        }
       }
     else if(name=="bondArray")
       {
@@ -233,10 +263,15 @@ namespace OpenBabel
       }
     else if(name=="formula")
       {
+        if(!xmlTextReaderIsEmptyElement(reader()))
+          inFormula=true;
         //Only concise form is currently supported
         const xmlChar* pformula = xmlTextReaderGetAttribute(reader(), BAD_CAST "concise");
         if(pformula)
+        {
           RawFormula = (const char*)pformula;
+          // free((void*)pformula);
+        }
       }
     else if(name=="crystal")
       {
@@ -267,15 +302,20 @@ namespace OpenBabel
             const xmlChar* pattr  = xmlTextReaderGetAttribute(reader(), BAD_CAST "title");
             if(!pattr)
               pattr  = xmlTextReaderGetAttribute(reader(), BAD_CAST "id");
-            if(!pattr)
-              pattr = BAD_CAST titleonproperty.c_str();
+            
+            string attr;
+            if(pattr)
+              attr = (const char*)pattr;
+            else
+              attr = titleonproperty;
+            // free((void*)pattr);//"The string must be deallocated by the caller."
 
             xmlTextReaderRead(reader());
             const xmlChar* pvalue = xmlTextReaderConstValue(reader());
-            if(pvalue && pattr)
+            if(pvalue && !attr.empty())
               {
                 OBPairData *dp = new OBPairData;
-                dp->SetAttribute((const char*)pattr);
+                dp->SetAttribute(attr);
                 string val((const char*)pvalue);
                 dp->SetValue(Trim(val));
                 dp->SetOrigin(fileformatInput);
@@ -284,8 +324,29 @@ namespace OpenBabel
             PropertyScalarsNeeded=0;
           }
       }
-    else if(name=="property")
+    else if(name=="symmetry")
       {
+        const xmlChar* pname  = xmlTextReaderGetAttribute(reader(), BAD_CAST "spaceGroup");
+        if (pname)
+		  {
+            SpaceGroupName = (const char*)pname;
+            // free((void*)pname);
+          }
+      }
+     else if(name=="transform3")
+      {
+        xmlTextReaderRead(reader());
+        const xmlChar* ptransform = xmlTextReaderConstValue(reader());
+        if (ptransform)
+		  {
+            string t = (const char*)ptransform;
+            _SpaceGroup.AddTransform(t);
+            // free((void*)ptransform);
+         }
+      }
+   else if(name=="property")
+      {
+        //***pattr need to be deleted***
         const char* pattr  = (const char*)xmlTextReaderGetAttribute(reader(), BAD_CAST "dictRef");
         if(pattr && !strcmp(pattr,"Thermo_OldNasa"))
           ReadNasaThermo();
@@ -311,8 +372,8 @@ namespace OpenBabel
           return false;
         string value = (const char*)pvalue;
         Trim(value);
-        pair<string,string> nameAndvalue(name,value);					
-        cmlBondOrAtom.push_back(nameAndvalue);			
+        pair<string,string> nameAndvalue(name,value);
+        cmlBondOrAtom.push_back(nameAndvalue);
       }
     else	if(name=="stringArray" || name=="floatArray" || name=="integerArray")
       {
@@ -334,7 +395,7 @@ namespace OpenBabel
         unsigned int i;
         for(i=0;i<items.size();++i)
           {				
-            pair<string,string> nameAndvalue(name,items[i]);					
+            pair<string,string> nameAndvalue(name,items[i]);
             arr[i].push_back(nameAndvalue);
           }
       }
@@ -358,6 +419,8 @@ namespace OpenBabel
       {
         BondArray.push_back(cmlBondOrAtom);
       }
+    else if(name=="formula")
+      inFormula=false;
     else if(name=="molecule")
       {
         DoAtoms();
@@ -379,6 +442,16 @@ namespace OpenBabel
         return (--_embedlevel>=0); //false to stop parsing if no further embedded mols
         //		return false;//means stop parsing
       }
+     else if(name=="symmetry")
+      {
+        const SpaceGroup *group = SpaceGroup::GetSpaceGroup(SpaceGroupName);
+        if ((!group || !(_SpaceGroup == *group)) && _SpaceGroup.IsValid())
+          group = SpaceGroup::Find(&_SpaceGroup);
+        if (group)
+          pUnitCell->SetSpaceGroup(group);
+        else
+          pUnitCell->SetSpaceGroup(SpaceGroupName);
+      }
     return true;
   }
 
@@ -386,7 +459,8 @@ namespace OpenBabel
 
   ///Interprets atoms from AtomArray and writes then to an OBMol
   bool CMLFormat::DoAtoms()
-  {	
+  {
+    OBAtomClassData aclass;
     int dim=0; //dimension of molecule
     bool use2d = _pxmlConv->IsOption("2", OBConversion::INOPTIONS);
 
@@ -400,18 +474,25 @@ namespace OpenBabel
         int nhvy = nAtoms;
 
         double x=0,y=0,z=0;
-		
+        bool using3=false, using2=false, usingFract=false;
+
         vector<pair<string,string> >::iterator AttributeIter;
         for(AttributeIter=AtomIter->begin();AttributeIter!=AtomIter->end();++AttributeIter)
           {
-            bool datanotused=false;
             string& attrname = AttributeIter->first;
             string& value    = AttributeIter->second;
 
             if(attrname=="id" || attrname=="atomId" || attrname=="atomID")//which one correct? 
               {
                 Trim(value);
+                if(AtomMap.count(value)>0)
+                  obErrorLog.ThrowError(GetMolID(),"The atom id " + value + " is not unique", obWarning);
                 AtomMap[value] = nhvy;//nAtoms;
+
+                //If the id begins with "aa", "ab", etc, the number that follows  is taken as an atom class
+                if(value[0]=='a' && value[1]>='a' && value[1]<='z')
+                  aclass.Add(nAtoms, atoi(value.c_str()+2));
+                continue;
               }
             else if(attrname=="elementType")
               {
@@ -420,32 +501,35 @@ namespace OpenBabel
                 pAtom->SetAtomicNum(atno);
                 if(iso)
                   pAtom->SetIsotope(iso);
-              }
-			
-            else if(attrname=="x2" && (dim!=3 || use2d))//ignore 2D dimensions if 3D also provided
-              x=strtod(value.c_str(),NULL);
-
-            else if(attrname=="y2" && (dim!=3 || use2d))
-              {
-                dim=2;
-                y=strtod(value.c_str(),NULL);
+                continue;
               }
 
-            else if((attrname=="x3" && !use2d) || (pUnitCell && attrname=="xFract"))
+            //If more than one set of coordinates provided, 
+            //prefer 3D over 2D over 3Dfractional,
+            //but if use2d is true, prefer 2D over 3D
+            else if((attrname=="x3" || attrname=="y3" || attrname=="z3" || attrname=="xyz3") && !use2d)
+            {
+              using3 = true;
+              usingFract = false;
+            }
+            else if((attrname=="x2" || attrname=="y2" || attrname=="z2" || attrname=="xy2") && !using3)
+            {
+              using2 = true;
+              usingFract = false;
+            }
+            else if(pUnitCell && !using3 && !using2 
+              && (attrname=="xFract" || attrname=="yFract" || attrname=="zFract"))
+              usingFract=true;
+            
+            if(using3 && attrname=="x3" || using2 && attrname=="x2" || usingFract && attrname=="xFract")
               x=strtod(value.c_str(),NULL);
-
-            else if((attrname=="y3" && !use2d) || (pUnitCell && attrname=="yFract"))
+            else if(using3 && attrname=="y3" || using2 && attrname=="y2" || usingFract && attrname=="yFract")
               y=strtod(value.c_str(),NULL);
+            else if(using3 && attrname=="z3" || using2 && attrname=="z2" || usingFract && attrname=="zFract")
+              z=strtod(value.c_str(),NULL);
 
-            else if((attrname=="z3" && !use2d) || (pUnitCell && attrname=="zFract"))
+            else if(using2 && attrname=="xy2")
               {
-                dim=3;
-                z=strtod(value.c_str(),NULL);
-              }
-
-            else if(attrname=="xy2" && (dim!=3 || use2d))
-              {
-                dim=2;
                 vector<string> vals;
                 tokenize(vals,value);
                 if(vals.size()==2)
@@ -454,9 +538,8 @@ namespace OpenBabel
                     y=strtod(vals[1].c_str(),NULL);
                   }
               }
-            else if((attrname=="xyz3" && !use2d) || (pUnitCell && attrname=="xyzFract"))
+            else if(using3 && attrname=="xyz3")
               {
-                dim=3;
                 vector<string> vals;
                 tokenize(vals,value);
                 if(vals.size()==3)
@@ -466,22 +549,6 @@ namespace OpenBabel
                     z=strtod(vals[2].c_str(),NULL);
                   }
               }
-            else
-              datanotused=true;
-
-            if(!datanotused && dim)
-              {
-                if(!pUnitCell)
-                  pAtom->SetVector(x, y , z);
-                else
-                  {
-                    //Coordinates are fractional
-                    vector3 v;
-                    v.Set(x, y, z);
-                    v *= pUnitCell->GetOrthoMatrix();
-                    pAtom->SetVector(v);
-                  }
-              }
 
             if(attrname=="hydrogenCount")
               {
@@ -491,7 +558,7 @@ namespace OpenBabel
                   {
                     OBAtom* hatom = _pmol->NewAtom();
                     hatom->SetAtomicNum(1);
-                    hatom->SetType(const_cast<char*>("H"));
+                    hatom->SetType("H");
                     _pmol->AddBond(nhvy,_pmol->NumAtoms(),1);
                     ++nAtoms;
                   }
@@ -538,9 +605,32 @@ namespace OpenBabel
               pAtom->SetIsotope(atoi(value.c_str()));
 
           } //each attribute
-		
+
+          //Save atom coordinates
+          if(using3 || usingFract)
+            dim=3;
+          else if(using2)
+          {
+            dim=2;
+            z=0.0;
+          }
+          else
+            dim=0;
+          if(usingFract)
+            {
+              //Coordinates are fractional
+              vector3 v;
+              v.Set(x, y, z);
+              v *= pUnitCell->GetOrthoMatrix();
+              pAtom->SetVector(v);
+            }
+          else
+            pAtom->SetVector(x, y, z);
       }//each atom
-	
+    
+    if(aclass.size()>0)
+      _pmol->SetData((new OBAtomClassData(aclass)));
+
     _pmol->SetDimension(dim);
     return true;
   }
@@ -583,8 +673,11 @@ namespace OpenBabel
                   {
                     if(pos!=string::npos)
                       {
-                        indx1 = AtomMap[value.substr(0,pos)];	
-                        indx2 = AtomMap[value.substr(pos+1)];
+                        indx1 = AtomMap[value.substr(0,pos)];
+                        string temp =value.substr(pos+1);
+                        indx2 = AtomMap[Trim(temp)];
+//C4239                        indx2 = AtomMap[Trim(value.substr(pos+1))];
+
                       }
                     else
                       indx1 = AtomMap[value];
@@ -599,7 +692,6 @@ namespace OpenBabel
               }
             else if(attrname=="order")
               {	
-                Trim(value);
                 const char bo = value[0];
                 if(bo=='S')
                   ord=1;
@@ -607,8 +699,10 @@ namespace OpenBabel
                   ord=2;
                 else if(bo=='A')
                   ord=5;
-                else
-                  ord=atoi(&bo);
+                else {
+                  char* endptr;
+                  ord = strtol(value.c_str(), &endptr, 10);
+                }
               }
           }
 
@@ -681,8 +775,8 @@ namespace OpenBabel
               }
             else //bondStereo
               {
-                OBBond* pbond1;
-                OBBond* pbond2;
+                OBBond* pbond1=NULL;
+                OBBond* pbond2=NULL;
                 if(atrefsvalue.empty())
                   {
                     OBBond* pDBond = _pmol->GetBond(Idx);
@@ -1040,30 +1134,14 @@ namespace OpenBabel
     if(!_pxmlConv)
       return false;
 
-    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
-    if(pmol==NULL)
-			return false;
-    OBMol &mol = *pmol;
-
-    int numbonds = mol.NumBonds(); //Capture this before deleting Hs
-    bool UseHydrogenCount=false;
-    if(_pxmlConv->IsOption("h"))
-      {
-        pmol->DeleteHydrogens();
-        UseHydrogenCount=true;
-      }
-	
-    bool UseFormulaWithNoBonds=true;
-
     bool cml1 = _pxmlConv->IsOption("1");
     bool arrayform = _pxmlConv->IsOption("a");
     bool WriteAromaticBonds =  _pxmlConv->IsOption("A");
-    int dim = mol.GetDimension();
-	
     prefix = BAD_CAST _pxmlConv->IsOption("N");
-	
     xmlChar* uri=NULL;
-	
+
+    //Write the header on the first object (incl OBReaction)
+    //unless x option set or if has been called from elsewhere (e.g. CMLReact)
     if(!_pxmlConv->IsOption("MolsNotStandalone") && _pxmlConv->GetOutputIndex()==1)
       {
         if(!_pxmlConv->IsOption("x"))
@@ -1075,7 +1153,6 @@ namespace OpenBabel
               uri=BAD_CAST NamespaceURI();// not the old CML2NamespaceURI();
           }
         //If more than one molecule to be output, write <cml> at start and </cml> at end.
-        //Except if Option "0" set, e.g. by CMLReactFormat
         if(!_pxmlConv->IsLast())
           {
             xmlTextWriterStartElementNS(writer(), prefix, C_CML, uri);
@@ -1083,13 +1160,65 @@ namespace OpenBabel
           }
       }
 	
+    OBMol* pmol = dynamic_cast<OBMol*>(pOb);
+    if(pmol==NULL)
+    {
+#ifdef HAVE_SHARED_POINTER
+        OBReaction* pReact = dynamic_cast<OBReaction*>(pOb);
+        if(!pReact)
+          return false;
+        //Use CMLReact to convert OBReaction object
+        OBFormat* pCMLRFormat = pConv->FindFormat("cmlr");
+        if(!pCMLRFormat)
+        {
+          obErrorLog.ThrowError(__FUNCTION__, "Cannot find CMLReact format", obError);
+          return false;
+        }
+        //Disable list option and supress topping and tailing in CMLReactFormat.
+        _pxmlConv->AddOption("l", OBConversion::OUTOPTIONS);
+        _pxmlConv->AddOption("ReactionsNotStandalone", OBConversion::OUTOPTIONS);
+        bool ret = pCMLRFormat->WriteMolecule(pOb,_pxmlConv);
+        _pxmlConv->RemoveOption("ReactionsNotStandalone", OBConversion::OUTOPTIONS);
+        return ret;
+#else
+        return false;
+#endif
+    }
+
+
+    OBMol &mol = *pmol;
+
+    int numbonds = mol.NumBonds(); //Capture this before deleting Hs
+    bool UseHydrogenCount=false;
+    if(_pxmlConv->IsOption("h"))
+      {
+        pmol->DeleteHydrogens();
+        UseHydrogenCount=true;
+      }
+
+    bool UseFormulaWithNoBonds=true;
+
+    int dim = mol.GetDimension();
+
+
     xmlTextWriterStartElementNS(writer(), prefix, C_MOLECULE, uri);
 	
     const char* id = mol.GetTitle();
     if(*id)
       {
         string name(id);
-        if(!isalpha(*id)) //since ids have to start with a letter, add "id" to those that don't...
+        //If name is a filename with a path, remove path and extension
+        string::size_type pos;
+        pos = name.find_last_of("/\\:");
+        if(pos!=string::npos)
+        {
+          name.erase(0, pos+1);
+          pos = name.rfind('.');
+          if(pos!=string::npos)
+            name.erase(pos);
+        }
+
+        if(!isalpha(name[0])) //since ids have to start with a letter, add "id" to those that don't...
           name = "id" + name;
         xmlTextWriterWriteAttribute(writer(), C_ID, BAD_CAST name.c_str());
         if(!isalpha(*id)) //...and write <name> orig title </name>
@@ -1099,6 +1228,12 @@ namespace OpenBabel
             xmlTextWriterEndElement(writer());//name
           }
       }
+
+    //spinMultiplicity is written as an attribute of <molecule> only when it is not 1 and the molecule has bonds
+    int smult = mol.GetTotalSpinMultiplicity();
+    if(smult!=1 && numbonds!=0)
+      xmlTextWriterWriteFormatAttribute(writer(), C_SPINMULTIPLICITY,"%d", smult);
+
     if(_pxmlConv->IsOption("m") && _pxmlConv->GetOutputIndex()==1) //only on first molecule
       WriteMetadataList();
 
@@ -1109,6 +1244,10 @@ namespace OpenBabel
         UseFormulaWithNoBonds = false;
       }
 
+    WriteInChI(mol);
+
+    vector<string> atomIds; 
+
     if(mol.NumAtoms()>0)
       {
         //if molecule has no bonds and atoms doesn't have coordinates, just output formula
@@ -1117,6 +1256,9 @@ namespace OpenBabel
         else
           {
             xmlTextWriterStartElementNS(writer(), prefix, C_ATOMARRAY, NULL);
+
+            MakeAtomIds(mol, atomIds);//Pre-construct to take into account atom class data
+
 #ifdef HAVE_SSTREAM
             stringstream id, eltyp, iso, chg, spn, hct, x, y, z;
 #else
@@ -1132,7 +1274,7 @@ namespace OpenBabel
             vector<OBAtom*>::iterator i;
             for (patom = mol.BeginAtom(i);patom;patom = mol.NextAtom(i))
               {
-                string el(etab.GetSymbol(patom->GetAtomicNum()));
+               string el(etab.GetSymbol(patom->GetAtomicNum()));
                 if(el=="Xx")
                   el="R";
 
@@ -1167,7 +1309,7 @@ namespace OpenBabel
                       anySpin = true;
                     if(isotope)
                       anyIsotope = true;
-                    id << " " << "a" << patom->GetIdx();
+                    id << " " << atomIds[patom->GetIdx()];
                     eltyp << " " << el;
                     iso << " " << isotope;
                     chg << " " << charge;
@@ -1182,8 +1324,8 @@ namespace OpenBabel
                   {
                     //Non-array form
                     xmlTextWriterStartElementNS(writer(), prefix, C_ATOM, NULL);
-                    xmlTextWriterWriteFormatAttribute(writer(), C_ID,"a%d", patom->GetIdx());
-					
+                      xmlTextWriterWriteFormatAttribute(writer(), C_ID,"%s", atomIds[patom->GetIdx()].c_str());
+
                     if(!cml1)
                       {	
                         xmlTextWriterWriteFormatAttribute(writer(), C_ELEMENTTYPE,"%s", el.c_str());
@@ -1222,9 +1364,13 @@ namespace OpenBabel
                               {
                                 //UseAtom4Refs from OBChiralData
                                 vector<unsigned int> ref = cd->GetAtom4Refs(input);
+                                while (ref.size()<4)
+                                  ref.push_back(patom->GetIdx());
                                 xmlTextWriterStartElementNS(writer(), prefix, C_ATOMPARITY, NULL);
-                                xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREFS4,
-                                                                  "a%d a%d a%d a%d", ref[0], ref[1], ref[2], ref[3]);														
+                                xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREFS4, "%s %s %s %s",
+//                                                     "a%d a%d a%d a%d", ref[0], ref[1], ref[2], ref[3]);
+                                    atomIds[ref[0]].c_str(), atomIds[ref[1]].c_str(),
+                                    atomIds[ref[2]].c_str(), atomIds[ref[3]].c_str());
                                 xmlTextWriterWriteFormatString(writer(),"%d", cfg);
                                 xmlTextWriterEndElement(writer());//atomParity
                               }
@@ -1370,51 +1516,49 @@ namespace OpenBabel
         xmlTextWriterStartElementNS(writer(), prefix, C_BONDARRAY, NULL);
 
 #ifdef HAVE_SSTREAM
-        stringstream ref1, ref2, ord;
+        stringstream ord;
 #else
-        strstream ref1, ref2, ord;
+        strstream ord;
 #endif
+        
+        string ref1, ref2;
         OBBond *pbond;
         vector<OBBond*>::iterator ib;
         for (pbond = mol.BeginBond(ib);pbond;pbond = mol.NextBond(ib))
           {
             int bo = pbond->GetBO();
-            if(arrayform)
+
+            if(!arrayform)
               {
-                ref1 << " a" << pbond->GetBeginAtomIdx();
-                ref2 << " a" << pbond->GetEndAtomIdx();
-                if(bo==5) //aromatic
-                  ord << " " << 'A';
+                if(bo==5 || (WriteAromaticBonds && pbond->IsAromatic())) //aromatic
+                  ord << 'A';
                 else
-                  ord << " " << bo;
-              }
-            else
-              {
+                  ord << bo;
+
+                ref1 = atomIds[pbond->GetBeginAtomIdx()];
+                ref2 = atomIds[pbond->GetEndAtomIdx()];
                 xmlTextWriterStartElementNS(writer(), prefix, C_BOND, NULL);
                 //				xmlTextWriterWriteFormatAttribute(writer(), C_ID,"b%d", pbond->GetIdx()); remove bond id
                 if(!cml1)
                   {
-                    xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREFS2,"a%d a%d",
-                                                      pbond->GetBeginAtomIdx(), pbond->GetEndAtomIdx() );
-                    if(bo==5 || (WriteAromaticBonds && pbond->IsAromatic())) //aromatic
-                      xmlTextWriterWriteFormatAttribute(writer(), C_ORDER,"%c", 'A');
-                    else
-                      xmlTextWriterWriteFormatAttribute(writer(), C_ORDER,"%d", bo);
+                    xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREFS2,"%s %s",
+                          ref1.c_str(), ref2.c_str());
+                    xmlTextWriterWriteFormatAttribute(writer(), C_ORDER,"%s", ord.str().c_str());
 					
                     if(bo==2 || pbond->IsWedge() || pbond->IsHash())
-                      WriteBondStereo(pbond);
+                      WriteBondStereo(pbond, atomIds);
                   }
                 else
                   {
                     //CML1
                     xmlTextWriterStartElementNS(writer(), prefix, C_STRING, NULL);
                     xmlTextWriterWriteFormatAttribute(writer(), C_BUILTIN,"%s", "atomRef");
-                    xmlTextWriterWriteFormatString(writer(),"a%d", pbond->GetBeginAtomIdx());
+                    xmlTextWriterWriteFormatString(writer(),"%s", ref1.c_str());
                     xmlTextWriterEndElement(writer());
 
                     xmlTextWriterStartElementNS(writer(), prefix, C_STRING, NULL);
                     xmlTextWriterWriteFormatAttribute(writer(), C_BUILTIN,"%s", "atomRef");
-                    xmlTextWriterWriteFormatString(writer(),"a%d", pbond->GetEndAtomIdx());
+                    xmlTextWriterWriteFormatString(writer(),"%s", ref2.c_str());
                     xmlTextWriterEndElement(writer());
 
                     xmlTextWriterStartElementNS(writer(), prefix, C_STRING, NULL);
@@ -1423,14 +1567,25 @@ namespace OpenBabel
                     xmlTextWriterEndElement(writer());
                   }
                 xmlTextWriterEndElement(writer());//bond
+                ord.str(""); //clear (For array form it accumulates.)
+              }
+            else
+              {
+                if(bo==5 || (WriteAromaticBonds && pbond->IsAromatic())) //aromatic
+                  ord << " " << 'A';
+                else
+                  ord << " " << bo;
+
+                ref1 += ' ' + atomIds[pbond->GetBeginAtomIdx()];
+                ref2 += ' ' + atomIds[pbond->GetEndAtomIdx()];
               }
           }
         if(arrayform)
           {
             if(!cml1)
               {
-                xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREF1, "%s", ref1.str().c_str());
-                xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREF2, "%s", ref2.str().c_str());
+                xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREF1, "%s", ref1.c_str());
+                xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREF2, "%s", ref2.c_str());
                 xmlTextWriterWriteFormatAttribute(writer(), C_ORDER, "%s", ord.str().c_str());
               }
             else
@@ -1438,12 +1593,12 @@ namespace OpenBabel
                 //CML1
                 xmlTextWriterStartElementNS(writer(), prefix, C_STRINGARRAY, NULL);
                 xmlTextWriterWriteFormatAttribute(writer(), C_BUILTIN,"%s", "atomRef");
-                xmlTextWriterWriteFormatString(writer(),"%s", ref1.str().c_str());
+                xmlTextWriterWriteFormatString(writer(),"%s", ref1.c_str());
                 xmlTextWriterEndElement(writer());
 
                 xmlTextWriterStartElementNS(writer(), prefix, C_STRINGARRAY, NULL);
                 xmlTextWriterWriteFormatAttribute(writer(), C_BUILTIN,"%s", "atomRef");
-                xmlTextWriterWriteFormatString(writer(),"%s", ref2.str().c_str());
+                xmlTextWriterWriteFormatString(writer(),"%s", ref2.c_str());
                 xmlTextWriterEndElement(writer());
 
                 xmlTextWriterStartElementNS(writer(), prefix, C_STRINGARRAY, NULL);
@@ -1461,7 +1616,7 @@ namespace OpenBabel
             for (pbond = mol.BeginBond(ib);pbond;pbond = mol.NextBond(ib))
               {
                 if(pbond->GetBO()==2 || pbond->IsWedge() || pbond->IsHash())
-                  WriteBondStereo(pbond);
+                  WriteBondStereo(pbond, atomIds);
               }
           }
       }
@@ -1477,6 +1632,8 @@ namespace OpenBabel
 
     xmlTextWriterEndElement(writer());//molecule
 
+    //Note that nothing will be written unless the next block is executed
+    //IsLast() MUST return true for the last molecule.
     if(!_pxmlConv->IsOption("MolsNotStandalone") && _pxmlConv->IsLast())
       {
         xmlTextWriterEndDocument(writer());
@@ -1485,6 +1642,39 @@ namespace OpenBabel
     return true;
   }
 
+  ///Constructs a unique id for each atom.
+  void CMLFormat::MakeAtomIds(OBMol& mol, vector<string>& atomIDs)
+  {
+    /* If there is no atom class data for the atom, the id is a followed by the atom index.
+       If there is atom class data then it is aa followed by the atom class. 
+       If a subsequent atom has the same atom class, its id is ab followed
+       by the atom class, and so on. */
+
+    stringstream ss;
+    map<int,char> acmap; //key=atom calss; value=last letter used as second in id
+    OBAtomClassData* pac = static_cast<OBAtomClassData*>(mol.GetData("Atom Class"));
+    atomIDs.push_back("Error"); //atom idex stats at 1. atomIDs[0] is not used
+    for (int idx=1; idx<=mol.NumAtoms(); ++idx)
+    {
+      ss.str("");
+      ss << 'a';
+      if(pac && pac->HasClass(idx))
+      {
+        int ac = pac->GetClass(idx);
+        char ch2='a'; //default 2nd char
+        if(acmap.count(ac)>0)
+          ch2 = acmap[ac]+1;
+        if(ch2>'z')
+          obErrorLog.ThrowError(_pmol->GetTitle(),"CML: too many atoms with same atom class." , obError);
+        ss << ch2 << ac;
+        acmap[ac] = ch2;
+      }
+      else
+        ss << idx;
+      atomIDs.push_back(ss.str());
+    }
+  }
+
   void CMLFormat::WriteFormula(OBMol mol)
   {
     //mol is a copy
@@ -1497,7 +1687,7 @@ namespace OpenBabel
     xmlTextWriterEndElement(writer());//formula
   }
 
-  void CMLFormat::WriteBondStereo(OBBond* pbond)
+  void CMLFormat::WriteBondStereo(OBBond* pbond, vector<string>& atomIDs)
   {
     static const xmlChar C_ATOMREFS4[]  = "atomRefs4";
     static const xmlChar C_BONDSTEREO[] = "bondStereo";
@@ -1515,7 +1705,7 @@ namespace OpenBabel
     {
       //double bond stereo
       int ud1=0, ud2=0;
-      int idx1, idx2;
+      int idx1=0, idx2=0;
       OBAtom* patomA = pbond->GetBeginAtom();
       FOR_BONDS_OF_ATOM(b1,patomA)
         {
@@ -1544,8 +1734,10 @@ namespace OpenBabel
         return;
   	
       xmlTextWriterStartElementNS(writer(), prefix, C_BONDSTEREO, NULL);
-      xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREFS4,
-                                        "a%d a%d a%d a%d", idx1, patomA->GetIdx(), patomB->GetIdx(), idx2);
+      xmlTextWriterWriteFormatAttribute(writer(), C_ATOMREFS4, "%s %s %s %s",
+//                        "a%d a%d a%d a%d", idx1, patomA->GetIdx(), patomB->GetIdx(), idx2);
+            atomIDs[idx1].c_str(), atomIDs[patomA->GetIdx()].c_str(),
+            atomIDs[patomB->GetIdx()].c_str(), atomIDs[idx2].c_str());
       ch = (ud1==ud2) ? 'C' : 'T';
     }
 
@@ -1560,6 +1752,9 @@ namespace OpenBabel
     // static const xmlChar C_Z[] = "z";
     static const xmlChar C_TITLE[] = "title";
     static const xmlChar C_UNITS[] = "units";
+    static const xmlChar C_SYMMETRY[]  = "symmetry";
+    static const xmlChar C_SPACEGROUP[]  = "spaceGroup";
+    static const xmlChar C_TRANSFORM3[]  = "transform3";
 
     pUnitCell = (OBUnitCell*)mol.GetData(OBGenericDataType::UnitCell);
 
@@ -1602,6 +1797,35 @@ namespace OpenBabel
     xmlTextWriterWriteFormatString(writer(),"%f", pUnitCell->GetGamma());
     xmlTextWriterEndElement(writer());//scalar
 
+    const SpaceGroup *group = pUnitCell->GetSpaceGroup();
+    string s;
+    if (group)
+	  {
+        xmlTextWriterStartElementNS(writer(), prefix, C_SYMMETRY, NULL);
+        xmlTextWriterWriteAttribute (writer(), C_SPACEGROUP, (const xmlChar*)group->GetHallName().c_str());
+        transform3dIterator ti;
+        const transform3d *t = group->BeginTransform(ti);
+        string s;
+        while(t)
+          {
+			s = t->DescribeAsValues() + " 0 0 0 1";
+            xmlTextWriterWriteElement(writer(), C_TRANSFORM3, (const xmlChar*)s.c_str());
+            t = group->NextTransform(ti);
+          }
+        xmlTextWriterEndElement(writer());//symmetry
+	  }
+    else
+	  {
+        //s = pUnitCell.GetSpaceGroupName();
+        s = pUnitCell->GetSpaceGroupName();
+        if (s.length())
+	      {
+            xmlTextWriterStartElementNS(writer(), prefix, C_SYMMETRY, NULL);
+            xmlTextWriterWriteAttribute (writer(), C_SPACEGROUP, (const xmlChar*)s.c_str());
+            xmlTextWriterEndElement(writer());//symmetry
+	      }
+	  }
+
     xmlTextWriterEndElement(writer());//crystal	
   }
 
@@ -1613,12 +1837,13 @@ namespace OpenBabel
     static const xmlChar C_SCALAR[]       = "scalar";
     static const xmlChar C_TITLE[]        = "title";
 
-    //	xmlTextWriterStartElementNS(writer(), prefix, C_PROPERTYLIST, NULL);
     vector<OBGenericData*>::iterator k;
     vector<OBGenericData*> vdata = mol.GetData();
     for (k = vdata.begin();k != vdata.end();k++)
       {
-        if ((*k)->GetDataType() == OBGenericDataType::PairData)
+        if ((*k)->GetDataType() == OBGenericDataType::PairData
+          && (*k)->GetAttribute()!="InChI" //InChI is output in <identifier>
+          && (*k)->GetAttribute()!="PartialCharges")//annotation not needed since partial charges are not output in this format
           {
             if(!propertyListWritten)
               {
@@ -1626,7 +1851,7 @@ namespace OpenBabel
                 propertyListWritten=true;
               }
             xmlTextWriterStartElementNS(writer(), prefix, C_PROPERTY, NULL);
-            //Title is onow on <property>
+            //Title is now on <property>
             xmlTextWriterWriteFormatAttribute(writer(), C_TITLE,"%s",(*k)->GetAttribute().c_str());
             xmlTextWriterStartElementNS(writer(), prefix, C_SCALAR, NULL);
             //Title used to be on <scalar>...
@@ -1636,6 +1861,11 @@ namespace OpenBabel
             xmlTextWriterEndElement(writer());//property
           }
       }
+    if(mol.HasData(OBGenericDataType::VibrationData))
+      WriteVibrationData(mol);
+    if(mol.HasData(OBGenericDataType::RotationData))
+      WriteRotationData(mol);
+
   }
 
   void CMLFormat::WriteThermo(OBMol& mol, bool& propertyListWritten)
@@ -1706,4 +1936,103 @@ namespace OpenBabel
     return molID.str();
   }
 
+  bool CMLFormat::WriteInChI(OBMol& mol)
+  {
+    //If OBPair data has an entry with attribute "inchi" it is not
+    //output in the property list but as a separate element in the form:
+    //<identifier convention="iupac:inchi" value="InChI=1/CH4/h1H4"/>
+    static const xmlChar C_IDENTIFIER[] = "identifier";
+    static const xmlChar C_CONVENTION[] = "convention";
+    static const xmlChar C_VALUE[]      = "value";
+    OBPairData* pData = dynamic_cast<OBPairData*>(mol.GetData("InChI"));
+    if(pData)
+    {
+      xmlTextWriterStartElementNS(writer(), prefix, C_IDENTIFIER, NULL);
+      xmlTextWriterWriteFormatAttribute(writer(), C_CONVENTION,"%s","iupac:inchi");
+      xmlTextWriterWriteFormatAttribute(writer(), C_VALUE,"%s", pData->GetValue().c_str());
+      xmlTextWriterEndElement(writer());//identifier
+      return true;
+    }
+    return false; //not written
+  }
+
+  bool CMLFormat::WriteVibrationData(OBMol& mol)
+  {
+    static const xmlChar C_PROPERTY[]     = "property";
+    static const xmlChar C_SCALAR[]       = "scalar";
+    static const xmlChar C_ARRAY[]        = "array";
+    static const xmlChar C_DICTREF[]      = "dictRef";
+    static const xmlChar C_UNITS[]        = "units";
+    static const xmlChar C_TITLE[]        = "title";
+
+    OBVibrationData* vd = (OBVibrationData*)mol.GetData(OBGenericDataType::VibrationData);
+
+    xmlTextWriterStartElementNS(writer(), prefix, C_PROPERTY, NULL);
+    xmlTextWriterWriteFormatAttribute(writer(), C_TITLE,"%s","Vibrational Frequencies");
+    xmlTextWriterWriteFormatAttribute(writer(), C_DICTREF,"%s","me:vibFreqs");
+
+    xmlTextWriterStartElementNS(writer(), prefix, C_ARRAY, NULL);
+    xmlTextWriterWriteFormatAttribute(writer(), C_UNITS,"%s","cm-1");
+    for(int i=0; i<vd->GetNumberOfFrequencies(); ++i)
+      xmlTextWriterWriteFormatString(writer(),"%.lf ", vd->GetFrequencies()[i]);
+    xmlTextWriterEndElement(writer());//array
+    xmlTextWriterEndElement(writer());//property
+    return true;
+  }
+
+  bool CMLFormat::WriteRotationData(OBMol& mol)
+  {
+    static const xmlChar C_PROPERTY[]     = "property";
+    static const xmlChar C_SCALAR[]       = "scalar";
+    static const xmlChar C_ARRAY[]        = "array";
+    static const xmlChar C_DICTREF[]      = "dictRef";
+    static const xmlChar C_UNITS[]        = "units";
+    static const xmlChar C_TITLE[]        = "title";
+
+    OBRotationData* rd = (OBRotationData*)mol.GetData(OBGenericDataType::RotationData);
+
+    xmlTextWriterStartElementNS(writer(), prefix, C_PROPERTY, NULL);
+    xmlTextWriterWriteFormatAttribute(writer(), C_TITLE,"%s","Rotational Constants");
+    xmlTextWriterWriteFormatAttribute(writer(), C_DICTREF,"%s","me:rotConsts");
+
+    xmlTextWriterStartElementNS(writer(), prefix, C_ARRAY, NULL);
+    xmlTextWriterWriteFormatAttribute(writer(), C_UNITS,"%s","cm-1");
+    const double WAVENUM_TO_GHZ=30.0;
+    for(int i=0; i<3; ++i)
+      if(rd->GetRotConsts()[i]!=0.0)
+        xmlTextWriterWriteFormatString(writer(),"%.1f ", rd->GetRotConsts()[i]/WAVENUM_TO_GHZ);
+    xmlTextWriterEndElement(writer());//array
+    xmlTextWriterEndElement(writer());//property
+    xmlTextWriterStartElementNS(writer(), prefix, C_PROPERTY, NULL);
+    xmlTextWriterWriteFormatAttribute(writer(), C_TITLE,"%s","Symmetry Number");
+    xmlTextWriterWriteFormatAttribute(writer(), C_DICTREF,"%s","me:symmetryNumber");
+
+    xmlTextWriterStartElementNS(writer(), prefix, C_SCALAR, NULL);
+    xmlTextWriterWriteFormatString(writer(),"%d ", rd->GetSymmetryNumber());
+    xmlTextWriterEndElement(writer());//scalar
+    xmlTextWriterEndElement(writer());//property
+    return true;
+  }
+
+  bool CMLFormat::WriteChemObject(OBConversion* pConv)
+  {
+    int OIndex = pConv->GetOutputIndex();
+    OBBase* pOb = pConv->GetChemObject();
+    if(dynamic_cast<OBMol*> (pOb))
+    {
+      //With an OBMol object, do the same as if this function wasn't defined,
+      //i.e.access the functionality in OBMoleculeFormat
+
+      //restore output index which is (unhelpfully) incremented by GetChemObject
+      pConv->SetOutputIndex(OIndex);
+      return XMLMoleculeFormat::WriteChemObject(pConv);
+    }
+
+    //With OBReaction object, handle directly in CMLFormat::WriteMolecule
+    bool ret = WriteMolecule(pOb,pConv);
+    delete pOb;
+    return ret;
+  }
+
+
 }//namespace
