@@ -24,8 +24,6 @@ GNU General Public License for more details.
 
 ***********************************************************/
 #include <openbabel/babelconfig.h>
-#include <climits>
-#include <cstring>
 
 /* Adapt on some 64bit machines! */
 typedef int int32;
@@ -365,7 +363,7 @@ bool YOBFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 
   bool hetatom;
   char buffer[8],resname[4],atomname[5];
-  const char *str;
+  string str;
   unsigned int i,j/*,m,q*/;
   unsigned int /*resno,chainNum,*/link,linked,linktype,atoms,element,links,chain;
   int /*samenames,*/reslen,charged;
@@ -435,8 +433,8 @@ bool YOBFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
     if (atomname[0]==' '&&(!pConv->IsOption("f",OBConversion::INOPTIONS))) memcpy(atomname,atomname+1,4);
     /* RENAME TERMINAL OXYGENS */
     str=atomname;
-    if (!strcmp(str,"OT1")) str="O";
-    if (!strcmp(str,"OT2")) str="OXT";
+    if (str=="OT1") str="O";
+    if (str=="OT2") str="OXT";
     res->SetAtomID(dstatom,str);
     if (srcatom->header[MOB_ELEMENT]&MOB_HETATOMFLAG) hetatom=true;
     else hetatom=false;
@@ -567,7 +565,7 @@ bool YOBFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
         if (strlen(mob_elementsym[element])==1||strncasecmp(mob_elementsym[element],atomname,2)) pos=5; }
       str_copy(&buffer[pos],atomname);
       str_copy(&buffer[8],(char*)res->GetName().c_str());
-      sprintf(&buffer[12],"%4d",res->GetNum()); }
+      snprintf(&buffer[12], 4, "%4d",res->GetNum()); }
     else
     { str_copy(&buffer[4],etab.GetSymbol(element));
       str_copy(&buffer[8],"UNK    1"); }
