@@ -24,6 +24,10 @@ GNU General Public License for more details.
 #include <openbabel/generic.h>
 #include <openbabel/math/matrix3x3.h>
 
+// needed for msvc to have at least one reference to AtomClass, AliasData in openbabel library
+#include <openbabel/atomclass.h>
+#include <openbabel/alias.h>
+
 using namespace std;
 
 namespace OpenBabel
@@ -244,7 +248,7 @@ namespace OpenBabel
   OBUnitCell::OBUnitCell():
     OBGenericData("UnitCell", OBGenericDataType::UnitCell),
     _a(0.0), _b(0.0), _c(0.0), _alpha(0.0), _beta(0.0), _gamma(0.0),
-    _numericSpaceGroup( -1 ), _lattice(Undefined)
+    _spaceGroup( NULL ), _lattice(Undefined)
   {  }
 
   OBUnitCell::OBUnitCell(const OBUnitCell &src) :
@@ -253,8 +257,8 @@ namespace OpenBabel
     _alpha(src._alpha), _beta(src._beta), _gamma(src._gamma),
     _offset(src._offset),
     _v1(src._v1), _v2(src._v2), _v3(src._v3),
+    _spaceGroupName(src._spaceGroupName),
     _spaceGroup(src._spaceGroup),
-    _numericSpaceGroup( -1 ),
     _lattice(src._lattice)
   {  }
 
@@ -276,6 +280,7 @@ namespace OpenBabel
     _v3 = src._v3;
 
     _spaceGroup = src._spaceGroup;
+    _spaceGroupName = src._spaceGroupName;
     _lattice = src._lattice;
 
     return(*this);
@@ -409,8 +414,11 @@ namespace OpenBabel
 	  //	143-167 Rhombohedral
 	  //	168-194 Hexagonal
 	  //	195-230 Cubic
+
+      if ( spacegroup == 0  && _spaceGroup)
+          spacegroup = _spaceGroup->GetId();
 	  
-	  if ( spacegroup < 0 )
+	  if ( spacegroup <= 0 )
 		  return OBUnitCell::Undefined;
 
 	  else if ( spacegroup == 1 ||
@@ -450,6 +458,8 @@ namespace OpenBabel
   {
     if (_lattice != Undefined)
       return _lattice;
+    else if (_spaceGroup != NULL)
+      return GetLatticeType(_spaceGroup->GetId());
 
     unsigned int rightAngles = 0;
     if (IsApprox(_alpha, 90.0, 1.0e-3)) rightAngles++;
@@ -484,6 +494,7 @@ namespace OpenBabel
 
     return _lattice;
   }
+
   int OBUnitCell::GetSpaceGroupNumber( std::string name)
   {
     static const char * const spacegroups[] = { 
@@ -520,6 +531,13 @@ namespace OpenBabel
       "Fd-3m", "Fd-3c", "Im-3m", "Ia-3d"
     };
 
+    if (name.length () == 0)
+	  {
+        if (_spaceGroup != NULL)
+          return _spaceGroup->GetId();
+        else
+          name = _spaceGroupName;
+      }
     static const int numStrings = sizeof( spacegroups ) / sizeof( spacegroups[0] );
     for ( int i = 0; i < numStrings; ++i ) {
       if (name == spacegroups[i] ) {
@@ -529,6 +547,43 @@ namespace OpenBabel
     return 0; //presumably never reached
   }
   
+  void OBUnitCell::FillUnitCell(OBMol *mol)
+  {
+    const SpaceGroup *sg = GetSpaceGroup(); // the actual space group and transformations for this unit cell
+    
+    // For each atom, we loop through: convert the coords back to inverse space, apply the transformations and create new atoms
+    vector3 uniqueV, newV;
+    list<vector3> transformedVectors; // list of symmetry-defined copies of the atom
+    list<vector3>::iterator transformIterator;
+    OBAtom *newAtom;
+    list<OBAtom*> atoms; // keep the current list of unique atoms -- don't double-create
+    FOR_ATOMS_OF_MOL(atom, *mol)
+      atoms.push_back(&(*atom));
+
+    list<OBAtom*>::iterator i;
+    for (i = atoms.begin(); i != atoms.end(); ++i) {
+      uniqueV = (*i)->GetVector();
+      uniqueV *= GetFractionalMatrix();
+        
+      transformedVectors = sg->Transform(uniqueV);
+      for (transformIterator = transformedVectors.begin();
+           transformIterator != transformedVectors.end(); ++transformIterator) {
+        // coordinates are in reciprocal space -- check if it's in the unit cell
+        // TODO: transform these into the unit cell and check for duplicates
+        if (transformIterator->x() < 0.0 || transformIterator->x() > 1.0)
+          continue;
+        else if (transformIterator->y() < 0.0 || transformIterator->y() > 1.0)
+          continue;
+        else if (transformIterator->z() < 0.0 || transformIterator->z() > 1.0)
+          continue;
+             
+        newAtom = mol->NewAtom();
+        newAtom->Duplicate(*i);
+        newAtom->SetVector(GetOrthoMatrix() * (*transformIterator));
+      } // end loop of transformed atoms
+    } // end loop of atoms
+  }
+  
   double OBUnitCell::GetCellVolume()
   {
     double result = 0.0;
@@ -584,7 +639,7 @@ namespace OpenBabel
 
   OBSymmetryData::OBSymmetryData(const OBSymmetryData &src) :
     OBGenericData(src._attr, src._type, src._source), 
-    _spaceGroup(src._spaceGroup), _pointGroup(src._pointGroup)
+    _pointGroup(src._pointGroup), _spaceGroup(src._spaceGroup)
   {  }
 
   OBSymmetryData & OBSymmetryData::operator=(const OBSymmetryData &src)
@@ -896,7 +951,7 @@ namespace OpenBabel
   */
   bool OBAngleData::FillAngleArray(std::vector<std::vector<unsigned int> > &angles)
   {
-    if(_angles.size() == 0)
+    if(_angles.empty())
       return(false);
 
     vector<OBAngle>::iterator angle;
@@ -1149,7 +1204,7 @@ namespace OpenBabel
   */
   bool OBTorsionData::FillTorsionArray(std::vector<std::vector<unsigned int> > &torsions)
   {
-    if(_torsions.size() == 0)
+    if(_torsions.empty())
       return(false);
 
     vector<quad<OBAtom*,OBAtom*,OBAtom*,OBAtom*> > tmpquads,quads;
@@ -1306,36 +1361,6 @@ namespace OpenBabel
 //member functions for OBVibrationData class
 //
 
-/*!
-**\brief OBVibrationData constructor
-*/
-OBVibrationData::OBVibrationData()
-{
-  this->_type = OBGenericDataType::VibrationData;
-  this->_attr = "VibrationData";
-}
-
-/*!
-**\brief OBVibrationData copy constructor
-*/
-OBVibrationData::OBVibrationData(const OBVibrationData & src)
-{
-  this->SetData(src._vLx, src._vFrequencies, src._vIntensities);
-}
-
-/*!
-**\brief OBVibrationData assignment operator
-*/
-OBVibrationData & OBVibrationData::operator=(const OBVibrationData & src)
-{
-  if (this == &src)
-    return *this;
-
-  this->SetData(src._vLx, src._vFrequencies, src._vIntensities);
-
-  return *this;
-}
-
 /*!
 **\brief Assign the data
 **\param vLx Normal modes in 1/sqrt(a.u.)
