@@ -1,16 +1,17 @@
 /**********************************************************************
 grid.cpp - Handle grids of values.
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
- 
+Some Portions Copyright (C) 2008 by Marcus D. Hanwell
+
 This file is part of the Open Babel project.
 For more information, see <http://openbabel.sourceforge.net/>
- 
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -30,35 +31,34 @@ namespace OpenBabel
   {
     OBAtom *atom;
     vector<OBAtom*>::iterator i;
-    
-    for (atom = box.BeginAtom(i);atom;atom = box.NextAtom(i))
-      if (atom->GetIdx() == 1)
-        {
+
+    for (atom = box.BeginAtom(i); atom; atom = box.NextAtom(i)) {
+      if (atom->GetIdx() == 1) {
+        _xmin = atom->GetX();
+        _xmax = atom->GetX();
+        _ymin = atom->GetY();
+        _ymax = atom->GetY();
+        _zmin = atom->GetZ();
+        _zmax = atom->GetZ();
+      }
+      else {
+        if (atom->GetX() < _xmin)
           _xmin = atom->GetX();
+        if (atom->GetX() > _xmax)
           _xmax = atom->GetX();
+        if (atom->GetY() < _ymin)
           _ymin = atom->GetY();
+        if (atom->GetY() > _ymax)
           _ymax = atom->GetY();
+        if (atom->GetZ() < _zmin)
           _zmin = atom->GetZ();
+        if (atom->GetZ() > _zmax)
           _zmax = atom->GetZ();
-        }
-      else
-        {
-          if (atom->GetX() < _xmin)
-            _xmin = atom->GetX();
-          if (atom->GetX() > _xmax)
-            _xmax = atom->GetX();
-          if (atom->GetY() < _ymin)
-            _ymin = atom->GetY();
-          if (atom->GetY() > _ymax)
-            _ymax = atom->GetY();
-          if (atom->GetZ() < _zmin)
-            _zmin = atom->GetZ();
-          if (atom->GetZ() > _zmax)
-            _zmax = atom->GetZ();
-        }
+      }
+    }
   }
-  
-  void OBFloatGrid::Init(OBMol &box,double spacing, double pad)
+
+  void OBFloatGrid::Init(OBMol &box, double spacing, double pad)
   {
     OBGrid::Init(box); // handle in the base class
 
@@ -73,70 +73,125 @@ namespace OpenBabel
     _midx=0.5*(_xmax+_xmin);
     _midy=0.5*(_ymax+_ymin);
     _midz=0.5*(_zmax+_zmin);
-    _xdim=3+(int) ((_xmax-_xmin)/spacing);
-    _ydim=3+(int) ((_ymax-_ymin)/spacing);
-    _zdim=3+(int) ((_zmax-_zmin)/spacing);
+    _xdim=static_cast<int>((_xmax-_xmin)/spacing) + 1;
+    _ydim=static_cast<int>((_ymax-_ymin)/spacing) + 1;
+    _zdim=static_cast<int>((_zmax-_zmin)/spacing) + 1;
     _spacing=spacing;
-    _halfSpace= _spacing/2.0;
+    _halfSpace=_spacing/2.0;
     _inv_spa=1.0/_spacing;
     _ival=NULL;
 
+    // Calculate the size needed, resize the vector and initialise it to 0.0
     int size = _xdim*_ydim*_zdim;
-    _val = new double [size];
-    double *valptr = _val;
-    for( int i = 0; i < size; i++)
-      {
-        *valptr = 0.0;
-        valptr++;
-      }
+    _values.resize(size, 0.0);
   }
 
-  double OBFloatGrid::Inject(double x,double y,double z)
+  void OBFloatGrid::SetLimits(const double origin[3], const double v1[3], const double v2[3],
+                              const double v3[3])
   {
+    // Convert to vectors and call the vector form of the function
+    vector3 originv(origin[0], origin[1], origin[2]);
+    vector3 x(v1[0], v1[1], v1[2]);
+    vector3 y(v2[0], v2[1], v2[2]);
+    vector3 z(v3[0], v3[1], v3[2]);
+    SetLimits(originv, x, y, z);
+  }
+
+  void OBFloatGrid::SetLimits(const vector3& origin, const vector3& x,
+                              const vector3& y, const vector3& z)
+  {
+    // Using vectors instead of arrays of doubles
+    _xmin = origin.x();
+    _ymin = origin.y();
+    _zmin = origin.z();
+
+    // Set the axes
+    SetXAxis(x);
+    SetYAxis(y);
+    SetZAxis(z);
+
+    // Now set the max and mid as well as spacing
+    _xmax = _xmin + _xdim * x.x() + _xdim * y.x() + _xdim * z.x();
+    _ymax = _ymin + _ydim * x.y() + _ydim * y.y() + _ydim * z.y();
+    _zmax = _zmin + _zdim * x.z() + _zdim * y.z() + _zdim * z.z();
+    _spacing = (_xmax - _xmin)/_xdim + (_ymax - _ymin)/_ydim +
+               (_zmax - _zmin)/_zdim;
+    _spacing /= 3.0;
+    _halfSpace= _spacing/2.0;
+    _inv_spa=1.0/_spacing;
+  }
+
+  void OBFloatGrid::SetNumberOfPoints(int nx, int ny, int nz)
+  {
+    _xdim = nx;
+    _ydim = ny;
+    _zdim = nz;
+    _values.resize(nx*ny*nz);
+  }
+
+  void OBFloatGrid::SetXAxis(vector3 v)
+  {
+    _xAxis = v;
+  }
+
+  void OBFloatGrid::SetYAxis(vector3 v)
+  {
+    _yAxis = v;
+  }
+
+  void OBFloatGrid::SetZAxis(vector3 v)
+  {
+    _zAxis = v;
+  }
+
+  double OBFloatGrid::Inject(double x, double y, double z)
+  {
+    if (_values.empty())
+      return 0.0;
+
     if( x<=_xmin || x>=_xmax
         || y<=_ymin || y>=_ymax
         || z<=_zmin || z>=_zmax ) return 0.0;
-  
-    int gx=(int)((x-_xmin)*_inv_spa);
-    int gy=(int)((y-_ymin)*_inv_spa);
-    int gz=(int)((z-_zmin)*_inv_spa);
-  
-    return(_val[(gz*_ydim*_xdim)+(gy*_xdim)+gx]);
+
+    return(_values[CoordsToIndex(x, y, z)]);
   }
 
   void OBFloatGrid::IndexToCoords(int idx, double &x, double &y, double &z)
   {
-    int grid_x,grid_y,grid_z;
+    int grid_x, grid_y, grid_z;
 
-    grid_x = idx % (int)_xdim;
-    grid_z = (int)(idx /(_xdim * _ydim));
-    grid_y = (int)((idx - (grid_z * _xdim * _ydim))/_xdim);
+    grid_z = idx % _zdim;
+    grid_x = static_cast<int>(idx / (_ydim * _zdim));
+    grid_y = static_cast<int>((idx - (grid_x * _ydim * _zdim)) / _zdim);
 
     x = (grid_x * _spacing + _xmin) + _halfSpace;
     y = (grid_y * _spacing + _ymin) + _halfSpace;
     z = (grid_z * _spacing + _zmin) + _halfSpace;
   }
 
-  int OBFloatGrid::CoordsToIndex(double &x, double &y, double &z)
+  int OBFloatGrid::CoordsToIndex(double x, double y, double z)
   {
-    int gx=(int)((x-_xmin)*_inv_spa);
-    int gy=(int)((y-_ymin)*_inv_spa);
-    int gz=(int)((z-_zmin)*_inv_spa);
+    int gx=static_cast<int>((x-_xmin)*_inv_spa);
+    int gy=static_cast<int>((y-_ymin)*_inv_spa);
+    int gz=static_cast<int>((z-_zmin)*_inv_spa);
 
-    return((gz*_ydim*_xdim)+(gy*_xdim)+gx);
+    return((gx*_ydim*_zdim) + (gy*_zdim) + gz);
   }
 
   void OBFloatGrid::CoordsToIndex(int *idx,double *c)
   {
-    idx[0]=(int)((c[0]-_xmin)*_inv_spa);
-    idx[1]=(int)((c[1]-_ymin)*_inv_spa);
-    idx[2]=(int)((c[2]-_zmin)*_inv_spa);
+    idx[0]=static_cast<int>((c[0]-_xmin)*_inv_spa);
+    idx[1]=static_cast<int>((c[1]-_ymin)*_inv_spa);
+    idx[2]=static_cast<int>((c[2]-_zmin)*_inv_spa);
   }
 
-  double OBFloatGrid::Interpolate(double x,double y,double z)
+  double OBFloatGrid::Interpolate(double x, double y, double z)
   {
+    if (_values.empty())
+      return 0.0;
+
     int n,igx,igy,igz;
-    double xydim;
+    double yzdim;
     double gx,gy,gz,fgx,fgy,fgz;
     double ax,ay,az,bx,by,bz;
     double AyA,ByA,AyB,ByB,Az,Bz;
@@ -145,26 +200,31 @@ namespace OpenBabel
         || y<=_ymin || y>=_ymax
         || z<=_zmin || z>=_zmax ) return 0.0;
 
-    xydim = _xdim*_ydim;
+    yzdim = _ydim*_zdim;
 
     /* calculate grid voxel and fractional offsets */
     gx=(x-_xmin-_halfSpace)*_inv_spa;
     if (gx<0)
       gx=0;
-    igx=(int)gx;
-    fgx=gx-(double)igx;
+    igx=static_cast<int>(gx);
+    fgx=gx-static_cast<double>(igx);
     gy=(y-_ymin-_halfSpace)*_inv_spa;
     if (gy<0)
       gy=0;
-    igy=(int) gy;
-    fgy= gy - (double) igy;
+    igy=static_cast<int>(gy);
+    fgy= gy - static_cast<double>(igy);
     gz=(z-_zmin-_halfSpace)*_inv_spa;
     if (gz<0)
       gz=0;
-    igz=(int) gz;
-    fgz= gz - (double) igz;
+    igz=static_cast<int>(gz);
+    fgz= gz - static_cast<double>(igz);
+
+    // Calculate the index of the nearest point
+    n=static_cast<int>(igx*yzdim + igy*_zdim + igz);
+
+    if ((n+1+_zdim+yzdim) >= (yzdim*_xdim))
+      return 0.0;
 
-    n=(int)(igx+ _xdim*igy + xydim*igz);
     /* calculate linear weightings */
     ax=1.0-fgx;
     bx=fgx;
@@ -174,23 +234,36 @@ namespace OpenBabel
     bz=fgz;
 
     /* calculate interpolated value */
-    AyA=ax*_val[n           ]+bx*_val[(int)(n+1)    ];
-    ByA=ax*_val[n+_xdim]+bx*_val[(int)(n+1+_xdim)  ];
+    AyA=az*_values[n           ]+bz*_values[(n+1)    ];
+    ByA=az*_values[n+_zdim]+bz*_values[(n+1+_zdim)  ];
 
     Az=ay*AyA+by*ByA;
 
-    AyB=ax*_val[(int)(n     +xydim)]+bx*_val[(int)(n+1     +xydim)];
-    ByB=ax*_val[(int)(n+_xdim+xydim)]+bx*_val[(int)(n+1+_xdim+xydim)];
+    AyB=az*_values[static_cast<int>(n     +yzdim)] +
+        bz*_values[static_cast<int>(n+1     +yzdim)];
+    ByB=az*_values[static_cast<int>(n+_zdim+yzdim)] +
+        bz*_values[static_cast<int>(n+1+_zdim+yzdim)];
     Bz=ay*AyB+by*ByB;
 
-    return(az*Az+bz*Bz);
+    return(ax*Az+bx*Bz);
   }
 
+  std::vector<double> OBFloatGrid::GetDataVector()
+  {
+    return _values;
+  }
+
+  void OBFloatGrid::SetVals(const std::vector<double> & vals)
+  {
+    int size = _xdim*_ydim*_zdim;
+    _values.clear();
+    _values = vals;
+  }
 
   double OBFloatGrid::InterpolateDerivatives(double x,double y,double z,double *derivatives)
   {
     int n,igx,igy,igz;
-    double xydim;
+    double yzdim;
     double gx,gy,gz,fgx,fgy,fgz;
     double ax,ay,az,bx,by,bz;
     double AyA,ByA,AyB,ByB,Az,Bz;
@@ -200,26 +273,26 @@ namespace OpenBabel
         || y<=_ymin || y>=_ymax
         || z<=_zmin || z>=_zmax ) return 0.0;
 
-    xydim = _xdim*_ydim;
+    yzdim = _ydim*_zdim;
 
     /* calculate grid voxel and fractional offsets */
     gx=(x-_xmin-_halfSpace)*_inv_spa;
     if (gx<0)
       gx=0;
-    igx=(int)gx;
+    igx=static_cast<int>(gx);
     fgx=gx-(double)igx;
     gy=(y-_ymin-_halfSpace)*_inv_spa;
     if (gy<0)
       gy=0;
-    igy=(int) gy;
+    igy=static_cast<int>(gy);
     fgy= gy - (double) igy;
     gz=(z-_zmin-_halfSpace)*_inv_spa;
     if (gz<0)
       gz=0;
-    igz=(int) gz;
+    igz=static_cast<int>(gz);
     fgz= gz - (double) igz;
 
-    n=(int)(igx+ _xdim*igy + xydim*igz);
+    n=static_cast<int>(igx*yzdim + igy*_zdim + igz);
     /* calculate linear weightings */
     ax=1.0-fgx;
     bx=fgx;
@@ -229,16 +302,18 @@ namespace OpenBabel
     bz=fgz;
 
     /* calculate interpolated value */
-    AyA=ax*_val[n           ]+bx*_val[(int)(n+1)    ];
-    ByA=ax*_val[n+_xdim]+bx*_val[(int)(n+1+_xdim)  ];
+    AyA=az*_values[n           ]+bx*_values[n+1    ];
+    ByA=az*_values[n+_zdim]+bx*_values[(n+1+_zdim)  ];
 
     Az=ay*AyA+by*ByA;
 
-    AyB=ax*_val[(int)(n      +xydim)]+bx*_val[(int)(n+1      +xydim)];
-    ByB=ax*_val[(int)(n+_xdim+xydim)]+bx*_val[(int)(n+1+_xdim+xydim)];
+    AyB=az*_values[static_cast<int>(n      +yzdim)]+
+        bz*_values[static_cast<int>(n+1      +yzdim)];
+    ByB=az*_values[static_cast<int>(n+_zdim+yzdim)]+
+        bz*_values[static_cast<int>(n+1+_zdim+yzdim)];
     Bz=ay*AyB+by*ByB;
 
-    energy = az*Az+bz*Bz;
+    energy = ax*Az+bx*Bz;
 
     //calculate derivatives
 
@@ -249,34 +324,34 @@ namespace OpenBabel
 
     fy=az*Az+bz*Bz;
 
-    AyA=-_val[n           ]+_val[(int)(n+1)     ];
-    ByA=-_val[n+_xdim      ]+_val[(int)(n+1+_xdim)];
+    AyA=-_values[n           ]+_values[(n+1)     ];
+    ByA=-_values[n+_zdim      ]+_values[(n+1+_zdim)];
 
     Az=ay*AyA+by*ByA;
 
-    AyB=-_val[(int)(n      +xydim)]+_val[(int)(n+1      +xydim)];
-    ByB=-_val[(int)(n+_xdim+xydim)]+_val[(int)(n+1+_xdim+xydim)];
+    AyB=-_values[static_cast<int>(n      +yzdim)]+
+         _values[static_cast<int>(n+1      +yzdim)];
+    ByB=-_values[static_cast<int>(n+_zdim+yzdim)]+
+         _values[static_cast<int>(n+1+_zdim+yzdim)];
 
     Bz=ay*AyB+by*ByB;
 
-    fx=az*Az+bz*Bz;
+    fx=ax*Az+bx*Bz;
 
     derivatives[0] += fx;
     derivatives[1] += fy;
     derivatives[2] += fz;
 
     return(energy);
-
   }
 
-
   ostream& operator<< ( ostream &os, const  OBFloatGrid& fg)
   {
     //FIXME: this code stores the data in way that depends on
     // the bit representation of floating-point numbers. One can say
     // it's OK because IEEE754 is a widely accepted standard, but then
     // one should at least make it write the data in an endianness-
-    // independant way. Of course this comment applies also to
+    // independent way. Of course this comment applies also to
     // the operator>> below.
     os.write((const char*)&fg._xmin,sizeof(double));
     os.write((const char*)&fg._xmax,sizeof(double));
@@ -293,7 +368,7 @@ namespace OpenBabel
     os.write((const char*)&fg._xdim,sizeof(int));
     os.write((const char*)&fg._ydim,sizeof(int));
     os.write((const char*)&fg._zdim,sizeof(int));
-    os.write((const char*)&fg._val[0],
+    os.write((const char*)&fg._values[0],
              (sizeof(double)*(fg._xdim*fg._ydim*fg._zdim)));
 
     return(os);
@@ -317,10 +392,10 @@ namespace OpenBabel
     is.read((char*)&fg._ydim,sizeof(int));
     is.read((char*)&fg._zdim,sizeof(int));
     int size = fg._xdim*fg._ydim*fg._zdim;
-    fg._val = new double [size];
+    fg._values.resize(size);
     size *= (int) sizeof(double);
 
-    is.read((char*)&fg._val[0],size);
+    is.read((char*)&fg._values[0],size);
     fg._halfSpace= fg._spacing/2.0;
 
     return(is);
@@ -368,7 +443,7 @@ namespace OpenBabel
                          double res)
   {
     this->Init(box); // handle in the base class
-    
+
     _inc = res; // 1/2 angstrom resolution
 
     _nxinc = (int) floor((_xmax - _xmin)/0.5);
