@@ -34,8 +34,9 @@ namespace OpenBabel
   ///////////////////////////////////////////////////////////////////////////////
   //! \brief Kekulize aromatic rings without using implicit valence
 
-  //! This new perceive kekule bonds function has been especifically designed to 
+  //! This new perceive kekule bonds function has been designed to 
   //! handle molecule files without explicit hydrogens such as pdb or xyz.
+  //! (It can, of course, easily handle explicit hydrogens too.)
   //! The function does not rely on GetImplicitValence function
   //! The function looks for groups of aromatic cycle 
   //! For each group it tries to guess the number of electrons given by each atom
@@ -57,7 +58,8 @@ namespace OpenBabel
     OBBond *bond;
     std::vector<OBBond*>::iterator bi;
     std::vector<int> electron;
-    int sume, orden, bestatom;
+    int sume, orden;
+    int bestatom = 1;
     int bestorden = 99;
     // Init the kekulized bonds
     unsigned i;
@@ -65,9 +67,9 @@ namespace OpenBabel
       {
         switch (bond->GetBO())
           {
-          case 1: bond->SetKSingle(); break;
           case 2: bond->SetKDouble(); break;
           case 3: bond->SetKTriple(); break;
+          case 1: bond->SetKSingle(); break;
           }
       }
 
@@ -90,7 +92,14 @@ namespace OpenBabel
             cycle.push_back(atom);
           }
         }
-        // At the begining each atom give one electron to the cycle
+
+        // This isn't a real aromatic cycle -- give up
+        // Fixes PR#1965566
+        // Fixes PR#1784204
+        if (cycle.size() < 3)
+          continue;
+
+        // At the beginning each atom give one electron to the cycle
         for(j=0; j< cycle.size(); ++j) {
           electron.push_back(1);
         }
@@ -120,7 +129,6 @@ namespace OpenBabel
           sume += electron[j];
         }
 
-
         // Save the electron state in case huckel rule is not satisfied
         vector<int> previousElectron = electron;
 
@@ -293,7 +301,7 @@ namespace OpenBabel
     //       }
     //     }
     //   }
-    bool expand_successful;
+    bool expand_successful=true;//was previously not initialized. true works but may not be correct
     atom = cycle[0];
     for (nbr = atom->BeginNbrAtom(b);nbr;nbr = atom->NextNbrAtom(b)) {
       if(initState[nbr->GetIdx()] == -1) //neighbor atom not in the cycle, try next one
@@ -475,11 +483,11 @@ namespace OpenBabel
     else {
       // if we found a good solution, then the state of Idx2 may have shifted from 1 to 0 during the kekulization
       // If it is the case, we should check if there is a remaining unmarked neighbor because it is possible
-      // that kekulizing from this neigbor failed just because Idx2 was equal to 1
+      // that kekulizing from this neighbor failed just because Idx2 was equal to 1
 
       if(previousState[Idx2] == 1) {
-        // Since now Idx2 is equal to 0 because it kekulized well the kekulizing of the failed neigbor could be successfull
-        // If there is still an unmarked neigbor try to kekulize it again
+        // Since now Idx2 is equal to 0 because it kekulized well the kekulizing of the failed neighbor could be successful
+        // If there is still an unmarked neighbor try to kekulize it again
         //mark[Idx2]=true;	
         return_false=false;
         //cout << "retry kekulizing from " << Idx2 << endl;
@@ -500,14 +508,14 @@ namespace OpenBabel
     
           }
   
-        // if we cannot kekulize the remaining neigbor again then we have to return false
+        // if we cannot kekulize the remaining neighbor again then we have to return false
         // we do not have to reset the states because the kekulize will fail anyway
         if(return_false) {
           //cout << "rekekulize failed" << endl;
           return(false);
         }
         else {
-          //cout << "rekekulized successfull" << endl;
+          //cout << "rekekulized successful" << endl;
           return (true);
         }
 	  
@@ -537,7 +545,7 @@ namespace OpenBabel
   }
 
   //! Recursively find the aromatic atoms with an aromatic bond to the current atom
-  void OBMol::expandcycle (OBAtom *atom, OBBitVec &avisit)
+  bool OBMol::expandcycle (OBAtom *atom, OBBitVec &avisit, OBAtom *, int)
   {
     OBAtom *nbr;
     //  OBBond *bond;
@@ -554,6 +562,8 @@ namespace OpenBabel
           expandcycle(nbr, avisit);
         }
       }
+
+    return true;
   }
 
 } // end namespace OpenBabel
