@@ -18,7 +18,6 @@ GNU General Public License for more details.
 ***********************************************************************/
 // Definition of OBConversion routines
 #include <openbabel/babelconfig.h>
-#include <climits>
 
 #ifdef _WIN32
 	#pragma warning (disable : 4786)
@@ -37,8 +36,12 @@ GNU General Public License for more details.
 #include <sstream>
 #include <string>
 #include <map>
+#include <locale>
+
+#include <stdlib.h>
 
 #include <openbabel/obconversion.h>
+#include <openbabel/locale.h>
 
 #ifdef HAVE_LIBZ
 #include "zipstream.h"
@@ -75,23 +78,6 @@ namespace OpenBabel {
       destruction of objects in ReadMolecule() and WriteMolecule() and no restriction
       on whether the pointers are to the heap or the stack.
   **/
-  const char* OBFormat::TargetClassDescription()
-  {
-    //Provides class of default format unless overridden
-    if(OBConversion::GetDefaultFormat())
-      return OBConversion::GetDefaultFormat()->TargetClassDescription();
-    else
-      return "";
-  }
-  const type_info& OBFormat::GetType()
-  {
-    //Provides info on class of default format unless overridden
-    if(OBConversion::GetDefaultFormat())
-      return OBConversion::GetDefaultFormat()->GetType();
-    else
-      return typeid(this); //rubbish return if DefaultFormat not set
-  }
-
   //***************************************************
 
   /** @class OBConversion obconversion.h <openbabel/obconversion.h>
@@ -171,7 +157,7 @@ namespace OpenBabel {
       OBConversion conv(&is,&os);
       if(conv.SetInAndOutFormats("SMI","MOL"))
       {
-         conv.SetOptions("h"); //Optional; (h adds expicit hydrogens)
+         conv.AddOption("h",OBConversion::GENOPTIONS); //Optional; (h adds expicit hydrogens)
          conv.Convert();
       }
       @endcode
@@ -180,7 +166,7 @@ namespace OpenBabel {
 
       The existing program inputs from the file identified by the 
       const char* filename into the istream is. The file is assumed to have
-      a format ORIG, but otherformats, identified by their file extensions,
+      a format ORIG, but other formats, identified by their file extensions,
       can now be used.
 
       @code
@@ -202,22 +188,22 @@ namespace OpenBabel {
       ...Carry on with original code using pIn
       @endcode
 
-      In Windows a degree of independence from OpenBabel can be achieved using DLLs.
-      This code would be linked with obconv.lib.
+      In certain Windows builds, a degree of independence from OpenBabel can be
+      achieved using DLLs. This code would be linked with obconv.lib.
       At runtime the following DLLs would be in the executable directory:
       obconv.dll, obdll.dll, one or more *.obf format files.
   */
     
   int OBConversion::FormatFilesLoaded = 0;
 
-  OBFormat* OBConversion::pDefaultFormat=NULL;
+//  OBFormat* OBConversion::pDefaultFormat=NULL;
 
   OBConversion::OBConversion(istream* is, ostream* os) : 
     pInFormat(NULL),pOutFormat(NULL), Index(0), StartNumber(1),
     EndNumber(0), Count(-1), m_IsFirstInput(true), m_IsLast(true),
     MoreFilesToCome(false), OneObjectOnly(false), CheckedForGzip(false),
     NeedToFreeInStream(false), NeedToFreeOutStream(false), 
-    pOb1(NULL), pAuxConv(NULL),pLineEndBuf(NULL)
+    pOb1(NULL), pAuxConv(NULL),pLineEndBuf(NULL),wInpos(0),wInlen(0)
   {
     pInStream=is;
     pOutStream=os;
@@ -229,26 +215,6 @@ namespace OpenBabel {
     RegisterOptionParam("l", NULL, 1,GENOPTIONS);
   }
 
-  ///This static function returns a reference to the FormatsMap
-  ///which, because it is a static local variable is constructed only once.
-  ///This fiddle is to avoid the "static initialization order fiasco"
-  ///See Marshall Cline's C++ FAQ Lite document, www.parashift.com/c++-faq-lite/". 
-  FMapType& OBConversion::FormatsMap()
-  {
-    static FMapType* fm = new FMapType;
-    return *fm;
-  }
-
-  ///This static function returns a reference to the FormatsMIMEMap
-  ///which, because it is a static local variable is constructed only once.
-  ///This fiddle is to avoid the "static initialization order fiasco"
-  ///See Marshall Cline's C++ FAQ Lite document, www.parashift.com/c++-faq-lite/". 
-  FMapType& OBConversion::FormatsMIMEMap()
-  {
-    static FMapType* fm = new FMapType;
-    return *fm;
-  }
-
   /////////////////////////////////////////////////
   OBConversion::OBConversion(const OBConversion& o)
   {
@@ -285,14 +251,22 @@ namespace OpenBabel {
   OBConversion::~OBConversion() 
   {
     if(pAuxConv!=this)
-      delete pAuxConv;
+      if(pAuxConv)
+      {
+        delete pAuxConv;
+        //pAuxConv has copies of pInStream, NeedToFreeInStream, pOutStream, NeedToFreeOutStream
+        //and may have already deleted the streams. So do not do it again.
+        NeedToFreeInStream = NeedToFreeOutStream = false;
+      }
     // Free any remaining streams from convenience functions
     if(pInStream && NeedToFreeInStream) {
       delete pInStream;
+      pInStream=NULL;
       NeedToFreeInStream = false;
     }
     if(pOutStream && NeedToFreeOutStream) {
       delete pOutStream;
+      pOutStream=NULL;
       NeedToFreeOutStream = false;
     }
   }
@@ -306,12 +280,7 @@ namespace OpenBabel {
   /// the initialization code makes an instance of the imported OBFormat class.
   int OBConversion::RegisterFormat(const char* ID, OBFormat* pFormat, const char* MIME)
   {
-    FormatsMap()[ID] = pFormat;
-    if (MIME)
-      FormatsMIMEMap()[MIME] = pFormat;
-    if(pFormat->Flags() & DEFAULTFORMAT)
-      pDefaultFormat=pFormat;
-    return FormatsMap().size();
+    return pFormat->RegisterFormat(ID, MIME);
   }
 
   //////////////////////////////////////////////////////
@@ -320,7 +289,7 @@ namespace OpenBabel {
     int count=0;
     //	if(FormatFilesLoaded) return 0;
     //	FormatFilesLoaded=true; //so will load files only once
-#ifdef USING_DYNAMIC_LIBS
+#if  defined(USING_DYNAMIC_LIBS)
     //Depending on availablilty, look successively in 
     //FORMATFILE_DIR, executable directory,or current directory
     string TargetDir;
@@ -344,60 +313,21 @@ namespace OpenBabel {
 #else
     count = 1; //avoid calling this function several times
 #endif //USING_DYNAMIC_LIBS
+
+    //Make instances for plugin classes defined in the data file
+    //This is hook for OBDefine, but does nothing if it is not loaded
+    //or if plugindefines.txt is not found.
+    OBPlugin* pdef = OBPlugin::GetPlugin("loaders","define");
+    if(pdef)
+    {
+      static vector<string> vec(3);
+      vec[1] = string("define");
+      vec[2] = string("plugindefines.txt"); 
+      pdef->MakeInstance(vec);
+    }
     return count;
   }
 
-  /**
-   *Returns the ID + the first line of the description in str 
-   *and a pointer to the format in pFormat.
-   *If called with str==NULL the first format is returned;
-   *subsequent formats are returned by calling with str!=NULL and the previous value of itr
-   *returns false, and str and pFormat NULL, when there are no more formats.
-   *Use like:
-   *@code
-   *	const char* str=NULL;
-   *	Formatpos pos;
-   *	OBConversion conv; // dummy to make sure static data is available
-   *	while(OBConversion::GetNextFormat(pos,str,pFormat))
-   *	{
-   *		// use str and pFormat
-   *	}
-   *@endcode
-   *
-   * NOTE: Because of dynamic loading problems, it is usually necessary to 
-   *  declare a "dummy" OBConversion object to access this static method.
-   *  (Not elegant, but will hopefully be fixed in the future.)
-   */
-  bool OBConversion::GetNextFormat(Formatpos& itr, const char*& str,OBFormat*& pFormat)
-  {
-
-    pFormat = NULL;
-    if(str==NULL) 
-      itr = FormatsMap().begin();
-    else
-      itr++;
-    if(itr == FormatsMap().end())
-      {
-        str=NULL; pFormat=NULL;
-        return false;
-      }
-    static string s;
-    s =itr->first;
-    pFormat = itr->second;
-    if(pFormat)
-      {
-        string description(pFormat->Description());
-        s += " -- ";
-        s += description.substr(0,description.find('\n'));
-      }
-
-    if(pFormat->Flags() & NOTWRITABLE) s+=" [Read-only]";
-    if(pFormat->Flags() & NOTREADABLE) s+=" [Write-only]";
-
-    str = s.c_str();
-    return true;
-  }
-
   //////////////////////////////////////////////////////
   /// Sets the formats from their ids, e g CML.
   /// If inID is NULL, the input format is left unchanged. Similarly for outID
@@ -465,7 +395,7 @@ namespace OpenBabel {
       else
         delete zIn;
     }
-
+#ifndef DISABLE_WRITE_COMPRESSION //Unsolved problem with compression under Windows
     zlib_stream::zip_ostream zOut(*pOutStream);
     if(IsOption("z",GENOPTIONS))
       {
@@ -473,6 +403,7 @@ namespace OpenBabel {
         zOut.make_gzip();
         pOutStream = &zOut;
       }
+#endif
 #endif
 
     //The FilteringInputStreambuf delivers characters to the istream, pInStream,
@@ -527,12 +458,15 @@ namespace OpenBabel {
     pOb1=NULL;
     wInlen=0;
 
+    if(pInFormat->Flags() & READONEONLY)
+      OneObjectOnly=true;
+
     //Input loop
     while(ReadyToInput && pInStream->good()) //Possible to omit? && pInStream->peek() != EOF 
       {
         if(pInStream==&cin)
           {
-            if(pInStream->peek()=='\n')
+            if(pInStream->peek()==-1) //Cntl Z Was \n but interfered with piping
               break;
           }
         else
@@ -542,18 +476,21 @@ namespace OpenBabel {
         try
           {
             ret = pInFormat->ReadChemObject(this);
-            m_IsFirstInput=false;
+            SetFirstInput(false);
           }		
         catch(...)
           {
             if(!IsOption("e", GENOPTIONS) && !OneObjectOnly)
+            {
+              obErrorLog.ThrowError(__FUNCTION__, "Convert failed with an exception" , obError);
               return Index; // the number we've actually output so far
+            }
           }
 
         if(!ret)
           {
             //error or termination request: terminate unless
-            // -e option requested and sucessfully can skip past current object
+            // -e option requested and successfully can skip past current object
             if(!IsOption("e", GENOPTIONS) || pInFormat->SkipObjects(0,this)!=1) 
               break;
           }
@@ -634,13 +571,13 @@ namespace OpenBabel {
   ///    Count is incremented with each call, even if pOb=NULL. 
   ///    Objects are not added to the queue if the count is outside the range  
   ///    StartNumber to EndNumber. There is no upper limit if EndNumber is zero. 
-  ///    The return value is the return code from WriteChemObject - false on error.
-  bool OBConversion::AddChemObject(OBBase* pOb)
+  ///    The return value is Count ((>0) or 0 if WriteChemObject returned false.
+  int OBConversion::AddChemObject(OBBase* pOb)
   {
     if(Count<0) 
       {
         pOb1=pOb;
-        return Count!=0;
+        return Count; // <0
       }
     Count++;
     if(Count>=(int)StartNumber)//keeps reading objects but does nothing with them
@@ -661,7 +598,7 @@ namespace OpenBabel {
                     --Index;
                     //ReadyToInput=false;
                     pOb1=NULL;
-                    return false;
+                    return 0;
                   }
                 //Stop after writing with single object output files
                 if(pOutFormat->Flags() & WRITEONEONLY)
@@ -685,7 +622,7 @@ namespace OpenBabel {
 
                     ReadyToInput = false;
                     pOb1 = NULL;
-                    return true;
+                    return Count; // >0
                   }
               }
             pOb1=pOb;
@@ -693,7 +630,7 @@ namespace OpenBabel {
             wInlen = rInlen;
           }
       }
-    return true;
+    return Count; // >0
   }
   //////////////////////////////////////////////////////
   ///Returns the number of objects which have been output or are currently being output.
@@ -711,11 +648,7 @@ namespace OpenBabel {
   //////////////////////////////////////////////////////
   OBFormat* OBConversion::FindFormat(const char* ID)
   {
-    //Case insensitive
-    if(FormatsMap().find(ID) == FormatsMap().end())
-      return NULL;
-    else
-      return FormatsMap()[ID];
+    return OBFormat::FindType(ID);
   }
 
   //////////////////////////////////////////////////
@@ -745,7 +678,7 @@ namespace OpenBabel {
       {
         // only do this if we actually can read .gz files
 #ifdef HAVE_LIBZ
-        if (file.substr(extPos,3) == ".gz")
+        if (file.substr(extPos) == ".gz")
           {
             file.erase(extPos);
             extPos = file.rfind('.');
@@ -761,10 +694,7 @@ namespace OpenBabel {
 
   OBFormat* OBConversion::FormatFromMIME(const char* MIME)
   {
-    if(FormatsMIMEMap().find(MIME) == FormatsMIMEMap().end())
-      return NULL;
-    else
-      return FormatsMIMEMap()[MIME];
+    return OBFormat::FormatFromMIME(MIME);
   }
 
   bool	OBConversion::Read(OBBase* pOb, std::istream* pin)
@@ -792,7 +722,23 @@ namespace OpenBabel {
 #endif
 
     InstallStreamFilter();
-    return pInFormat->ReadMolecule(pOb, this);
+
+    // Set the locale for number parsing to avoid locale issues: PR#1785463
+    obLocale.SetLocale();
+    
+    // Also set the C++ stream locale
+    locale originalLocale = pInStream->getloc(); // save the original
+    locale cNumericLocale(originalLocale, "C", locale::numeric);
+    pInStream->imbue(cNumericLocale);
+
+    bool success = pInFormat->ReadMolecule(pOb, this);
+
+    // return the C locale to the original one
+    obLocale.RestoreLocale();
+    // Restore the original C++ locale as well
+    pInStream->imbue(originalLocale);
+
+    return success;
   }
 
     void OBConversion::InstallStreamFilter()
@@ -822,7 +768,7 @@ namespace OpenBabel {
 
     ostream* pOrigOutStream = pOutStream;
 #ifdef HAVE_LIBZ
-#ifndef _WIN32
+#ifndef DISABLE_WRITE_COMPRESSION
     zlib_stream::zip_ostream zOut(*pOutStream);
     if(IsOption("z",GENOPTIONS))
       {
@@ -832,10 +778,25 @@ namespace OpenBabel {
       }
 #endif
 #endif
+    SetOneObjectOnly(); //So that IsLast() returns true, which is important for XML formats
+
+    // Set the locale for number parsing to avoid locale issues: PR#1785463
+    obLocale.SetLocale();
+    // Also set the C++ stream locale
+    locale originalLocale = pOutStream->getloc(); // save the original
+    locale cNumericLocale(originalLocale, "C", locale::numeric);
+    pOutStream->imbue(cNumericLocale);
+
+    // The actual work is done here
+    bool success = pOutFormat->WriteMolecule(pOb,this);
 
-    bool ret = pOutFormat->WriteMolecule(pOb,this);
     pOutStream = pOrigOutStream;
-    return ret;
+    // return the C locale to the original one
+    obLocale.RestoreLocale();
+    // Restore the C++ stream locale too
+    pOutStream->imbue(originalLocale);
+
+    return success;
   }
 
   //////////////////////////////////////////////////
@@ -920,6 +881,9 @@ namespace OpenBabel {
   {
     if(!pInFormat) return false;
 
+    // save the filename
+    InFilename = filePath;
+
     // if we have an old stream, free this first before creating a new one
     if (pInStream && NeedToFreeInStream) {
       delete pInStream;
@@ -940,25 +904,56 @@ namespace OpenBabel {
     return Read(pOb,ifs);
   }
 
+  ////////////////////////////////////////////
+  bool OBConversion::OpenInAndOutFiles(std::string infilepath, std::string outfilepath)
+  {
+    // if we have an old input stream, free this first before creating a new one
+    if (pInStream && NeedToFreeInStream)
+      delete pInStream;
+
+    // if we have an old output stream, free this first before creating a new one
+    if (pOutStream && NeedToFreeOutStream)
+      delete pOutStream;
+
+    ifstream *ifs = new ifstream;
+    NeedToFreeInStream = true; // make sure we free this
+    ifs->open(infilepath.c_str(),ios_base::in|ios_base::binary); //always open in binary mode
+    if(!ifs || !ifs->good())
+    {
+      obErrorLog.ThrowError(__FUNCTION__,"Cannot read from " + infilepath, obError);
+      return false;
+    }
+    pInStream = ifs;
+    InFilename = infilepath;
+
+    ofstream *ofs = new ofstream;
+    NeedToFreeOutStream = true; // make sure we clean this up later
+    ofs->open(outfilepath.c_str(),ios_base::out|ios_base::binary);//always open in binary mode
+    if(!ofs || !ofs->good())
+    {
+      obErrorLog.ThrowError(__FUNCTION__,"Cannot write to " + outfilepath, obError);
+      return false;
+    }
+    pOutStream = ofs;
+
+    return true;
+  }
 
   ////////////////////////////////////////////
   const char* OBConversion::Description()
   {
-#ifdef HAVE_LIBZ
-    return "Conversion options\n \
- -f <#> Start import at molecule # specified\n \
- -l <#> End import at molecule # specified\n \
- -e Continue with next object after error, if possible\n \
- -z Compress the output with gzip\n \
- -k Attempt to translate keywords\n";
-#else
-    return "Conversion options\n \
- -f <#> Start import at molecule # specified\n \
- -l <#> End import at molecule # specified\n \
- -e Continue with next object after error, if possible\n \
- -k Attempt to translate keywords\n";
-#endif
-    // -t All input files describe a single molecule
+    return 
+      "Conversion options\n"
+      "-f <#> Start import at molecule # specified\n"
+      "-l <#> End import at molecule # specified\n"
+      "-e Continue with next object after error, if possible\n"
+      #ifdef HAVE_LIBZ
+      #ifndef DISABLE_WRITE_COMPRESSION //Unsolved problem with compression under Windows
+      "-z Compress the output with gzip\n"
+      #endif
+      #endif
+      "-k Attempt to translate keywords\n";
+      // -t All input files describe a single molecule
   }
 
   ////////////////////////////////////////////
@@ -971,6 +966,16 @@ namespace OpenBabel {
   {
     return m_IsFirstInput;
   }
+  void OBConversion::SetFirstInput(bool b)
+  {
+    m_IsFirstInput = b;
+/*    //Also set or clear a general option
+    if(b)
+      AddOption("firstinput",GENOPTIONS);
+    else
+      RemoveOption("firstinput",GENOPTIONS);
+  */
+  }
 
   /////////////////////////////////////////////////
   string OBConversion::BatchFileName(string& BaseName, string& InFile)
@@ -986,7 +991,7 @@ namespace OpenBabel {
           posdot = InFile.size();
         else {
 #ifdef HAVE_LIBZ
-          if (InFile.substr(posdot,3) == ".gz")
+          if (InFile.substr(posdot) == ".gz")
             {
               InFile.erase(posdot);
               posdot = InFile.rfind('.');
@@ -1022,7 +1027,7 @@ namespace OpenBabel {
     //If infile == outfile issue error message and return false
     //If name without the extensions are the same issue warning and return true;
     //Otherwise return true
-    // bool ret=true;
+    bool ret=true;
     string inname1, inname2;
     string::size_type pos;
     pos = infile.rfind('.');
@@ -1089,7 +1094,7 @@ namespace OpenBabel {
     stringstream ssOut;
     bool HasMultipleOutputFiles=false;
     int Count=0;
-    m_IsFirstInput=true;
+    SetFirstInput();
     bool CommonInFormat = pInFormat ? true:false; //whether set in calling routine
     ios_base::openmode omode = 
       pOutFormat->Flags() & WRITEBINARY ? ios_base::out|ios_base::binary : ios_base::out;
@@ -1160,7 +1165,14 @@ namespace OpenBabel {
 
         //INPUT
         if(FileList.empty())
-          pIs = NULL;
+          {
+            pIs = NULL;
+            if(HasMultipleOutputFiles)
+              {
+                obErrorLog.ThrowError(__FUNCTION__,"Cannot use multiple output files without an input file", obError);
+                return 0;
+              }
+          }
         else
           {
             if(FileList.size()>1 || OutputFileName.substr(0,2)=="*.")
@@ -1266,6 +1278,7 @@ namespace OpenBabel {
 						
                         OutputFileList.push_back(incrfile);
 #ifdef HAVE_LIBZ
+#ifndef DISABLE_WRITE_COMPRESSION
                         if(IsOption("z",GENOPTIONS))
                           {
                             zlib_stream::zip_ostream zOut(ofs);
@@ -1274,6 +1287,7 @@ namespace OpenBabel {
                             zOut << ss.rdbuf();
                           }
                         else
+#endif
 #endif
                           ofs << ss.rdbuf();
 
@@ -1433,42 +1447,85 @@ namespace OpenBabel {
    */
   std::vector<std::string> OBConversion::GetSupportedInputFormat()
   {
-    if( SupportedInputFormat.empty() )
-    {
-      Formatpos pos;
-      OBFormat *pFormat;
-      const char* str = NULL;
-      while( GetNextFormat(pos, str, pFormat) )
-      {
-        if( (pFormat->Flags() & NOTREADABLE) )
-        {
-          continue;
-        }
-        SupportedInputFormat.push_back(str);
-      }
-    }
-    return( SupportedInputFormat );
+    vector<string> vlist;
+    OBPlugin::ListAsVector("formats", "in", vlist);
+    return vlist;
   }
   /**
    * Returns the list of supported output format
    */
   std::vector<std::string> OBConversion::GetSupportedOutputFormat()
   {
-    if( SupportedOutputFormat.empty() )
-    {
-      Formatpos pos;
-      OBFormat *pFormat;
-      const char* str = NULL;
-      while( GetNextFormat(pos, str, pFormat) )
+    vector<string> vlist;
+    OBPlugin::ListAsVector("formats", "out", vlist);
+    return vlist;
+  }
+
+  void OBConversion::ReportNumberConverted(int count, OBFormat* pFormat)
+  {
+    //Send info message to clog. This constructed from the TargetClassDescription
+    //of the specified class (or the output format if not specified).
+    //Get the last word on the first line of the description which should
+    //be "molecules", "reactions", etc and remove the s if only one object converted
+    if(!pFormat)
+      pFormat = pOutFormat;
+    string objectname(pFormat->TargetClassDescription());
+    string::size_type pos = objectname.find('\n');
+    if(pos==std::string::npos)
+      pos=objectname.size();
+    if(count==1) --pos;
+    objectname.erase(pos);
+    pos = objectname.rfind(' ');
+    if(pos==std::string::npos)
+      pos=0;
+    std::clog << count << objectname.substr(pos) << " converted" << endl;
+  }
+
+  void OBConversion::CopyOptions(OBConversion* pSourceConv, Option_type typ)
+  {
+    if(typ==ALL)
+    for(int i=0;i<3;++i)
+     OptionsArray[i]=pSourceConv->OptionsArray[i];
+    else
+     OptionsArray[typ]=pSourceConv->OptionsArray[typ];
+  }
+
+  //The following function and typedef are deprecated, and are present only
+  //for backward compatibility.
+  //Use OBConversion::GetSupportedInputFormat(), OBConversion::GetSupportedOutputFormat(),
+  //OBPlugin::List(), OBPlugin::OBPlugin::ListAsVector(),OBPlugin::OBPlugin::ListAsString(),
+  //or (in extremis) OBPlugin::PluginIterator instead.
+
+  typedef OBPlugin::PluginIterator Formatpos;
+
+  bool OBConversion::GetNextFormat(Formatpos& itr, const char*& str,OBFormat*& pFormat)
+  {
+
+    pFormat = NULL;
+    if(str==NULL) 
+      itr = OBPlugin::Begin("formats");
+    else
+      itr++;
+    if(itr == OBPlugin::End("formats"))
       {
-        if( ( pFormat->Flags() & NOTWRITABLE) )
-        {
-          continue;
-        }
-        SupportedOutputFormat.push_back(str);
+        str=NULL; pFormat=NULL;
+        return false;
       }
-    }
-    return( SupportedOutputFormat );
+    static string s;
+    s =itr->first;
+    pFormat = static_cast<OBFormat*>(itr->second);
+    if(pFormat)
+      {
+        string description(pFormat->Description());
+        s += " -- ";
+        s += description.substr(0,description.find('\n'));
+      }
+
+    if(pFormat->Flags() & NOTWRITABLE) s+=" [Read-only]";
+    if(pFormat->Flags() & NOTREADABLE) s+=" [Write-only]";
+
+    str = s.c_str();
+    return true;
   }
 
 }//namespace OpenBabel
