@@ -211,6 +211,8 @@ namespace OpenBabel
             _ptr = _parent->GetAtom(next + 1);
             _notVisited.SetBitOff(next);
           }
+        else
+          _ptr = NULL;
       }
 
     if (_ptr)
@@ -232,7 +234,7 @@ namespace OpenBabel
   OBMolAtomDFSIter OBMolAtomDFSIter::operator++(int)
   {
     OBMolAtomDFSIter tmp(*this);
-    operator++();
+    return operator++();
     return tmp;
   }
 
@@ -276,36 +278,46 @@ namespace OpenBabel
       \endcode
   **/
 
-  OBMolAtomBFSIter::OBMolAtomBFSIter(OBMol *mol):
-    _parent(mol), _ptr(_parent->GetFirstAtom())
+  OBMolAtomBFSIter::OBMolAtomBFSIter(OBMol *mol, int StartIndex):
+    _parent(mol), _ptr(_parent->GetAtom(StartIndex))
   {
     _notVisited.Resize(_parent->NumAtoms());
     _notVisited.Negate(); // all on
     _notVisited.SetBitOff(_ptr->GetIdx() - 1);
 
+    // Set up storage for the depths
+    _depth.resize(_parent->NumAtoms(), 0);
+    _depth[_ptr->GetIdx()] = 1;
+
     vector<OBBond*>::iterator i;
     OBAtom *a;
 
     for (a = _ptr->BeginNbrAtom(i); a; a = _ptr->NextNbrAtom(i))
       {
         _queue.push(a);
+        _depth[a->GetIdx()] = 2;
         _notVisited.SetBitOff(a->GetIdx() - 1);
       }
   }
 
-  OBMolAtomBFSIter::OBMolAtomBFSIter(OBMol &mol):
-    _parent(&mol), _ptr(_parent->GetFirstAtom())
+  OBMolAtomBFSIter::OBMolAtomBFSIter(OBMol &mol, int StartIndex):
+    _parent(&mol), _ptr(_parent->GetAtom(StartIndex))
   {
     _notVisited.Resize(_parent->NumAtoms());
     _notVisited.Negate(); // all on
     _notVisited.SetBitOff(_ptr->GetIdx() - 1);
 
+    // Set up storage for the depths
+    _depth.resize(_parent->NumAtoms() + 1, 0);
+    _depth[_ptr->GetIdx()] = 1;
+
     vector<OBBond*>::iterator i;
     OBAtom *a;
 
     for (a = _ptr->BeginNbrAtom(i); a; a = _ptr->NextNbrAtom(i))
       {
         _queue.push(a);
+        _depth[a->GetIdx()] = 2;
         _notVisited.SetBitOff(a->GetIdx() - 1);
       }
   }
@@ -316,6 +328,7 @@ namespace OpenBabel
     _ptr = ai._ptr;
     _notVisited = ai._notVisited;
     _queue = ai._queue;
+    _depth = ai._depth;
   }
 
   OBMolAtomBFSIter& OBMolAtomBFSIter::operator=(const OBMolAtomBFSIter &ai)
@@ -326,6 +339,7 @@ namespace OpenBabel
         _ptr = ai._ptr;
         _notVisited = ai._notVisited;
         _queue = ai._queue;
+        _depth = ai._depth;
       }
     return *this;
   }
@@ -342,9 +356,13 @@ namespace OpenBabel
         int next = _notVisited.FirstBit();
         if (next != _notVisited.EndBit())
           {
-            _ptr = _parent->GetAtom(next + 1);
+            _ptr = _parent->GetAtom(next + 1); // Atom index issue
+            if (_ptr != NULL)
+              _depth[_ptr->GetIdx()] = 1; // new island
             _notVisited.SetBitOff(next);
           }
+        else
+          _ptr = NULL;
       }
 
     if (_ptr)
@@ -356,6 +374,7 @@ namespace OpenBabel
           if (_notVisited[a->GetIdx() - 1])
             {
               _queue.push(a);
+              _depth[a->GetIdx()] = _depth[_ptr->GetIdx()] + 1;
               _notVisited.SetBitOff(a->GetIdx() - 1);
             }
       }
@@ -369,6 +388,14 @@ namespace OpenBabel
     operator++();
     return tmp;
   }
+  
+  int OBMolAtomBFSIter::CurrentDepth() const
+  {
+    if (_ptr == NULL)
+      return 0;
+      
+    return _depth[_ptr->GetIdx()];
+  }
 
   /** \class OBMolBondIter obiter.h <openbabel/obiter.h>
 
@@ -881,7 +908,9 @@ namespace OpenBabel
     if (!_vangle.empty()) {
       _i = _vangle.begin();
       _angle = *_i;
-    }
+	} else {
+	  _i = _vangle.end();
+	}
   }
 
   OBMolAngleIter::OBMolAngleIter(OBMol &mol)
@@ -894,7 +923,9 @@ namespace OpenBabel
     if (!_vangle.empty()) {
       _i = _vangle.begin();
       _angle = *_i;
-    }
+	} else {
+      _i = _vangle.end();
+	}
   }
 
   OBMolAngleIter::OBMolAngleIter(const OBMolAngleIter &ai)
@@ -976,7 +1007,9 @@ namespace OpenBabel
     if (!_vtorsion.empty()) {
       _i = _vtorsion.begin();
       _torsion = *_i;
-    }
+	} else {
+	  _i = _vtorsion.end();
+	}
   }
 
   OBMolTorsionIter::OBMolTorsionIter(OBMol &mol)
@@ -984,12 +1017,16 @@ namespace OpenBabel
     mol.FindTorsions();
     OBTorsionData *td = (OBTorsionData *) mol.GetData(OBGenericDataType::TorsionData);
     td->FillTorsionArray(_vtorsion);
-    
+
     _parent = &mol;
     if (!_vtorsion.empty()) {
       _i = _vtorsion.begin();
       _torsion = *_i;
-    }
+	} else {
+	  // Avogadro bug #1972244
+	  // always set _i, _i will be compared to _vtorsion.end() in OBMolTorsionIter::bool()
+	  _i = _vtorsion.end();
+	}
   }
 
   OBMolTorsionIter::OBMolTorsionIter(const OBMolTorsionIter &ai)
@@ -1062,88 +1099,72 @@ namespace OpenBabel
 
   OBMolPairIter::OBMolPairIter(OBMol *mol)
   {
-    std::vector<unsigned int> atoms;
-    OBAtom *i, *j;
-    bool not14;
-    
     _parent = mol;
 
-    FOR_ATOMS_OF_MOL(a, mol) {
-      FOR_ATOMS_OF_MOL(b, mol) {
-        if (a->GetIdx() < b->GetIdx()) {
-          not14 = false;
+    bool foundPair = false;
+    OBAtom *a = _parent->BeginAtom(_i);
+    if (!a) 
+      return;
+    OBAtom *b = _parent->BeginAtom(_j);
+    while (!foundPair) {
+      b = _parent->NextAtom(_j);
 
-          i = &*a;
-          FOR_NBORS_OF_ATOM(nbr, i) {
-            if (&*nbr == &*b)
-              not14 = true;
-
-            j = &*nbr;
-            FOR_NBORS_OF_ATOM(nbr2, j)
-              if (&*nbr2 == &*b)
-                not14 = true;
-          }
-
-          if (!not14) {
-	    atoms.clear();
-            atoms.push_back(a->GetIdx());
-	    atoms.push_back(b->GetIdx());
-	    _vpair.push_back(atoms);
-	  }
-	} 
+      if (!b) {
+        a = _parent->NextAtom(_i);
+        if (!a)
+	  return;
+        b = _parent->BeginAtom(_j);
       }
+
+      if (a->GetIdx() >= b->GetIdx()) continue;
+      if (a->IsConnected(b)) continue;
+      if (a->IsOneThree(b)) continue;
+
+      foundPair = true;
     }
 
-    _i = _vpair.begin();
-    if (!_vpair.empty())
-      _pair = *_i;
+    _pair.clear();
+    _pair.push_back(a->GetIdx());
+    _pair.push_back(b->GetIdx());
   }
 
   OBMolPairIter::OBMolPairIter(OBMol &mol)
   {
-    std::vector<unsigned int> atoms;
-    OBAtom *i, *j;
-    bool not14;
- 
     _parent = &mol;
-    
-    FOR_ATOMS_OF_MOL(a, mol) {
-      FOR_ATOMS_OF_MOL(b, mol) {
-        if (a->GetIdx() < b->GetIdx()) {
-          not14 = false;
 
-          i = &*a;
-          FOR_NBORS_OF_ATOM(nbr, i) {
-            if (&*nbr == &*b)
-              not14 = true;
+    bool foundPair = false;
+    OBAtom *a = _parent->BeginAtom(_i);
+    if (!a) 
+      return;
+    OBAtom *b = _parent->BeginAtom(_j);
+    while (!foundPair) {
+      b = _parent->NextAtom(_j);
 
-            j = &*nbr;
-            FOR_NBORS_OF_ATOM(nbr2, j)
-              if (&*nbr2 == &*b)
-                not14 = true;
-          }
-
-          if (!not14) {
-	    atoms.clear();
-            atoms.push_back(a->GetIdx());
-	    atoms.push_back(b->GetIdx());
-	    _vpair.push_back(atoms);
-          }
-        } 
+      if (!b) {
+        a = _parent->NextAtom(_i);
+	if (!a)
+          return;
+        b = _parent->BeginAtom(_j);
       }
+
+      if (a->GetIdx() >= b->GetIdx()) continue;
+      if (a->IsConnected(b)) continue;
+      if (a->IsOneThree(b)) continue;
+
+      foundPair = true;
     }
 
-    _i = _vpair.begin();
-    if (!_vpair.empty())
-      _pair = *_i;
+    _pair.clear();
+    _pair.push_back(a->GetIdx());
+    _pair.push_back(b->GetIdx());
   }
 
   OBMolPairIter::OBMolPairIter(const OBMolPairIter &ai)
   {
     _parent = ai._parent;
     _pair = ai._pair;
-    _vpair = ai._vpair;
     _i = ai._i;
+    _j = ai._j;
   }
 
   OBMolPairIter& OBMolPairIter::operator=(const OBMolPairIter &ai)
@@ -1151,18 +1172,39 @@ namespace OpenBabel
     if (this != &ai) {
       _parent = ai._parent;
       _pair = ai._pair;
-      _vpair = ai._vpair;
       _i = ai._i;
+      _j = ai._j;
     }
     return *this;
   }
 
   OBMolPairIter& OBMolPairIter::operator++()
   {
-    _i++;
+    _pair.clear();
 
-    if (_i != _vpair.end())
-      _pair = *_i;
+    bool foundPair = false;
+    OBAtom *a, *b;
+    a = *_i;
+    while (!foundPair) {
+      b = _parent->NextAtom(_j);
+
+      if (!b) {
+        a = _parent->NextAtom(_i);
+	if (!a)
+          return *this;
+        b = _parent->BeginAtom(_j);
+      }
+
+      if (a->GetIdx() >= b->GetIdx()) continue;
+      if (a->IsConnected(b)) continue;
+      if (a->IsOneThree(b)) continue;
+
+
+      foundPair = true;
+    }
+
+    _pair.push_back(a->GetIdx());
+    _pair.push_back(b->GetIdx());
  
     return *this;
   }
