@@ -203,7 +203,8 @@ namespace OpenBabel
   {
     register AtomExpr *result;
   
-    result = (AtomExpr*)malloc(sizeof(AtomExpr));
+    //result = (AtomExpr*)malloc(sizeof(AtomExpr));
+    result = new AtomExpr;
     return result;
   }
 
@@ -255,7 +256,8 @@ namespace OpenBabel
           }
         if (expr)
           {
-            free(expr);
+            //free(expr);
+	    delete expr;
             expr = (AtomExpr*)NULL;
           }
       }
@@ -335,7 +337,8 @@ namespace OpenBabel
   {
     register BondExpr *result;
   
-    result = (BondExpr*)malloc(sizeof(BondExpr));
+    //result = (BondExpr*)malloc(sizeof(BondExpr));
+    result = new BondExpr;
     return result;
   }
 
@@ -415,7 +418,8 @@ namespace OpenBabel
       
         if (expr)
           {
-            free(expr);
+            //free(expr);
+	    delete expr;
             expr = (BondExpr*)NULL;
           }
       }
@@ -471,7 +475,8 @@ namespace OpenBabel
   {
     Pattern *ptr;
   
-    ptr = (Pattern*)malloc(sizeof(Pattern));
+    //ptr = (Pattern*)malloc(sizeof(Pattern));
+    ptr = new Pattern;
     if( !ptr )
       FatalAllocationError("pattern");
   
@@ -484,6 +489,8 @@ namespace OpenBabel
     ptr->bcount = 0;
   
     ptr->parts = 1;
+
+    ptr->hasExplicitH=false;
     return ptr;
   }
 
@@ -497,10 +504,15 @@ namespace OpenBabel
         size = (int)(pat->aalloc*sizeof(AtomSpec));
         if( pat->atom )
           {
-            pat->atom = (AtomSpec*)realloc(pat->atom,size);
+            //pat->atom = (AtomSpec*)realloc(pat->atom,size);
+	    AtomSpec *tmp = new AtomSpec[pat->aalloc];
+            copy(pat->atom, pat->atom + pat->aalloc - ATOMPOOL, tmp);
+            delete [] pat->atom;
+            pat->atom = tmp;
           }
         else
-          pat->atom = (AtomSpec*)malloc(size);
+          //pat->atom = (AtomSpec*)malloc(size);
+          pat->atom = new AtomSpec[pat->aalloc];
         if( !pat->atom )
           FatalAllocationError("atom pool");
       }
@@ -523,10 +535,15 @@ namespace OpenBabel
         size = (int)(pat->balloc*sizeof(BondSpec));
         if( pat->bond )
           {
-            pat->bond = (BondSpec*)realloc(pat->bond,size);
+            //pat->bond = (BondSpec*)realloc(pat->bond,size);
+	    BondSpec *tmp = new BondSpec[pat->balloc];
+            copy(pat->bond, pat->bond + pat->balloc - BONDPOOL, tmp);
+            delete [] pat->bond;
+            pat->bond = tmp;
           }
         else
-          pat->bond = (BondSpec*)malloc(size);
+          //pat->bond = (BondSpec*)malloc(size);
+          pat->bond = new BondSpec[pat->balloc];
         if( !pat->bond )
           FatalAllocationError("bond pool");
       }
@@ -572,16 +589,25 @@ namespace OpenBabel
           {
             for( i=0; i<pat->acount; i++ )
               FreeAtomExpr(pat->atom[i].expr);
-            free(pat->atom);
+            if (pat->atom != NULL) {
+              //free(pat->atom);
+              delete [] pat->atom;
+              pat->atom = NULL;
+            }
           }
       
         if( pat->balloc )
           {
             for( i=0; i<pat->bcount; i++ )
               FreeBondExpr(pat->bond[i].expr);
-            free(pat->bond);
+            if (pat->bond != NULL) {
+              //free(pat->bond);
+              delete [] pat->bond;
+              pat->bond = NULL;
+            }
           }
-        free(pat);
+        delete pat;
+        pat = NULL;
       }
   }
 
@@ -1527,6 +1553,7 @@ namespace OpenBabel
               {
                 aexpr = GenerateElement(1);
                 LexPtr++; // skip the 'H'
+                pat->hasExplicitH = true;
               }
             else
               aexpr = ParseAtomExpr(0);
@@ -2528,6 +2555,15 @@ namespace OpenBabel
   bool OBSmartsPattern::Match(OBMol &mol,bool single)
   {
     RSCACHE.clear();
+    if(_pat == NULL)
+      return false;
+    if(_pat->hasExplicitH) //The SMARTS pattern contains [H]
+    {
+      //Do matching on a copy of mol with explict hydrogens
+      OBMol tmol = mol;
+      tmol.AddHydrogens(false,false);
+      return(match(tmol,_pat,_mlist,single));
+    }
     return(match(mol,_pat,_mlist,single));
   }
 
@@ -2730,25 +2766,21 @@ namespace OpenBabel
               r2 = j;
               for (k = 0;k < pat->bcount;++k)
                 if (pat->bond[k].dst == r2)
-                  {
-                    if (r1 == -1)
-                      r1 = pat->bond[k].src;
-                    else if (r3 == -1)
-                      r3 = pat->bond[k].src;
-                    else if (r4 == -1)
-                      r4 = pat->bond[k].src;
-                  }
+                  if (r1 == -1)
+                    r1 = pat->bond[k].src;
+                  else if (r3 == -1)
+                    r3 = pat->bond[k].src;
+                  else if (r4 == -1)
+                    r4 = pat->bond[k].src;
 	    
               for (k = 0;k < pat->bcount;++k)
                 if (pat->bond[k].src == r2)
-                  {
-                    if (r1 == -1)
-                      r1 = pat->bond[k].dst;
-                    else if (r3 == -1)
-                      r3 = pat->bond[k].dst;
-                    else if (r4 == -1)
-                      r4 = pat->bond[k].dst;
-                  }
+                  if (r1 == -1)
+                    r1 = pat->bond[k].dst;
+                  else if (r3 == -1)
+                    r3 = pat->bond[k].dst;
+                  else if (r4 == -1)
+                    r4 = pat->bond[k].dst;
 	    
               if (r1 == -1 || r2 == -1 || r3 == -1 || r4 == -1)
                 continue;
@@ -2805,6 +2837,8 @@ namespace OpenBabel
               return(expr->leaf.value == -(atom->GetFormalCharge()));
             case AL_POSITIVE:
               return(expr->leaf.value == atom->GetFormalCharge());
+            case AL_MASS:
+              return(expr->leaf.value == atom->GetIsotope());
             case AL_HYB:
               return(expr->leaf.value == (int)atom->GetHyb());
 	    
