@@ -33,6 +33,8 @@ obtained in part or whole from RasMol2 by Roger Sayle.
 #include <openbabel/oberror.h>
 #include <openbabel/bitvec.h>
 #include <openbabel/bond.h>
+#include <cstring>
+#include <cstdlib>
 
 using namespace std;
 
@@ -354,7 +356,7 @@ namespace OpenBabel
 
   static unsigned int GetResidueNumber(const char *res)
   {
-    if (res != NULL)
+    if (res != NULL && strlen(res) > 2)
       {
         int ch1 = toupper(res[0]);
         int ch2 = toupper(res[1]);
@@ -848,7 +850,7 @@ namespace OpenBabel
     _idx      = 0;
     _aakey    = 0;
     _reskey   = OBResidueIndex::UNK;
-    _resnum   = 0;
+    _resnum   = "";
     _resname  = "";
     _vdata.clear();
 
@@ -949,7 +951,7 @@ namespace OpenBabel
     _idx     = 0;
     _aakey   = 0;
     _reskey  = OBResidueIndex::UNK;
-    _resnum  = 0;
+    _resnum  = "";
     _resname = "";
 
     _atoms.clear();
@@ -981,7 +983,14 @@ namespace OpenBabel
     SetResidueKeys(_resname.c_str(), _reskey, _aakey);
   }
 
-  void OBResidue::SetNum(unsigned int resnum)
+  void OBResidue::SetNum(const unsigned int resnum)
+  {
+    stringstream temp;
+    temp << resnum;
+    _resnum = temp.str();
+  }
+
+  void OBResidue::SetNum(const string resnum)
   {
     _resnum = resnum;
   }
@@ -1050,11 +1059,16 @@ namespace OpenBabel
     return _resname;
   }
 
-  unsigned int OBResidue::GetNum(void) const
+  std::string OBResidue::GetNumString(void)
   {
     return _resnum;
   }
 
+  unsigned int OBResidue::GetNum(void) 
+  {
+    return atoi(_resnum.c_str());
+  }
+
   unsigned int OBResidue::GetNumAtoms(void) const
   {
     return (unsigned int)_atoms.size();
