@@ -16,7 +16,10 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
 #include <openbabel/babelconfig.h>
+#include <sstream>
 #include <openbabel/mol.h>
+#include <openbabel/descriptor.h>
+#include <openbabel/op.h>
 
 using namespace std;
 namespace OpenBabel
@@ -43,10 +46,15 @@ namespace OpenBabel
     if(pOptions->empty())
       return this;
 
-    bool ret=true;
-    bool smatch=true, vmatch=true;
+    // DoOps calls Do() for each of the plugin options in the map
+    // It normally returns true, even if there are no options but
+    // can return false if one of the options decides that the 
+    // molecule should not be output
+    bool fmatch = OBOp::DoOps(this, pOptions); 
 
-    map<string,string>::const_iterator itr;
+    bool ret=true;
+
+    map<string,string>::const_iterator itr, itr2;
 
     if(pOptions->find("b")!=pOptions->end())
       if(!ConvertDativeBonds())
@@ -119,25 +127,67 @@ namespace OpenBabel
           }
       }
 
-    itr = pOptions->find("v");
+    itr = pOptions->find("add");  //adds new properties from descriptors in list
+    if(itr!=pOptions->end())
+      OBDescriptor::AddProperties(this, itr->second);
+    
+    itr = pOptions->find("delete"); //deletes the specified properties
+    if(itr!=pOptions->end())
+      OBDescriptor::DeleteProperties(this, itr->second);
+
+    itr = pOptions->find("append"); //Appends values of descriptors or properties to title
     if(itr!=pOptions->end())
       {
-        //inverse match quoted SMARTS string which follows
-        OBSmartsPattern sp;
-        sp.Init(itr->second);
-        vmatch = !sp.Match(*this); //(*pmol) ;
+        string title(GetTitle());
+        title += OBDescriptor::GetValues(this, itr->second);
+        SetTitle(Trim(title).c_str());
       }
 
-    itr = pOptions->find("s");
+
+    
+      //Filter using OBDescriptor comparison and (older) SMARTS tests
+    //Continue only if previous test was true.
+    itr = pOptions->find("filter");
     if(itr!=pOptions->end())
       {
-        //match quoted SMARTS string which follows
-        OBSmartsPattern sp;
-        sp.Init(itr->second.c_str());
-        smatch = sp.Match(*this); //(*pmol) ;
+        std::istringstream optionText(itr->second);
+        fmatch = OBDescriptor::FilterCompare(this, optionText, false);
       }
 
-    if(!smatch || !vmatch)
+    if(fmatch)
+      {
+        itr = pOptions->find("v");
+        if(itr!=pOptions->end())
+          {
+            //inverse match quoted SMARTS string which follows
+            OBSmartsPattern sp;
+            sp.Init(itr->second);
+            fmatch = !sp.Match(*this); //(*pmol) ;
+          }
+      }
+    if(fmatch)
+    {
+      itr = pOptions->find("s");
+      if(itr!=pOptions->end())
+        {
+          //SMARTS filter
+          //If exactmatch option set (probably in fastsearchformat) the
+          //number of atoms in the pattern (passed as a string in the option text)
+          //has to be the same as in the molecule.
+          itr2 = pOptions->find("exactmatch");
+          if(itr2!=pOptions->end() && NumHvyAtoms()!=atoi(itr2->second.c_str()))
+            fmatch=false;
+          else
+          {
+            //match quoted SMARTS string which follows
+            OBSmartsPattern sp;
+            sp.Init(itr->second.c_str());
+            fmatch = sp.Match(*this); //(*pmol) ;
+          }
+        }
+    }
+
+    if(!fmatch)
       {
         //filter failed: delete OBMol and return NULL
         delete this;
@@ -159,22 +209,33 @@ namespace OpenBabel
   ///////////////////////////////////////////////////
   const char* OBMol::ClassDescription()
   {
-    return "For conversions of molecules\n \
-Additional options :\n \
--d Delete hydrogens (make implicit)\n \
--h Add hydrogens (make explicit)\n \
--p Add Hydrogens appropriate for pH model\n \
--b Convert dative bonds e.g.[N+]([O-])=O to N(=O)=O\n \
--c Center Coordinates\n \
---join Join all input molecules into a single output molecule\n \
---separate Output disconnected fragments separately\n \
--C Combine mols in first file with others having same name\n \
--s\"smarts\" Convert only molecules matching SMARTS:\n \
--v\"smarts\" Convert only molecules NOT matching SMARTS:\n \
---property <attrib> <value> add or replace a property (SDF)\n \
---title <title> Add or replace molecule title\n \
---addtotitle <text> Append to title\n \
---addformula Append formula to title\n\n" ;
+    static string ret;
+    ret = "For conversions of molecules\n"
+"Additional options :\n"
+"-d Delete hydrogens (make implicit)\n"
+"-h Add hydrogens (make explicit)\n"
+"-p Add Hydrogens appropriate for pH model\n"
+"-b Convert dative bonds e.g.[N+]([O-])=O to N(=O)=O\n"
+"-c Center Coordinates\n"
+"-C Combine mols in first file with others having same name\n"
+"--filter <filterstring> Filter: convert only when tests are true:\n"
+"--add <list> Add properties from descriptors:\n"
+"--delete <list> Delete properties in list:\n"
+"--append <list> Appends properties or descriptors in list to title:\n"
+"-s\"smarts\" Convert only molecules matching SMARTS:\n"
+"-v\"smarts\" Convert only molecules NOT matching SMARTS:\n"
+"--join Join all input molecules into a single output molecule\n"
+"--separate Output disconnected fragments separately\n"
+"--property <attrib> <value> add or replace a property (SDF)\n"
+"--title <title> Add or replace molecule title\n"
+"--addtotitle <text> Append to title\n"
+"--addformula Append formula to title\n" ;
+
+    //Append lines from OBOp plugins that work with OBMol
+    OBMol dummymol; //just needed to carry class type information; messy!
+    ret += OBOp::OpOptions(&dummymol);
+
+      return ret.c_str();
   }
 
 } //namespace OpenBabel
