@@ -171,6 +171,27 @@ namespace OpenBabel
           for (j = _mlist.begin();j != _mlist.end();++j)
             mol.GetAtom((*j)[0])->SetType(i->second);
         }
+
+    // Special cases
+    vector<OBAtom*>::iterator a;
+    OBAtom* atom;
+    for (atom = mol.BeginAtom(a); atom; atom = mol.NextAtom(a)) {
+      // guanidinium. Fixes PR#1800964
+      if (strncasecmp(atom->GetType(),"C2", 2) == 0) {
+        int guanidineN = 0;
+        OBAtom *nbr;
+        vector<OBBond*>::iterator k;
+        for (nbr = atom->BeginNbrAtom(k);nbr;nbr = atom->NextNbrAtom(k)) {
+          if (strncasecmp(nbr->GetType(),"Npl", 3) == 0 ||
+              strncasecmp(nbr->GetType(),"N2", 2) == 0 ||
+              strncasecmp(nbr->GetType(),"Ng+", 3) == 0)
+            ++guanidineN;
+        }
+        atom->SetType("C+");
+
+      } // end C2 carbon for guanidinium
+
+    } // end special cases
   }
 
   void OBAtomTyper::AssignHyb(OBMol &mol)
@@ -260,8 +281,118 @@ namespace OpenBabel
       return;
     mol.SetAromaticCorrected();
 
+    FOR_ATOMS_OF_MOL(atom, mol) {
+      if (atom->IsNitrogen() && atom->IsAromatic()) {
+        atom->SetHyb(2);
+        atom->SetType("Nar");
+        if (atom->HasDoubleBond()) {
+          atom->SetImplicitValence(2 + atom->GetFormalCharge());
+        } else {
+          if (atom->GetImplicitValence() == 2)
+          atom->SetImplicitValence(3 + atom->GetFormalCharge());
+        }
+      }
+    }
+
     return;
   }
+  
+  /*! \class OBRingTyper typer.h <openbabel/typer.h>
+    \brief Assigns ring types
+
+    The OBRingTyper class is designed to read in a list of ring typing
+    rules and apply them to molecules. The code that performs ring
+    typing is not usually used directly as ring typing is done
+    automatically when the ring type is requested of rings:
+    \code
+      vector<OBRing*>::iterator i;
+      vector<OBRing*> rlist = mol.GetSSSR();
+      
+      for (i = rlist.begin();i != rlist.end();++i)
+        cout << "ring type = " << (*i)->GetType() << endl;
+    \endcode
+  */
+  OBRingTyper::OBRingTyper()
+  {
+    _init = false;
+    _dir = BABEL_DATADIR;
+    _envvar = "BABEL_DATADIR";
+    _filename = "ringtyp.txt";
+    _subdir = "data";
+    //_dataptr = RingTypeData;
+  }
+
+  void OBRingTyper::ParseLine(const char *buffer)
+  {
+    vector<string> vs;
+    OBSmartsPattern *sp;
+
+    if (EQn(buffer,"RINGTYP",7)) {
+      tokenize(vs,buffer);
+      if (vs.empty() || vs.size() < 3) {
+        obErrorLog.ThrowError(__FUNCTION__, " Could not parse RING line in ring type table from ringtyp.txt", obInfo);
+        return;
+      }
+      sp = new OBSmartsPattern;
+      if (sp->Init(vs[2]))
+        _ringtyp.push_back(pair<OBSmartsPattern*,string> (sp,vs[1]));
+      else {
+        delete sp;
+        sp = NULL;
+        obErrorLog.ThrowError(__FUNCTION__, " Could not parse RING line in ring type table from ringtyp.txt", obInfo);
+        return;
+      }
+    }
+  }
+
+  OBRingTyper::~OBRingTyper()
+  {
+    vector<pair<OBSmartsPattern*,string> >::iterator i;
+    for (i = _ringtyp.begin();i != _ringtyp.end();++i) {
+        delete i->first;
+        i->first = NULL;
+    }
+  }
+
+  void OBRingTyper::AssignTypes(OBMol &mol)
+  {
+    if (!_init)
+      Init();
+
+    obErrorLog.ThrowError(__FUNCTION__,
+                          "Ran OBRing::AssignTypes", obAuditMsg);
+
+    mol.SetRingTypesPerceived();
+
+    vector<vector<int> >::iterator j2;
+    vector<pair<OBSmartsPattern*,string> >::iterator i2;
+
+    vector<OBRing*>::iterator i;
+    vector<int>::iterator j;
+    vector<OBRing*> rlist = mol.GetSSSR();
+
+    int member_count;
+    for (i2 = _ringtyp.begin();i2 != _ringtyp.end();++i2) { // for each ring type
+      if (i2->first->Match(mol)) {
+        _mlist = i2->first->GetMapList();
+        for (j2 = _mlist.begin();j2 != _mlist.end();++j2) { // for each found match
+
+          for (i = rlist.begin();i != rlist.end();++i) { // for each ring
+            member_count = 0;
+            
+            for(j = j2->begin(); j != j2->end(); ++j) { // for each atom in the match
+	      if ((*i)->IsMember(mol.GetAtom(*j))) 
+	        member_count++;
+	    }
+	    
+	    if ((*i)->Size() == member_count)
+              (*i)->SetType(i2->second);
+	  }
+	}
+      }
+    }
+  }
+
 
   /*! \class OBAromaticTyper typer.h <openbabel/typer.h>
     \brief Assigns aromatic typing to atoms and bonds
