@@ -7,16 +7,24 @@
 # wrapper.sh --shuffle --debug
 # wrapper.sh --verbose # display full output of tests while running
 
-TESTS="aromatic.pl atom bond cansmi conversion data format"
-TESTS="${TESTS} formula formalcharge inchi.pl inchi2.pl"
+TESTS="aromatic.pl atom bond cansmi cmlreadfile conversion data"
+TESTS="${TESTS} ffghemical ffmmff94 ffuff"
+TESTS="${TESTS} format formula formalcharge"
 TESTS="${TESTS} internalcoord iterators" 
 TESTS="${TESTS} invalidsmarts invalidsmiles"
 TESTS="${TESTS} logp_psa math"
-TESTS="${TESTS} mol residue rings"
+TESTS="${TESTS} mol phmodel residue rings"
 TESTS="${TESTS} smarts smilesmatch unitcell"
 TESTS="${TESTS} cml.sh test-set.sh"
+if [ "x${srcdir}" != "x" ]; then
+  TESTS="${TESTS} ${srcdir}/inchi.pl ${srcdir}/inchi2.pl"
+else
+  TESTS="${TESTS} inchi.pl inchi2.pl"
+fi
 PROVE=prove
 
+echo "top srcdir: .${topsrcdir}."
+
 unset BABEL_LIBDIR
 unset BABEL_DATADIR
 if [ -d ../src/formats/.libs ]; then
@@ -25,12 +33,15 @@ if [ -d ../src/formats/.libs ]; then
         export BABEL_LIBDIR
     fi
     if [ "x${BABEL_DATADIR}" = "x" ]; then
-	      if [ "x${top_srcdir}" != "x" ]; then
-	          BABEL_DATADIR="${top_srcdir}/data"
+	      if [ "x${srcdir}" != "x" ]; then
+	          BABEL_DATADIR="${srcdir}/../data"
+		  TESTDATADIR="${srcdir}/files"
 	      else
 	          BABEL_DATADIR="`pwd`/../data"
+	          BABEL_DATADIR="`pwd`/files"
 	      fi
 	      export BABEL_DATADIR
+	      export TESTDATADIR
     fi
 fi
 
