@@ -34,13 +34,14 @@ GNU General Public License for more details.
 #if HAVE_CONIO_H
 	#include <conio.h>
 #endif
-#include <cstdlib>
 
 #if !HAVE_STRNCASECMP
 extern "C" int strncasecmp(const char *s1, const char *s2, size_t n);
 #endif
 
 #include <openbabel/obconversion.h>
+#include <openbabel/plugin.h>
+#include <cstdlib>
 
 using namespace std;
 using namespace OpenBabel;
@@ -66,8 +67,8 @@ int main(int argc,char *argv[])
   bool gotInType = false, gotOutType = false;
   bool SplitOrBatch=false;
 
-  char *oext;
-  char *iext;
+  char *oext = NULL;
+  char *iext = NULL;
 
   //Save name of program without its path (and .exe)
   string pn(argv[0]);
@@ -147,7 +148,7 @@ int main(int argc,char *argv[])
                     }
                   break;
 				
-                case 'F':
+                /*case 'F':
                   if(!Conv.SetOutFormat("fpt"))
                     cout << "FingerprintFormat needs to be loaded" << endl;
                   else
@@ -156,44 +157,45 @@ int main(int argc,char *argv[])
                       Conv.Write(NULL);
                     }
                   return 0;
-				
+				        */
+                case 'L': //display a list of plugin type or classes
+                  {
+                    const char* param=NULL;
+                    if(argc>arg+1)
+                      param = argv[arg+2];
+                    OBPlugin::List(argv[arg+1], param);
+                    return 0;
+                  }
                 case '?':
                 case 'H':
-                  if(isalnum(argv[arg][2]))
+                  if(isalnum(argv[arg][2]) || arg==argc-2)
                     {
                       if(strncasecmp(argv[arg]+2,"all",3))
                         {
-                          OBFormat* pFormat = Conv.FindFormat(argv[arg]+2);
+                          const char* pID= (arg==argc-2) ? argv[arg+1] : argv[arg]+2;
+                          OBFormat* pFormat = Conv.FindFormat(pID);
                           if(pFormat)
                             {
-                              cout << argv[arg]+2 << "  " << pFormat->Description() << endl;
+                              cout << pID << "  " << pFormat->Description() << endl;
+                              if(pFormat->Flags() & NOTWRITABLE)
+                                cout << " This format is Read-only" << endl;
+                              if(pFormat->Flags() & NOTREADABLE)
+                                cout << " This format is Write-only" << endl;
+
                               if(strlen(pFormat->SpecificationURL()))
                                 cout << "Specification at: " << pFormat->SpecificationURL() << endl;
                             }
                           else
-                            cout << "Format type: " << argv[arg]+2 << " was not recognized" <<endl;
+                            cout << "Format type: " << pID << " was not recognized" <<endl;
                         }
                       else
                         {
-                          Formatpos pos;
-                          OBFormat* pFormat;
-                          const char* str=NULL;
-                          while(OBConversion::GetNextFormat(pos,str,pFormat))
-                            {
-                              if((pFormat->Flags() & NOTWRITABLE) && (pFormat->Flags() & NOTREADABLE))
-                                continue;
-                              cout << str << endl;
-                              const char* p = strchr(pFormat->Description(),'\n');
-                              cout << p+1; //second line of description
-                              if(strlen(pFormat->SpecificationURL()))
-                                cout << "Specification at: " << pFormat->SpecificationURL();
-                              cout << endl << endl;
-                            }
+                          OBPlugin::List("formats","verbose");
                         }
                     }
                   else
                     help();
-                  exit(0);
+                  return 0;
 					
                 case '-': //long option --name text
                   {
@@ -300,15 +302,25 @@ int main(int argc,char *argv[])
         }
     }
   
-  Conv.SetInAndOutFormats(pInFormat,pOutFormat);
-  
+    if(!Conv.SetInFormat(pInFormat))
+    {
+      cerr << "Invalid input format" << endl;
+      usage();
+    }
+    if(!Conv.SetOutFormat(pOutFormat))
+    {
+      cerr << "Invalid output format" << endl;
+      usage();
+    }
+
   if(SplitOrBatch)
     {
       //Put * into output file name before extension (or ext.gz)
       if(OutputFileName.empty())
         {
           OutputFileName = "*.";
-          OutputFileName += oext;
+          if (oext != NULL)
+            OutputFileName += oext;
         }
       else
         {
@@ -326,17 +338,8 @@ int main(int argc,char *argv[])
 
   int count = Conv.FullConvert(FileList, OutputFileName, OutputFileList);
  
-  // send info message to clog -- don't mess up cerr or cout for user programs
-  //Get the last word on the first line of the description which should
-  //be "molecules", "reactions", etc and remove the s if only one object converted
-  std::string objectname(pOutFormat->TargetClassDescription());
-  pos = objectname.find('\n');
-  if(count==1) --pos;
-  objectname.erase(pos);
-  pos = objectname.rfind(' ');
-  if(pos==std::string::npos)
-    pos=0;
-  std::clog << count << objectname.substr(pos) << " converted" << endl;
+  Conv.ReportNumberConverted(count);
+
   if(OutputFileList.size()>1)
     {
       clog << OutputFileList.size() << " files output. The first is " << OutputFileList[0] <<endl;
@@ -419,19 +422,20 @@ void help()
   cout << "More than one input file can be specified and their names can contain" <<endl;
   cout << "wildcard chars (* and ?).The molecules are aggregated in the output file.\n" << endl;
   cout << OBConversion::Description(); // Conversion options
-  cout << "  -H Outputs this help text" << endl;
-  cout << "  -Hxxx (xxx is file format ID e.g. -Hcml) gives format info" <<endl; 
-  cout << "  -Hall Outputs details of all formats" <<endl; 
-  cout << "  -V Outputs version number" <<endl; 
-  cout << "  -F Outputs the available fingerprint types" <<endl; 
-  cout << "  -m Produces multiple output files, to allow:" <<endl;
-  cout << "     Splitting: e.g.        " << program_name << " infile.mol new.smi -m" <<endl;
-  cout << "       puts each molecule into new1.smi new2.smi etc" <<endl;
-  cout << "     Batch conversion: e.g. " << program_name << " *.mol -osmi -m" <<endl;
-  cout << "       converts each input file to a .smi file" << endl;
+  cout << "-H Outputs this help text" << endl;
+  cout << "-Hxxx (xxx is file format ID e.g. -Hcml) gives format info" <<endl; 
+  cout << "-Hall Outputs details of all formats" <<endl; 
+  cout << "-V Outputs version number" <<endl; 
+  cout << "-L <BaseType> Lists plugin classes of this type" << endl;
+  cout << "    e.g. <fingerprints>, or <plugins> for a list of BaseTypes" << endl; 
+  cout << "-m Produces multiple output files, to allow:" <<endl;
+  cout << "   Splitting: e.g.        " << program_name << " infile.mol new.smi -m" <<endl;
+  cout << "     puts each molecule into new1.smi new2.smi etc" <<endl;
+  cout << "   Batch conversion: e.g. " << program_name << " *.mol -osmi -m" <<endl;
+  cout << "     converts each input file to a .smi file" << endl;
 #ifdef _WIN32
-  cout << "     In Windows these can also be done using the forms" <<endl;
-  cout << "       " << program_name << " infile.mol new*.smi and " << program_name << " *.mol *.smi respectively.\n" <<endl;
+  cout << "   In Windows these can also be done using the forms" <<endl;
+  cout << "     " << program_name << " infile.mol new*.smi and " << program_name << " *.mol *.smi respectively.\n" <<endl;
 #endif
   
   OBFormat* pDefault = OBConversion::GetDefaultFormat();
@@ -443,15 +447,7 @@ void help()
     cout << pAPI->Description();
   
   cout << "The following file formats are recognized:" << endl;
-  Formatpos pos;
-  OBFormat* pFormat;
-  const char* str=NULL;
-  while(OBConversion::GetNextFormat(pos,str,pFormat))
-    {
-      if((pFormat->Flags() & NOTWRITABLE) && (pFormat->Flags() & NOTREADABLE))
-	continue;
-      cout << "  " << str << endl;
-    }
+  OBPlugin::List("formats");
   cout << "\nSee further specific info and options using -H<format-type>, e.g. -Hcml" << endl;
 }
 
