@@ -26,9 +26,11 @@ GNU General Public License for more details.
 #include <openbabel/babelconfig.h>
 
 #include <openbabel/mol.h>
-#include <openbabel/groupcontrib.h>
+#include <openbabel/descriptor.h>
 #include <openbabel/obconversion.h>
-#include <unistd.h>
+#ifndef _MSC_VER
+  #include <unistd.h>
+#endif
 
 using namespace std;
 using namespace OpenBabel;
@@ -66,9 +68,11 @@ int main(int argc,char **argv)
         "sequence RESIDUE_SEQUENCE\n"
         "num_rings NUMBER_OF_RING_(SSSR)\n"
         "logP   NUM\n"
-        "PSA    POLAR_SURFACE_AREA\n";
+        "PSA    POLAR_SURFACE_AREA\n"
+        "MR     MOLAR REFRACTIVITY";
       err += "$$$$";
-      ThrowError(err);
+//      ThrowError(err); wasn't being output because error level too low
+      cerr << err; //Why not do directly
       exit(-1);
     }
   else
@@ -97,9 +101,6 @@ int main(int argc,char **argv)
     }
   
   OBMol mol;
-  OBLogP logP;
-  OBPSA psa;
-  OBMR mr;
   OBFormat *canSMIFormat = conv.FindFormat("can");
   OBFormat *inchiFormat = conv.FindFormat("inchi");
 
@@ -131,19 +132,27 @@ int main(int argc,char **argv)
       cout << "mol_weight       " << mol.GetMolWt() << endl;
       cout << "exact_mass       " << mol.GetExactMass() << endl;
 
+      string smilesString = "-";
       if (canSMIFormat) {
         conv.SetOutFormat(canSMIFormat);
-        cout << "canonical_SMILES " << conv.WriteString(&mol);
+        smilesString = conv.WriteString(&mol);
+        if ( smilesString.length() == 0 )
+        {
+          smilesString = "-";
+        }
       }
-      else
-        cout << "canonical_SMILES " << "-" << endl;
+      cout << "canonical_SMILES " << smilesString << endl;
 
+      string inchiString = "-";
       if (inchiFormat) {
         conv.SetOutFormat(inchiFormat);
-        cout << "InChI            " << conv.WriteString(&mol);
+        inchiString = conv.WriteString(&mol);
+        if ( inchiString.length() == 0 )
+        {
+          inchiString = "-";
+        }
       }
-      else
-        cout << "InChI            " << "-" << endl;
+      cout << "InChI            " << inchiString << endl;
 
       cout << "num_atoms        " << mol.NumAtoms() << endl;
       cout << "num_bonds        " << mol.NumBonds() << endl;
@@ -154,11 +163,25 @@ int main(int argc,char **argv)
         cout << "sequence         " << "-" << endl;
 
       cout << "num_rings        " << nrings(mol) << endl;
-      cout << "logP             " << logP.Predict(mol) << endl;
-      cout << "PSA              " << psa.Predict(mol) << endl;
-      cout << "MR               " << mr.Predict(mol) << endl;
+
+      OBDescriptor* pDesc;
+      pDesc= OBDescriptor::FindType("logP");
+      if(pDesc)
+        cout << "logP             " << pDesc->Predict(&mol) << endl;
+
+      pDesc = OBDescriptor::FindType("TPSA");
+      if(pDesc)
+        cout << "PSA              " << pDesc->Predict(&mol) << endl;
+
+      pDesc = OBDescriptor::FindType("MR");
+      if(pDesc)
+        cout << "MR               " << pDesc->Predict(&mol) << endl;
+
       cout << "$$$$" << endl; // SDF like end of compound descriptor list
       
+      //Other OBDescriptors could be output here, even ones that were rarely
+      // used. Since these are plugin classes, they may not be loaded, but
+      // then with code like the above they are just ignored.
     } // end for loop
   
   return(1);
@@ -202,7 +225,7 @@ string sequence(OBMol &mol)
   if (residueSequence.size() != 0) // remove the trailing "-"
     residueSequence.erase(residueSequence.size() - 1);
 
-  return residueSequence;
+  return residueSequence; 
 }
 
 /* obprop man page*/
