@@ -40,7 +40,7 @@ int main(int argc,char *argv[])
 {
   if (argc != 2)
     {
-      cout << "Usage: rotamer <file>" << endl;
+      cout << "Usage: obrotamer <file>" << endl;
       cout << "  Outputs the number of rotable bonds and generate a random"
            << " rotamer" << endl;
       return(-1);
@@ -73,7 +73,7 @@ int main(int argc,char *argv[])
   
   OBRotorList rl;
   OBRotamerList rotamers;
-  // int *rotorKey = NULL;
+  int *rotorKey = NULL;
   OBRandom rand;
   rand.TimeSeed();
 
